@@ -1,4 +1,5 @@
-from sympy import Symbol, var, Function, FunctionClass
+from sympy.core.function import (Function, FunctionClass)
+from sympy.core.symbol import (Symbol, var)
 from sympy.testing.pytest import raises
 
 def test_var():
