@@ -1,4 +1,9 @@
-from sympy import Basic, Expr, sympify, S
+from sympy.core.basic import Basic
+from sympy.core.expr import Expr, ExprBuilder
+from sympy.core.singleton import S
+from sympy.core.sorting import default_sort_key
+from sympy.core.symbol import uniquely_named_symbol
+from sympy.core.sympify import sympify
 from sympy.matrices.matrices import MatrixBase
 from sympy.matrices.common import NonSquareMatrixError
 
@@ -32,7 +37,7 @@ class Trace(Expr):
         if not mat.is_Matrix:
             raise TypeError("input to Trace, %s, is not a matrix" % str(mat))
 
-        if not mat.is_square:
+        if mat.is_square is False:
             raise NonSquareMatrixError("Trace of a non-square matrix")
 
         return Basic.__new__(cls, mat)
@@ -41,7 +46,7 @@ class Trace(Expr):
         return self
 
     def _eval_derivative(self, v):
-        from sympy import Sum
+        from sympy.concrete.summations import Sum
         from .matexpr import MatrixElement
         if isinstance(v, MatrixElement):
             return self.rewrite(Sum).diff(v)
@@ -52,16 +57,15 @@ class Trace(Expr):
         return expr._eval_derivative(v)
 
     def _eval_derivative_matrix_lines(self, x):
-        from sympy.codegen.array_utils import CodegenArrayContraction, CodegenArrayTensorProduct
-        from sympy.core.expr import ExprBuilder
+        from sympy.tensor.array.expressions.array_expressions import ArrayTensorProduct, ArrayContraction
         r = self.args[0]._eval_derivative_matrix_lines(x)
         for lr in r:
             if lr.higher == 1:
                 lr.higher = ExprBuilder(
-                    CodegenArrayContraction,
+                    ArrayContraction,
                     [
                         ExprBuilder(
-                            CodegenArrayTensorProduct,
+                            ArrayTensorProduct,
                             [
                                 lr._lines[0],
                                 lr._lines[1],
@@ -69,15 +73,15 @@ class Trace(Expr):
                         ),
                         (1, 3),
                     ],
-                    validator=CodegenArrayContraction._validate
+                    validator=ArrayContraction._validate
                 )
             else:
                 # This is not a matrix line:
                 lr.higher = ExprBuilder(
-                    CodegenArrayContraction,
+                    ArrayContraction,
                     [
                         ExprBuilder(
-                            CodegenArrayTensorProduct,
+                            ArrayTensorProduct,
                             [
                                 lr._lines[0],
                                 lr._lines[1],
@@ -98,9 +102,9 @@ class Trace(Expr):
     def arg(self):
         return self.args[0]
 
-    def doit(self, **kwargs):
-        if kwargs.get('deep', True):
-            arg = self.arg.doit(**kwargs)
+    def doit(self, **hints):
+        if hints.get('deep', True):
+            arg = self.arg.doit(**hints)
             try:
                 return arg._eval_trace()
             except (AttributeError, NotImplementedError):
@@ -112,10 +116,37 @@ class Trace(Expr):
             else:
                 return Trace(self.arg)
 
+    def as_explicit(self):
+        return Trace(self.arg.as_explicit()).doit()
+
+    def _normalize(self):
+        # Normalization of trace of matrix products. Use transposition and
+        # cyclic properties of traces to make sure the arguments of the matrix
+        # product are sorted and the first argument is not a transposition.
+        from sympy.matrices.expressions.matmul import MatMul
+        from sympy.matrices.expressions.transpose import Transpose
+        trace_arg = self.arg
+        if isinstance(trace_arg, MatMul):
+
+            def get_arg_key(x):
+                a = trace_arg.args[x]
+                if isinstance(a, Transpose):
+                    a = a.arg
+                return default_sort_key(a)
+
+            indmin = min(range(len(trace_arg.args)), key=get_arg_key)
+            if isinstance(trace_arg.args[indmin], Transpose):
+                trace_arg = Transpose(trace_arg).doit()
+                indmin = min(range(len(trace_arg.args)), key=lambda x: default_sort_key(trace_arg.args[x]))
+            trace_arg = MatMul.fromiter(trace_arg.args[indmin:] + trace_arg.args[:indmin])
+            return Trace(trace_arg)
+        return self
+
     def _eval_rewrite_as_Sum(self, expr, **kwargs):
-        from sympy import Sum, Dummy
-        i = Dummy('i')
-        return Sum(self.arg[i, i], (i, 0, self.arg.rows-1)).doit()
+        from sympy.concrete.summations import Sum
+        i = uniquely_named_symbol('i', expr)
+        s = Sum(self.arg[i, i], (i, 0, self.arg.rows - 1))
+        return s.doit()
 
 
 def trace(expr):
