@@ -1,12 +1,16 @@
 """sympify -- convert objects SymPy internal format"""
 
-from __future__ import print_function, division
+from __future__ import annotations
+from typing import Any, Callable
 
 from inspect import getmro
+import string
+from sympy.core.random import choice
 
-from .core import all_classes as sympy_classes
-from .compatibility import iterable, string_types, range
-from .evaluate import global_evaluate
+from .parameters import global_parameters
+
+from sympy.utilities.exceptions import sympy_deprecation_warning
+from sympy.utilities.iterables import iterable
 
 
 class SympifyError(ValueError):
@@ -22,16 +26,24 @@ class SympifyError(ValueError):
             "raised:\n%s: %s" % (self.expr, self.base_exc.__class__.__name__,
             str(self.base_exc)))
 
-converter = {}  # See sympify docstring.
 
-class CantSympify(object):
+converter: dict[type[Any], Callable[[Any], Basic]] = {}
+
+#holds the conversions defined in SymPy itself, i.e. non-user defined conversions
+_sympy_converter: dict[type[Any], Callable[[Any], Basic]] = {}
+
+#alias for clearer use in the library
+_external_converter = converter
+
+class CantSympify:
     """
     Mix in this trait to a class to disallow sympification of its instances.
 
     Examples
     ========
 
-    >>> from sympy.core.sympify import sympify, CantSympify
+    >>> from sympy import sympify
+    >>> from sympy.core.sympify import CantSympify
 
     >>> class Something(dict):
     ...     pass
@@ -48,7 +60,18 @@ class CantSympify(object):
     SympifyError: SympifyError: {}
 
     """
-    pass
+
+    __slots__ = ()
+
+
+def _is_numpy_instance(a):
+    """
+    Checks if an object is an instance of a type from the numpy module.
+    """
+    # This check avoids unnecessarily importing NumPy.  We check the whole
+    # __mro__ in case any base type is a numpy type.
+    return any(type_.__module__ == 'numpy'
+               for type_ in type(a).__mro__)
 
 
 def _convert_numpy_types(a, **sympify_args):
@@ -58,12 +81,12 @@ def _convert_numpy_types(a, **sympify_args):
     import numpy as np
     if not isinstance(a, np.floating):
         if np.iscomplex(a):
-            return converter[complex](a.item())
+            return _sympy_converter[complex](a.item())
         else:
             return sympify(a.item(), **sympify_args)
     else:
         try:
-            from sympy.core.numbers import Float
+            from .numbers import Float
             prec = np.finfo(a).nmant + 1
             # E.g. double precision means prec=53 but nmant=52
             # Leading bit of mantissa is always 1, so is not stored
@@ -77,19 +100,16 @@ def _convert_numpy_types(a, **sympify_args):
 
 def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
         evaluate=None):
-    """Converts an arbitrary expression to a type that can be used inside SymPy.
+    """
+    Converts an arbitrary expression to a type that can be used inside SymPy.
 
-    For example, it will convert Python ints into instances of sympy.Integer,
-    floats into instances of sympy.Float, etc. It is also able to coerce symbolic
-    expressions which inherit from Basic. This can be useful in cooperation
-    with SAGE.
+    Explanation
+    ===========
 
-    It currently accepts as arguments:
-       - any object defined in SymPy
-       - standard numeric python types: int, long, float, Decimal
-       - strings (like "0.09" or "2e-19")
-       - booleans, including ``None`` (will leave ``None`` unchanged)
-       - dict, lists, sets or tuples containing any of the above
+    It will convert Python ints into instances of :class:`~.Integer`, floats
+    into instances of :class:`~.Float`, etc. It is also able to coerce
+    symbolic expressions which inherit from :class:`~.Basic`. This can be
+    useful in cooperation with SAGE.
 
     .. warning::
         Note that this function uses ``eval``, and thus shouldn't be used on
@@ -99,6 +119,9 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
     nothing but return that value. This can be used at the beginning of a
     function to ensure you are working with the correct type.
 
+    Examples
+    ========
+
     >>> from sympy import sympify
 
     >>> sympify(2).is_integer
@@ -118,7 +141,7 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
     >>> sympify("x***2")
     Traceback (most recent call last):
     ...
-    SympifyError: SympifyError: "could not parse u'x***2'"
+    SympifyError: SympifyError: "could not parse 'x***2'"
 
     Locals
     ------
@@ -127,7 +150,7 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
     by ``from sympy import *``; anything used in a string that is not
     defined by that import will be converted to a symbol. In the following,
     the ``bitcount`` function is treated as a symbol and the ``O`` is
-    interpreted as the Order object (used with series) and it raises
+    interpreted as the :class:`~.Order` object (used with series) and it raises
     an error when used improperly:
 
     >>> s = 'bitcount(42)'
@@ -143,9 +166,8 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
     In order to have ``bitcount`` be recognized it can be imported into a
     namespace dictionary and passed as locals:
 
-    >>> from sympy.core.compatibility import exec_
     >>> ns = {}
-    >>> exec_('from sympy.core.evalf import bitcount', ns)
+    >>> exec('from sympy.core.evalf import bitcount', ns)
     >>> sympify(s, locals=ns)
     6
 
@@ -155,20 +177,20 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
 
     >>> from sympy import Symbol
     >>> ns["O"] = Symbol("O")  # method 1
-    >>> exec_('from sympy.abc import O', ns)  # method 2
+    >>> exec('from sympy.abc import O', ns)  # method 2
     >>> ns.update(dict(O=Symbol("O")))  # method 3
     >>> sympify("O + 1", locals=ns)
     O + 1
 
     If you want *all* single-letter and Greek-letter variables to be symbols
     then you can use the clashing-symbols dictionaries that have been defined
-    there as private variables: _clash1 (single-letter variables), _clash2
-    (the multi-letter Greek names) or _clash (both single and multi-letter
-    names that are defined in abc).
+    there as private variables: ``_clash1`` (single-letter variables),
+    ``_clash2`` (the multi-letter Greek names) or ``_clash`` (both single and
+    multi-letter names that are defined in ``abc``).
 
     >>> from sympy.abc import _clash1
-    >>> _clash1
-    {'C': C, 'E': E, 'I': I, 'N': N, 'O': O, 'Q': Q, 'S': S}
+    >>> set(_clash1)  # if this fails, see issue #23903
+    {'E', 'I', 'N', 'O', 'Q', 'S'}
     >>> sympify('I & Q', _clash1)
     I & Q
 
@@ -186,6 +208,13 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
     ...
     SympifyError: SympifyError: None
 
+    .. deprecated:: 1.6
+
+       ``sympify(obj)`` automatically falls back to ``str(obj)`` when all
+       other conversion methods fail, but this is deprecated. ``strict=True``
+       will disable this deprecated behavior. See
+       :ref:`deprecated-sympify-string-fallback`.
+
     Evaluation
     ----------
 
@@ -194,12 +223,21 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
     ``evaluate=False`` option will be added. Nested ``Add`` or ``Mul`` will
     be denested first. This is done via an AST transformation that replaces
     operators with their SymPy equivalents, so if an operand redefines any
-    of those operations, the redefined operators will not be used.
+    of those operations, the redefined operators will not be used. If
+    argument a is not a string, the mathematical expression is evaluated
+    before being passed to sympify, so adding ``evaluate=False`` will still
+    return the evaluated result of expression.
 
     >>> sympify('2**2 / 3 + 5')
     19/3
     >>> sympify('2**2 / 3 + 5', evaluate=False)
     2**2/3 + 5
+    >>> sympify('4/2+7', evaluate=True)
+    9
+    >>> sympify('4/2+7', evaluate=False)
+    4/2 + 7
+    >>> sympify(4/2+7, evaluate=False)
+    9.00000000000000
 
     Extending
     ---------
@@ -245,11 +283,27 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
     The keywords ``rational`` and ``convert_xor`` are only used
     when the input is a string.
 
+    convert_xor
+    -----------
+
+    >>> sympify('x^y',convert_xor=True)
+    x**y
+    >>> sympify('x^y',convert_xor=False)
+    x ^ y
+
+    rational
+    --------
+
+    >>> sympify('0.1',rational=False)
+    0.1
+    >>> sympify('0.1',rational=True)
+    1/10
+
     Sometimes autosimplification during sympification results in expressions
     that are very different in structure than what was entered. Until such
     autosimplification is no longer done, the ``kernS`` function might be of
     some use. In the example below you can see how an expression reduces to
-    -1 by autosimplification, but does not do so when ``kernS`` is used.
+    $-1$ by autosimplification, but does not do so when ``kernS`` is used.
 
     >>> from sympy.core.sympify import kernS
     >>> from sympy.abc import x
@@ -261,25 +315,72 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
     >>> kernS(s)
     -2*(-(-x + 1/x)/(x*(x - 1/x)**2) - 1/(x*(x - 1/x))) - 1
 
+    Parameters
+    ==========
+
+    a :
+        - any object defined in SymPy
+        - standard numeric Python types: ``int``, ``long``, ``float``, ``Decimal``
+        - strings (like ``"0.09"``, ``"2e-19"`` or ``'sin(x)'``)
+        - booleans, including ``None`` (will leave ``None`` unchanged)
+        - dicts, lists, sets or tuples containing any of the above
+
+    convert_xor : bool, optional
+        If true, treats ``^`` as exponentiation.
+        If False, treats ``^`` as XOR itself.
+        Used only when input is a string.
+
+    locals : any object defined in SymPy, optional
+        In order to have strings be recognized it can be imported
+        into a namespace dictionary and passed as locals.
+
+    strict : bool, optional
+        If the option strict is set to ``True``, only the types for which
+        an explicit conversion has been defined are converted. In the
+        other cases, a SympifyError is raised.
+
+    rational : bool, optional
+        If ``True``, converts floats into :class:`~.Rational`.
+        If ``False``, it lets floats remain as it is.
+        Used only when input is a string.
+
+    evaluate : bool, optional
+        If False, then arithmetic and operators will be converted into
+        their SymPy equivalents. If True the expression will be evaluated
+        and the result will be returned.
+
     """
+    # XXX: If a is a Basic subclass rather than instance (e.g. sin rather than
+    # sin(x)) then a.__sympy__ will be the property. Only on the instance will
+    # a.__sympy__ give the *value* of the property (True). Since sympify(sin)
+    # was used for a long time we allow it to pass. However if strict=True as
+    # is the case in internal calls to _sympify then we only allow
+    # is_sympy=True.
+    #
+    # https://github.com/sympy/sympy/issues/20124
     is_sympy = getattr(a, '__sympy__', None)
-    if is_sympy is not None:
+    if is_sympy is True:
         return a
+    elif is_sympy is not None:
+        if not strict:
+            return a
+        else:
+            raise SympifyError(a)
 
     if isinstance(a, CantSympify):
         raise SympifyError(a)
-    cls = getattr(a, "__class__", None)
-    if cls is None:
-        cls = type(a)  # Probably an old-style class
-    conv = converter.get(cls, None)
-    if conv is not None:
-        return conv(a)
 
+    cls = getattr(a, "__class__", None)
+
+    #Check if there exists a converter for any of the types in the mro
     for superclass in getmro(cls):
-        try:
-            return converter[superclass](a)
-        except KeyError:
-            continue
+        #First check for user defined converters
+        conv = _external_converter.get(superclass)
+        if conv is None:
+            #if none exists, check for SymPy defined converters
+            conv = _sympy_converter.get(superclass)
+        if conv is not None:
+            return conv(a)
 
     if cls is type(None):
         if strict:
@@ -288,14 +389,10 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
             return a
 
     if evaluate is None:
-        if global_evaluate[0] is False:
-            evaluate = global_evaluate[0]
-        else:
-            evaluate = True
+        evaluate = global_parameters.evaluate
 
     # Support for basic numpy datatypes
-    # Note that this check exists to avoid importing NumPy when not necessary
-    if type(a).__module__ == 'numpy':
+    if _is_numpy_instance(a):
         import numpy as np
         if np.isscalar(a):
             return _convert_numpy_types(a, locals=locals,
@@ -306,7 +403,7 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
     if _sympy_ is not None:
         try:
             return a._sympy_()
-        # XXX: Catches AttributeError: 'SympyConverter' object has no
+        # XXX: Catches AttributeError: 'SymPyConverter' object has no
         # attribute 'tuple'
         # This is probably a bug somewhere but for now we catch it here.
         except AttributeError:
@@ -319,22 +416,34 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
         if flat is not None:
             shape = getattr(a, "shape", None)
             if shape is not None:
-                from ..tensor.array import Array
+                from sympy.tensor.array import Array
                 return Array(a.flat, a.shape)  # works with e.g. NumPy arrays
 
-    if not isinstance(a, string_types):
-        for coerce in (float, int):
-            try:
-                coerced = coerce(a)
-            except (TypeError, ValueError):
-                continue
-            # XXX: AttributeError only needed here for Py2
-            except AttributeError:
-                continue
-            try:
-                return sympify(coerced)
-            except SympifyError:
-                continue
+    if not isinstance(a, str):
+        if _is_numpy_instance(a):
+            import numpy as np
+            assert not isinstance(a, np.number)
+            if isinstance(a, np.ndarray):
+                # Scalar arrays (those with zero dimensions) have sympify
+                # called on the scalar element.
+                if a.ndim == 0:
+                    try:
+                        return sympify(a.item(),
+                                       locals=locals,
+                                       convert_xor=convert_xor,
+                                       strict=strict,
+                                       rational=rational,
+                                       evaluate=evaluate)
+                    except SympifyError:
+                        pass
+        else:
+            # float and int can coerce size-one numpy arrays to their lone
+            # element.  See issue https://github.com/numpy/numpy/issues/10404.
+            for coerce in (float, int):
+                try:
+                    return sympify(coerce(a))
+                except (TypeError, ValueError, AttributeError, SympifyError):
+                    continue
 
     if strict:
         raise SympifyError(a)
@@ -342,31 +451,32 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
     if iterable(a):
         try:
             return type(a)([sympify(x, locals=locals, convert_xor=convert_xor,
-                rational=rational) for x in a])
-        except TypeError:
-            # Not all iterables are rebuildable with their type.
-            pass
-    if isinstance(a, dict):
-        try:
-            return type(a)([sympify(x, locals=locals, convert_xor=convert_xor,
-                rational=rational) for x in a.items()])
+                rational=rational, evaluate=evaluate) for x in a])
         except TypeError:
             # Not all iterables are rebuildable with their type.
             pass
 
-    # At this point we were given an arbitrary expression
-    # which does not inherit from Basic and doesn't implement
-    # _sympy_ (which is a canonical and robust way to convert
-    # anything to SymPy expression).
-    #
-    # As a last chance, we try to take "a"'s normal form via unicode()
-    # and try to parse it. If it fails, then we have no luck and
-    # return an exception
-    try:
-        from .compatibility import unicode
-        a = unicode(a)
-    except Exception as exc:
-        raise SympifyError(a, exc)
+    if not isinstance(a, str):
+        try:
+            a = str(a)
+        except Exception as exc:
+            raise SympifyError(a, exc)
+        sympy_deprecation_warning(
+            f"""
+The string fallback in sympify() is deprecated.
+
+To explicitly convert the string form of an object, use
+sympify(str(obj)). To add define sympify behavior on custom
+objects, use sympy.core.sympify.converter or define obj._sympy_
+(see the sympify() docstring).
+
+sympify() performed the string fallback resulting in the following string:
+
+{a!r}
+            """,
+            deprecated_since_version='1.6',
+            active_deprecations_target="deprecated-sympify-string-fallback",
+        )
 
     from sympy.parsing.sympy_parser import (parse_expr, TokenError,
                                             standard_transformations)
@@ -391,10 +501,10 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
 
 def _sympify(a):
     """
-    Short version of sympify for internal usage for __add__ and __eq__ methods
-    where it is ok to allow some things (like Python integers and floats) in
-    the expression. This excludes things (like strings) that are unwise to
-    allow into such an expression.
+    Short version of :func:`~.sympify` for internal usage for ``__add__`` and
+    ``__eq__`` methods where it is ok to allow some things (like Python
+    integers and floats) in the expression. This excludes things (like strings)
+    that are unwise to allow into such an expression.
 
     >>> from sympy import Integer
     >>> Integer(1) == 1
@@ -420,14 +530,14 @@ def _sympify(a):
 
 def kernS(s):
     """Use a hack to try keep autosimplification from distributing a
-    a number into an Add; this modification doesn't
+    a number into an Add; this modification does not
     prevent the 2-arg Mul from becoming an Add, however.
 
     Examples
     ========
 
     >>> from sympy.core.sympify import kernS
-    >>> from sympy.abc import x, y, z
+    >>> from sympy.abc import x, y
 
     The 2-arg Mul distributes a number (or minus sign) across the terms
     of an expression, but kernS will prevent that:
@@ -444,9 +554,6 @@ def kernS(s):
 
     XXX This hack should not be necessary once issue 4596 has been resolved.
     """
-    import string
-    from random import choice
-    from sympy.core.symbol import Symbol
     hit = False
     quoted = '"' in s or "'" in s
     if '(' in s and not quoted:
@@ -492,13 +599,15 @@ def kernS(s):
             while kern in s:
                 kern += choice(string.ascii_letters + string.digits)
             s = s.replace(' ', kern)
-        hit = kern in s
+            hit = kern in s
+        else:
+            hit = False
 
     for i in range(2):
         try:
             expr = sympify(s)
             break
-        except:  # the kern might cause unknown errors, so use bare except
+        except TypeError:  # the kern might cause unknown errors...
             if hit:
                 s = olds  # maybe it didn't like the kern; use un-kerned s
                 hit = False
@@ -508,6 +617,7 @@ def kernS(s):
     if not hit:
         return expr
 
+    from .symbol import Symbol
     rep = {Symbol(kern): 1}
     def _clear(expr):
         if isinstance(expr, (list, tuple, set)):
@@ -518,3 +628,7 @@ def kernS(s):
     expr = _clear(expr)
     # hope that kern is not there anymore
     return expr
+
+
+# Avoid circular import
+from .basic import Basic
