@@ -6,32 +6,36 @@ Contains
 
 """
 
-from sympy import Expr, Eq
+from sympy.core.expr import Expr
+from sympy.core.relational import Eq
 from sympy.core import S, pi, sympify
+from sympy.core.evalf import N
 from sympy.core.parameters import global_parameters
 from sympy.core.logic import fuzzy_bool
 from sympy.core.numbers import Rational, oo
-from sympy.core.compatibility import ordered
+from sympy.core.sorting import ordered
 from sympy.core.symbol import Dummy, uniquely_named_symbol, _symbol
 from sympy.simplify import simplify, trigsimp
 from sympy.functions.elementary.miscellaneous import sqrt, Max
 from sympy.functions.elementary.trigonometric import cos, sin
 from sympy.functions.special.elliptic_integrals import elliptic_e
-from sympy.geometry.exceptions import GeometryError
-from sympy.geometry.line import Ray2D, Segment2D, Line2D, LinearEntity3D
+from .entity import GeometryEntity, GeometrySet
+from .exceptions import GeometryError
+from .line import Line, Segment, Ray2D, Segment2D, Line2D, LinearEntity3D
+from .point import Point, Point2D, Point3D
+from .util import idiff, find
 from sympy.polys import DomainError, Poly, PolynomialError
 from sympy.polys.polyutils import _not_a_coeff, _nsort
 from sympy.solvers import solve
 from sympy.solvers.solveset import linear_coeffs
 from sympy.utilities.misc import filldedent, func_name
 
-from .entity import GeometryEntity, GeometrySet
-from .point import Point, Point2D, Point3D
-from .line import Line, Segment
-from .util import idiff
+from mpmath.libmp.libmpf import prec_to_dps
 
 import random
 
+x, y = [Dummy('ellipse_dummy', real=True) for i in range(2)]
+
 
 class Ellipse(GeometrySet):
     """An elliptical GeometryEntity.
@@ -101,9 +105,6 @@ class Ellipse(GeometrySet):
 
     def __contains__(self, o):
         if isinstance(o, Point):
-            x = Dummy('x', real=True)
-            y = Dummy('y', real=True)
-
             res = self.equation(x, y).subs({x: o.x, y: o.y})
             return trigsimp(simplify(res)) is S.Zero
         elif isinstance(o, Ellipse):
@@ -121,25 +122,24 @@ class Ellipse(GeometrySet):
 
     def __new__(
         cls, center=None, hradius=None, vradius=None, eccentricity=None, **kwargs):
+
         hradius = sympify(hradius)
         vradius = sympify(vradius)
 
-        eccentricity = sympify(eccentricity)
-
         if center is None:
             center = Point(0, 0)
         else:
+            if len(center) != 2:
+                raise ValueError('The center of "{}" must be a two dimensional point'.format(cls))
             center = Point(center, dim=2)
 
-        if len(center) != 2:
-            raise ValueError('The center of "{}" must be a two dimensional point'.format(cls))
-
         if len(list(filter(lambda x: x is not None, (hradius, vradius, eccentricity)))) != 2:
             raise ValueError(filldedent('''
                 Exactly two arguments of "hradius", "vradius", and
                 "eccentricity" must not be None.'''))
 
         if eccentricity is not None:
+            eccentricity = sympify(eccentricity)
             if eccentricity.is_negative:
                 raise GeometryError("Eccentricity of ellipse/circle should lie between [0, 1)")
             elif hradius is None:
@@ -150,7 +150,7 @@ class Ellipse(GeometrySet):
         if hradius == vradius:
             return Circle(center, hradius, **kwargs)
 
-        if hradius == 0 or vradius == 0:
+        if S.Zero in (hradius, vradius):
             return Segment(Point(center[0] - hradius, center[1] - vradius), Point(center[0] + hradius, center[1] + vradius))
 
         if hradius.is_real is False or vradius.is_real is False:
@@ -170,8 +170,6 @@ class Ellipse(GeometrySet):
             Hex string for fill color. Default is "#66cc99".
         """
 
-        from sympy.core.evalf import N
-
         c = N(self.center)
         h, v = N(self.hradius), N(self.vradius)
         return (
@@ -425,7 +423,7 @@ class Ellipse(GeometrySet):
         Returns
         =======
 
-        equation : sympy expression
+        equation : SymPy expression
 
         See Also
         ========
@@ -459,7 +457,7 @@ class Ellipse(GeometrySet):
         ==========
 
         .. [1] https://math.stackexchange.com/questions/108270/what-is-the-equation-of-an-ellipse-that-is-not-aligned-with-the-axis
-        .. [2] https://en.wikipedia.org/wiki/Ellipse#Equation_of_a_shifted_ellipse
+        .. [2] https://en.wikipedia.org/wiki/Ellipse#Shifted_ellipse
 
         """
 
@@ -496,7 +494,7 @@ class Ellipse(GeometrySet):
         Returns
         =======
 
-        equation : sympy expression
+        equation : SymPy expression
 
         Examples
         ========
@@ -665,8 +663,6 @@ class Ellipse(GeometrySet):
         [Point2D(-17/5, -12/5), Point2D(-17/5, 12/5), Point2D(7/5, -12/5), Point2D(7/5, 12/5)]
         """
         # TODO: Replace solve with nonlinsolve, when nonlinsolve will be able to solve in real domain
-        x = Dummy('x', real=True)
-        y = Dummy('y', real=True)
 
         if isinstance(o, Point):
             if o in self:
@@ -676,7 +672,9 @@ class Ellipse(GeometrySet):
 
         elif isinstance(o, (Segment2D, Ray2D)):
             ellipse_equation = self.equation(x, y)
-            result = solve([ellipse_equation, Line(o.points[0], o.points[1]).equation(x, y)], [x, y])
+            result = solve([ellipse_equation, Line(
+                o.points[0], o.points[1]).equation(x, y)], [x, y],
+                set=True)[1]
             return list(ordered([Point(i) for i in result if i in o]))
 
         elif isinstance(o, Polygon):
@@ -687,7 +685,9 @@ class Ellipse(GeometrySet):
                 return self
             else:
                 ellipse_equation = self.equation(x, y)
-                return list(ordered([Point(i) for i in solve([ellipse_equation, o.equation(x, y)], [x, y])]))
+                return list(ordered([Point(i) for i in solve(
+                    [ellipse_equation, o.equation(x, y)], [x, y],
+                    set=True)[1]]))
         elif isinstance(o, LinearEntity3D):
             raise TypeError('Entity must be two dimensional, not three dimensional')
         else:
@@ -761,7 +761,7 @@ class Ellipse(GeometrySet):
                 intersect = self.intersection(segment)
                 if len(intersect) == 1:
                     if not any(intersect[0] in i for i in segment.points) \
-                        and all(not self.encloses_point(i) for i in segment.points):
+                        and not any(self.encloses_point(i) for i in segment.points):
                         all_tangents = True
                         continue
                     else:
@@ -922,7 +922,6 @@ class Ellipse(GeometrySet):
 
         # find the 4 normal points and construct lines through them with
         # the corresponding slope
-        x, y = Dummy('x', real=True), Dummy('y', real=True)
         eq = self.equation(x, y)
         dydx = idiff(eq, y, x)
         norm = -1/dydx
@@ -982,7 +981,7 @@ class Ellipse(GeometrySet):
         """
         Calculates the semi-latus rectum of the Ellipse.
 
-        Semi-latus rectum is defined as one half of the the chord through a
+        Semi-latus rectum is defined as one half of the chord through a
         focus parallel to the conic section directrix of a conic section.
 
         Returns
@@ -1009,8 +1008,8 @@ class Ellipse(GeometrySet):
         References
         ==========
 
-        [1] http://mathworld.wolfram.com/SemilatusRectum.html
-        [2] https://en.wikipedia.org/wiki/Ellipse#Semi-latus_rectum
+        .. [1] https://mathworld.wolfram.com/SemilatusRectum.html
+        .. [2] https://en.wikipedia.org/wiki/Ellipse#Semi-latus_rectum
 
         """
         return self.major * (1 - self.eccentricity ** 2)
@@ -1130,7 +1129,6 @@ class Ellipse(GeometrySet):
         sympy.geometry.point.Point
         arbitrary_point : Returns parameterized point on ellipse
         """
-        from sympy import sin, cos, Rational
         t = _symbol('t', real=True)
         x, y = self.arbitrary_point(t).args
         # get a random value in [-1, 1) corresponding to cos(t)
@@ -1297,7 +1295,6 @@ class Ellipse(GeometrySet):
             # else p is outside the ellipse or we can't tell. In case of the
             # latter, the solutions returned will only be valid if
             # the point is not inside the ellipse; if it is, nan will result.
-            x, y = Dummy('x'), Dummy('y')
             eq = self.equation(x, y)
             dydx = idiff(eq, y, x)
             slope = Line(p, Point(x, y)).slope
@@ -1307,9 +1304,11 @@ class Ellipse(GeometrySet):
 
             # handle horizontal and vertical tangent lines
             if len(tangent_points) == 1:
-                assert tangent_points[0][
-                           0] == p.x or tangent_points[0][1] == p.y
-                return [Line(p, p + Point(1, 0)), Line(p, p + Point(0, 1))]
+                if tangent_points[0][
+                           0] == p.x or tangent_points[0][1] == p.y:
+                    return [Line(p, p + Point(1, 0)), Line(p, p + Point(0, 1))]
+                else:
+                    return [Line(p, p + Point(0, 1)), Line(p, tangent_points[0])]
 
             # others
             return [Line(p, tangent_points[0]), Line(p, tangent_points[1])]
@@ -1355,7 +1354,7 @@ class Ellipse(GeometrySet):
         Returns
         =======
 
-        I_xx, I_yy, I_xy : number or sympy expression
+        I_xx, I_yy, I_xy : number or SymPy expression
             I_xx, I_yy are second moment of area of an ellise.
             I_xy is product moment of area of an ellipse.
 
@@ -1371,7 +1370,7 @@ class Ellipse(GeometrySet):
         References
         ==========
 
-        https://en.wikipedia.org/wiki/List_of_second_moments_of_area
+        .. [1] https://en.wikipedia.org/wiki/List_of_second_moments_of_area
 
         """
 
@@ -1402,11 +1401,6 @@ class Ellipse(GeometrySet):
         plane perpendicular to the object's central axis (i.e. parallel to
         the cross-section)
 
-        References
-        ==========
-
-        https://en.wikipedia.org/wiki/Polar_moment_of_inertia
-
         Examples
         ========
 
@@ -1418,6 +1412,12 @@ class Ellipse(GeometrySet):
         >>> e = Ellipse((0, 0), a, b)
         >>> e.polar_second_moment_of_area()
         pi*a**3*b/4 + pi*a*b**3/4
+
+        References
+        ==========
+
+        .. [1] https://en.wikipedia.org/wiki/Polar_moment_of_inertia
+
         """
         second_moment = self.second_moment_of_area()
         return second_moment[0] + second_moment[1]
@@ -1430,11 +1430,6 @@ class Ellipse(GeometrySet):
         ratio of second moment of area to the distance of the extreme end of
         the ellipse from the centroidal axis.
 
-        References
-        ==========
-
-        https://en.wikipedia.org/wiki/Section_modulus
-
         Parameters
         ==========
 
@@ -1465,6 +1460,12 @@ class Ellipse(GeometrySet):
         (8*pi, 4*pi)
         >>> e.section_modulus((2, 2))
         (16*pi, 4*pi)
+
+        References
+        ==========
+
+        .. [1] https://en.wikipedia.org/wiki/Section_modulus
+
         """
         x_c, y_c = self.center
         if point is None:
@@ -1495,7 +1496,7 @@ class Circle(Ellipse):
     ==========
 
     center : Point
-    radius : number or sympy expression
+    radius : number or SymPy expression
     points : sequence of three Points
     equation : equation of a circle
 
@@ -1521,8 +1522,7 @@ class Circle(Ellipse):
     Examples
     ========
 
-    >>> from sympy import Eq
-    >>> from sympy.geometry import Point, Circle
+    >>> from sympy import Point, Circle, Eq
     >>> from sympy.abc import x, y, a, b
 
     A circle constructed from a center and radius:
@@ -1551,13 +1551,11 @@ class Circle(Ellipse):
     """
 
     def __new__(cls, *args, **kwargs):
-        from sympy.geometry.util import find
-        from .polygon import Triangle
         evaluate = kwargs.get('evaluate', global_parameters.evaluate)
         if len(args) == 1 and isinstance(args[0], (Expr, Eq)):
             x = kwargs.get('x', 'x')
             y = kwargs.get('y', 'y')
-            equation = args[0]
+            equation = args[0].expand()
             if isinstance(equation, Eq):
                 equation = equation.lhs - equation.rhs
             x = find(x, equation)
@@ -1568,12 +1566,12 @@ class Circle(Ellipse):
             except ValueError:
                 raise GeometryError("The given equation is not that of a circle.")
 
-            if a == 0 or b == 0 or a != b:
+            if S.Zero in (a, b) or a != b:
                 raise GeometryError("The given equation is not that of a circle.")
 
             center_x = -c/a/2
             center_y = -d/b/2
-            r2 = (center_x**2) + (center_y**2) - e
+            r2 = (center_x**2) + (center_y**2) - e/a
 
             return Circle((center_x, center_y), sqrt(r2), evaluate=evaluate)
 
@@ -1603,6 +1601,13 @@ class Circle(Ellipse):
 
             raise GeometryError("Circle.__new__ received unknown arguments")
 
+    def _eval_evalf(self, prec=15, **options):
+        pt, r = self.args
+        dps = prec_to_dps(prec)
+        pt = pt.evalf(n=dps, **options)
+        r = r.evalf(n=dps, **options)
+        return self.func(pt, r, evaluate=False)
+
     @property
     def circumference(self):
         """The circumference of the circle.
@@ -1693,7 +1698,7 @@ class Circle(Ellipse):
         Returns
         =======
 
-        radius : number or sympy expression
+        radius : number or SymPy expression
 
         See Also
         ========
@@ -1772,4 +1777,4 @@ class Circle(Ellipse):
         return abs(self.radius)
 
 
-from .polygon import Polygon
+from .polygon import Polygon, Triangle
