@@ -1,12 +1,24 @@
-from sympy import (meijerg, I, S, integrate, Integral, oo, gamma, cosh, sinc,
-                   hyperexpand, exp, simplify, sqrt, pi, erf, erfc, sin, cos,
-                   exp_polar, polygamma, hyper, log, expand_func, Rational)
+from sympy.core.function import expand_func
+from sympy.core.numbers import (I, Rational, oo, pi)
+from sympy.core.singleton import S
+from sympy.core.sorting import default_sort_key
+from sympy.functions.elementary.complexes import Abs, arg, re, unpolarify
+from sympy.functions.elementary.exponential import (exp, exp_polar, log)
+from sympy.functions.elementary.hyperbolic import cosh, acosh
+from sympy.functions.elementary.miscellaneous import sqrt
+from sympy.functions.elementary.piecewise import Piecewise, piecewise_fold
+from sympy.functions.elementary.trigonometric import (cos, sin, sinc, asin)
+from sympy.functions.special.error_functions import (erf, erfc)
+from sympy.functions.special.gamma_functions import (gamma, polygamma)
+from sympy.functions.special.hyper import (hyper, meijerg)
+from sympy.integrals.integrals import (Integral, integrate)
+from sympy.simplify.hyperexpand import hyperexpand
+from sympy.simplify.simplify import simplify
 from sympy.integrals.meijerint import (_rewrite_single, _rewrite1,
-        meijerint_indefinite, _inflate_g, _create_lookup_table,
-        meijerint_definite, meijerint_inversion)
-from sympy.utilities import default_sort_key
+    meijerint_indefinite, _inflate_g, _create_lookup_table,
+    meijerint_definite, meijerint_inversion)
 from sympy.testing.pytest import slow
-from sympy.testing.randtest import (verify_numerically,
+from sympy.core.random import (verify_numerically,
         random_complex_number as randcplx)
 from sympy.abc import x, y, a, b, c, d, s, t, z
 
@@ -28,7 +40,7 @@ def test_rewrite_single():
     tn(x**y)
 
     def u(expr, x):
-        from sympy import Add, exp, exp_polar
+        from sympy.core.add import Add
         r = _rewrite_single(expr, x)
         e = Add(*[res[0]*res[2] for res in r[0]]).replace(
             exp_polar, exp)  # XXX Hack?
@@ -86,7 +98,7 @@ def test_inflate():
             d: randcplx(), y: randcplx()/10}
 
     def t(a, b, arg, n):
-        from sympy import Mul
+        from sympy.core.mul import Mul
         m1 = meijerg(a, b, arg)
         m2 = Mul(*_inflate_g(m1, n))
         # NOTE: (the random number)**9 must still be on the principal sheet.
@@ -98,7 +110,7 @@ def test_inflate():
 
 
 def test_recursive():
-    from sympy import symbols
+    from sympy.core.symbol import symbols
     a, b, c = symbols('a b c', positive=True)
     r = exp(-(x - a)**2)*exp(-(x - b)**2)
     e = integrate(r, (x, 0, oo), meijerg=True)
@@ -117,7 +129,8 @@ def test_recursive():
 
 @slow
 def test_meijerint():
-    from sympy import symbols, expand, arg
+    from sympy.core.function import expand
+    from sympy.core.symbol import symbols
     s, t, mu = symbols('s t mu', real=True)
     assert integrate(meijerg([], [], [0], [], s*t)
                      *meijerg([], [], [mu/2], [-mu/2], t**2/4),
@@ -140,7 +153,7 @@ def test_meijerint():
         b**(a + 1)/(a + 1)
 
     # This tests various conditions and expansions:
-    meijerint_definite((x + 1)**3*exp(-x), x, 0, oo) == (16, True)
+    assert meijerint_definite((x + 1)**3*exp(-x), x, 0, oo) == (16, True)
 
     # Again, how about simplifications?
     sigma, mu = symbols('sigma mu', positive=True)
@@ -187,13 +200,13 @@ def test_meijerint():
     # (This is besselj*besselj in disguise, to stop the product from being
     #  recognised in the tables.)
     a, b, s = symbols('a b s')
-    from sympy import And, re
     assert meijerint_definite(meijerg([], [], [a/2], [-a/2], x/4)
-                  *meijerg([], [], [b/2], [-b/2], x/4)*x**(s - 1), x, 0, oo) == \
+                  *meijerg([], [], [b/2], [-b/2], x/4)*x**(s - 1), x, 0, oo
+        ) == (
         (4*2**(2*s - 2)*gamma(-2*s + 1)*gamma(a/2 + b/2 + s)
          /(gamma(-a/2 + b/2 - s + 1)*gamma(a/2 - b/2 - s + 1)
            *gamma(a/2 + b/2 - s + 1)),
-            And(0 < -2*re(4*s) + 8, 0 < re(a/2 + b/2 + s), re(2*s) < 1))
+            (re(s) < 1) & (re(s) < S(1)/2) & (re(a)/2 + re(b)/2 + re(s) > 0)))
 
     # test a bug
     assert integrate(sin(x**a)*sin(x**b), (x, 0, oo), meijerg=True) == \
@@ -204,7 +217,7 @@ def test_meijerint():
         (sqrt(pi)*polygamma(0, S.Half)/4).expand()
 
     # Test hyperexpand bug.
-    from sympy import lowergamma
+    from sympy.functions.special.gamma_functions import lowergamma
     n = symbols('n', integer=True)
     assert simplify(integrate(exp(-x)*x**n, x, meijerg=True)) == \
         lowergamma(n + 1, x)
@@ -222,7 +235,7 @@ def test_meijerint():
 
 
 def test_bessel():
-    from sympy import besselj, besseli
+    from sympy.functions.special.bessel import (besseli, besselj)
     assert simplify(integrate(besselj(a, z)*besselj(b, z)/z, (z, 0, oo),
                      meijerg=True, conds='none')) == \
         2*sin(pi*(a/2 - b/2))/(pi*(a - b)*(a + b))
@@ -270,7 +283,8 @@ def test_bessel():
 
 
 def test_inversion():
-    from sympy import piecewise_fold, besselj, sqrt, sin, cos, Heaviside
+    from sympy.functions.special.bessel import besselj
+    from sympy.functions.special.delta_functions import Heaviside
 
     def inv(f):
         return piecewise_fold(meijerint_inversion(f, s, t))
@@ -286,7 +300,8 @@ def test_inversion():
 
 
 def test_inversion_conditional_output():
-    from sympy import Symbol, InverseLaplaceTransform
+    from sympy.core.symbol import Symbol
+    from sympy.integrals.transforms import InverseLaplaceTransform
 
     a = Symbol('a', positive=True)
     F = sqrt(pi/a)*exp(-2*sqrt(a)*sqrt(s))
@@ -306,7 +321,8 @@ def test_inversion_conditional_output():
 
 
 def test_inversion_exp_real_nonreal_shift():
-    from sympy import Symbol, DiracDelta
+    from sympy.core.symbol import Symbol
+    from sympy.functions.special.delta_functions import DiracDelta
     r = Symbol('r', real=True)
     c = Symbol('c', extended_real=False)
     a = 1 + 2*I
@@ -321,12 +337,12 @@ def test_inversion_exp_real_nonreal_shift():
 
 @slow
 def test_lookup_table():
-    from random import uniform, randrange
-    from sympy import Add
+    from sympy.core.random import uniform, randrange
+    from sympy.core.add import Add
     from sympy.integrals.meijerint import z as z_dummy
     table = {}
     _create_lookup_table(table)
-    for _, l in sorted(table.items()):
+    for _, l in table.items():
         for formula, terms, cond, hint in sorted(l, key=default_sort_key):
             subs = {}
             for ai in list(formula.free_symbols) + [z_dummy]:
@@ -353,7 +369,8 @@ def test_lookup_table():
 
 
 def test_branch_bug():
-    from sympy import powdenest, lowergamma
+    from sympy.functions.special.gamma_functions import lowergamma
+    from sympy.simplify.powsimp import powdenest
     # TODO gammasimp cannot prove that the factor is unity
     assert powdenest(integrate(erf(x**3), x, meijerg=True).diff(x),
            polar=True) == 2*erf(x**3)*gamma(Rational(2, 3))/3/gamma(Rational(5, 3))
@@ -363,7 +380,7 @@ def test_branch_bug():
 
 
 def test_linear_subs():
-    from sympy import besselj
+    from sympy.functions.special.bessel import besselj
     assert integrate(sin(x - 1), x, meijerg=True) == -cos(1 - x)
     assert integrate(besselj(1, x - 1), x, meijerg=True) == -besselj(0, 1 - x)
 
@@ -371,8 +388,10 @@ def test_linear_subs():
 @slow
 def test_probability():
     # various integrals from probability theory
-    from sympy.abc import x, y
-    from sympy import symbols, Symbol, Abs, expand_mul, gammasimp, powsimp, sin
+    from sympy.core.function import expand_mul
+    from sympy.core.symbol import (Symbol, symbols)
+    from sympy.simplify.gammasimp import gammasimp
+    from sympy.simplify.powsimp import powsimp
     mu1, mu2 = symbols('mu1 mu2', nonzero=True)
     sigma1, sigma2 = symbols('sigma1 sigma2', positive=True)
     rate = Symbol('lambda', positive=True)
@@ -442,7 +461,7 @@ def test_probability():
     i = integrate(x*betadist, (x, 0, oo), meijerg=True, conds='separate')
     assert (gammasimp(i[0]), i[1]) == (alpha/(beta - 1), 1 < beta)
     j = integrate(x**2*betadist, (x, 0, oo), meijerg=True, conds='separate')
-    assert j[1] == (1 < beta - 1)
+    assert j[1] == (beta > 2)
     assert gammasimp(j[0] - i[0]**2) == (alpha + beta - 1)*alpha \
         /(beta - 2)/(beta - 1)**2
 
@@ -542,7 +561,7 @@ def test_probability():
         lamda**n*gamma(1 + n/k)
 
     # rice distribution
-    from sympy import besseli
+    from sympy.functions.special.bessel import besseli
     nu, sigma = symbols('nu sigma', positive=True)
     rice = x/sigma**2*exp(-(x**2 + nu**2)/2/sigma**2)*besseli(0, x*nu/sigma**2)
     assert integrate(rice, (x, 0, oo), meijerg=True) == 1
@@ -568,8 +587,9 @@ def test_probability():
 @slow
 def test_expint():
     """ Test various exponential integrals. """
-    from sympy import (expint, unpolarify, Symbol, Ci, Si, Shi, Chi,
-                       sin, cos, sinh, cosh, Ei)
+    from sympy.core.symbol import Symbol
+    from sympy.functions.elementary.hyperbolic import sinh
+    from sympy.functions.special.error_functions import (Chi, Ci, Ei, Shi, Si, expint)
     assert simplify(unpolarify(integrate(exp(-z*x)/x**y, (x, 1, oo),
                 meijerg=True, conds='none'
                 ).rewrite(expint).expand(func=True))) == expint(y, z)
@@ -620,24 +640,30 @@ def test_expint():
 
 
 def test_messy():
-    from sympy import (laplace_transform, Si, Shi, Chi, atan, Piecewise,
-                       acoth, E1, besselj, acosh, asin, And, re,
-                       fourier_transform, sqrt)
-    assert laplace_transform(Si(x), x, s) == ((-atan(s) + pi/2)/s, 0, True)
+    from sympy.functions.elementary.hyperbolic import (acosh, acoth)
+    from sympy.functions.elementary.trigonometric import (asin, atan)
+    from sympy.functions.special.bessel import besselj
+    from sympy.functions.special.error_functions import (Chi, E1, Shi, Si)
+    from sympy.integrals.transforms import (fourier_transform, laplace_transform)
+    assert (laplace_transform(Si(x), x, s, simplify=True) ==
+            ((-atan(s) + pi/2)/s, 0, True))
 
-    assert laplace_transform(Shi(x), x, s) == (acoth(s)/s, 1, True)
+    assert laplace_transform(Shi(x), x, s, simplify=True) == (
+        acoth(s)/s, -oo, s**2 > 1)
 
     # where should the logs be simplified?
-    assert laplace_transform(Chi(x), x, s) == \
-        ((log(s**(-2)) - log(1 - 1/s**2))/(2*s), 1, True)
+    assert laplace_transform(Chi(x), x, s, simplify=True) == (
+        (log(s**(-2)) - log(1 - 1/s**2))/(2*s), -oo, s**2 > 1)
 
-    # TODO maybe simplify the inequalities?
+    # TODO maybe simplify the inequalities? when the simplification
+    # allows for generators instead of symbols this will work
     assert laplace_transform(besselj(a, x), x, s)[1:] == \
-        (0, And(re(a/2) + S.Half > S.Zero, re(a/2) + 1 > S.Zero))
+        (0, (re(a) > -2) & (re(a) > -1))
 
     # NOTE s < 0 can be done, but argument reduction is not good enough yet
-    assert fourier_transform(besselj(1, x)/x, x, s, noconds=False) == \
-        (Piecewise((0, 4*abs(pi**2*s**2) > 1),
+    ans = fourier_transform(besselj(1, x)/x, x, s, noconds=False)
+    assert (ans[0].factor(deep=True).expand(), ans[1]) == \
+        (Piecewise((0, (s > 1/(2*pi)) | (s < -1/(2*pi))),
                    (2*sqrt(-4*pi**2*s**2 + 1), True)), s > 0)
     # TODO FT(besselj(0,x)) - conditions are messy (but for acceptable reasons)
     #                       - folding could be better
@@ -670,7 +696,7 @@ def test_issue_6348():
 
 
 def test_fresnel():
-    from sympy import fresnels, fresnelc
+    from sympy.functions.special.error_functions import (fresnelc, fresnels)
 
     assert expand_func(integrate(sin(pi*x**2/2), x)) == fresnels(x)
     assert expand_func(integrate(cos(pi*x**2/2), x)) == fresnelc(x)
@@ -698,13 +724,13 @@ def test_issue_10211():
 
 
 def test_issue_11806():
-    from sympy import symbols
+    from sympy.core.symbol import symbols
     y, L = symbols('y L', positive=True)
     assert integrate(1/sqrt(x**2 + y**2)**3, (x, -L, L)) == \
         2*L/(y**2*sqrt(L**2 + y**2))
 
 def test_issue_10681():
-    from sympy import RR
+    from sympy.polys.domains.realfield import RR
     from sympy.abc import R, r
     f = integrate(r**2*(R**2-r**2)**0.5, r, meijerg=True)
     g = (1.0/3)*R**1.0*r**3*hyper((-0.5, Rational(3, 2)), (Rational(5, 2),),
@@ -712,16 +738,27 @@ def test_issue_10681():
     assert RR.almosteq((f/g).n(), 1.0, 1e-12)
 
 def test_issue_13536():
-    from sympy import Symbol
-    a = Symbol('a', real=True, positive=True)
+    from sympy.core.symbol import Symbol
+    a = Symbol('a', positive=True)
     assert integrate(1/x**2, (x, oo, a)) == -1/a
 
 
 def test_issue_6462():
-    from sympy import Symbol
+    from sympy.core.symbol import Symbol
     x = Symbol('x')
     n = Symbol('n')
     # Not the actual issue, still wrong answer for n = 1, but that there is no
     # exception
     assert integrate(cos(x**n)/x**n, x, meijerg=True).subs(n, 2).equals(
             integrate(cos(x**2)/x**2, x, meijerg=True))
+
+
+def test_indefinite_1_bug():
+    assert integrate((b + t)**(-a), t, meijerg=True
+        ) == -b**(1 - a)*(1 + t/b)**(1 - a)/(a - 1)
+
+
+def test_pr_23583():
+    # This result is wrong. Check whether new result is correct when this test fail.
+    assert integrate(1/sqrt((x - I)**2-1), meijerg=True) == \
+           Piecewise((acosh(x - I), Abs((x - I)**2) > 1), (-I*asin(x - I), True))
