@@ -1,8 +1,19 @@
-from sympy import (symbols, MatrixSymbol, MatPow, BlockMatrix, KroneckerDelta,
-        Identity, ZeroMatrix, ImmutableMatrix, eye, Sum, Dummy, trace,
-        Symbol)
-from sympy.testing.pytest import raises, XFAIL
-from sympy.matrices.expressions.matexpr import MatrixElement, MatrixExpr
+from sympy.concrete.summations import Sum
+from sympy.core.symbol import symbols, Symbol, Dummy
+from sympy.functions.elementary.miscellaneous import sqrt
+from sympy.functions.special.tensor_functions import KroneckerDelta
+from sympy.matrices.dense import eye
+from sympy.matrices.expressions.blockmatrix import BlockMatrix
+from sympy.matrices.expressions.hadamard import HadamardPower
+from sympy.matrices.expressions.matexpr import (MatrixSymbol,
+    MatrixExpr, MatrixElement)
+from sympy.matrices.expressions.matpow import MatPow
+from sympy.matrices.expressions.special import (ZeroMatrix, Identity,
+    OneMatrix)
+from sympy.matrices.expressions.trace import Trace, trace
+from sympy.matrices.immutable import ImmutableMatrix
+from sympy.tensor.array.expressions.array_expressions import ArrayTensorProduct
+from sympy.testing.pytest import XFAIL, raises
 
 k, l, m, n = symbols('k l m n', integer=True)
 i, j = symbols('i j', integer=True)
@@ -51,7 +62,22 @@ def test_pow_index():
     assert Q[0, 0] == A[0, 0]**2 + A[0, 1]*A[1, 0]
     n = symbols("n")
     Q2 = A**n
-    assert Q2[0, 0] == MatrixElement(Q2, 0, 0)
+    assert Q2[0, 0] == 2*(
+            -sqrt((A[0, 0] + A[1, 1])**2 - 4*A[0, 0]*A[1, 1] +
+            4*A[0, 1]*A[1, 0])/2 + A[0, 0]/2 + A[1, 1]/2
+        )**n * \
+        A[0, 1]*A[1, 0]/(
+            (sqrt(A[0, 0]**2 - 2*A[0, 0]*A[1, 1] + 4*A[0, 1]*A[1, 0] +
+                  A[1, 1]**2) + A[0, 0] - A[1, 1])*
+            sqrt(A[0, 0]**2 - 2*A[0, 0]*A[1, 1] + 4*A[0, 1]*A[1, 0] + A[1, 1]**2)
+        ) - 2*(
+            sqrt((A[0, 0] + A[1, 1])**2 - 4*A[0, 0]*A[1, 1] +
+            4*A[0, 1]*A[1, 0])/2 + A[0, 0]/2 + A[1, 1]/2
+        )**n * A[0, 1]*A[1, 0]/(
+            (-sqrt(A[0, 0]**2 - 2*A[0, 0]*A[1, 1] + 4*A[0, 1]*A[1, 0] +
+            A[1, 1]**2) + A[0, 0] - A[1, 1])*
+            sqrt(A[0, 0]**2 - 2*A[0, 0]*A[1, 1] + 4*A[0, 1]*A[1, 0] + A[1, 1]**2)
+        )
 
 
 def test_transpose_index():
@@ -224,7 +250,11 @@ def test_matrix_expression_from_index_summation():
     expr = Sum(C[c, d]*A[b, a]*B[c, b], (b, 0, k-1), (c, 0, k-1))
     assert MatrixSymbol.from_index_summation(expr, a) == A.T*B.T*C
     expr = Sum(A[a, b] + B[a, b], (a, 0, k-1), (b, 0, k-1))
-    assert MatrixExpr.from_index_summation(expr, a) == A + B
+    assert MatrixExpr.from_index_summation(expr, a) == OneMatrix(1, k)*A*OneMatrix(k, 1) + OneMatrix(1, k)*B*OneMatrix(k, 1)
+    expr = Sum(A[a, b]**2, (a, 0, k - 1), (b, 0, k - 1))
+    assert MatrixExpr.from_index_summation(expr, a) == Trace(A * A.T)
+    expr = Sum(A[a, b]**3, (a, 0, k - 1), (b, 0, k - 1))
+    assert MatrixExpr.from_index_summation(expr, a) == Trace(HadamardPower(A.T, 2) * A)
     expr = Sum((A[a, b] + B[a, b])*C[b, c], (b, 0, k-1))
     assert MatrixExpr.from_index_summation(expr, a) == (A+B)*C
     expr = Sum((A[a, b] + B[b, a])*C[b, c], (b, 0, k-1))
@@ -259,11 +289,11 @@ def test_matrix_expression_from_index_summation():
     assert MatrixExpr.from_index_summation(expr, a) == A*B
 
     expr = Sum(KroneckerDelta(i1, m)*KroneckerDelta(i2, n)*A[i, i1]*A[j, i2], (i1, 0, k-1), (i2, 0, k-1))
-    assert MatrixExpr.from_index_summation(expr, m) == A.T*A[j, n]
+    assert MatrixExpr.from_index_summation(expr, m) == ArrayTensorProduct(A.T, A)
 
     # Test numbered indices:
     expr = Sum(A[i1, i2]*w1[i2, 0], (i2, 0, k-1))
-    assert MatrixExpr.from_index_summation(expr, i1) == A*w1
+    assert MatrixExpr.from_index_summation(expr, i1) == MatrixElement(A*w1, i1, 0)
 
     expr = Sum(A[i1, i2]*B[i2, 0], (i2, 0, k-1))
     assert MatrixExpr.from_index_summation(expr, i1) == MatrixElement(A*B, i1, 0)
