@@ -1,8 +1,10 @@
 from sympy.functions import adjoint, conjugate, transpose
 from sympy.matrices.expressions import MatrixSymbol, Adjoint, trace, Transpose
 from sympy.matrices import eye, Matrix
-from sympy import symbols, S
-from sympy import refine, Q
+from sympy.assumptions.ask import Q
+from sympy.assumptions.refine import refine
+from sympy.core.singleton import S
+from sympy.core.symbol import symbols
 
 n, m, l, k, p = symbols('n m l k p', integer=True)
 A = MatrixSymbol('A', n, m)
@@ -36,6 +38,17 @@ def test_transpose():
     assert Transpose(A*B).doit() == Transpose(B) * Transpose(A)
 
 
+def test_transpose_MatAdd_MatMul():
+    # Issue 16807
+    from sympy.functions.elementary.trigonometric import cos
+
+    x = symbols('x')
+    M = MatrixSymbol('M', 3, 3)
+    N = MatrixSymbol('N', 3, 3)
+
+    assert (N + (cos(x) * M)).T == cos(x)*M.T + N.T
+
+
 def test_refine():
     assert refine(C.T, Q.symmetric(C)) == C
 
@@ -44,3 +57,13 @@ def test_transpose1x1():
     m = MatrixSymbol('m', 1, 1)
     assert m == refine(m.T)
     assert m == refine(m.T.T)
+
+def test_issue_9817():
+    from sympy.matrices.expressions import Identity
+    v = MatrixSymbol('v', 3, 1)
+    A = MatrixSymbol('A', 3, 3)
+    x = Matrix([i + 1 for i in range(3)])
+    X = Identity(3)
+    quadratic = v.T * A * v
+    subbed = quadratic.xreplace({v:x, A:X})
+    assert subbed.as_explicit() == Matrix([[14]])
