@@ -1,4 +1,5 @@
-from sympy import print_gtk, sin
+from sympy.functions.elementary.trigonometric import sin
+from sympy.printing.gtk import print_gtk
 from sympy.testing.pytest import XFAIL, raises
 
 # this test fails if python-lxml isn't installed. We don't want to depend on
