@@ -7,11 +7,12 @@ from sympy.functions import (arg, atan2, bernoulli, beta, ceiling, chebyshevu,
                              factorial, floor, harmonic, Heaviside, im,
                              laguerre, LambertW, log, Max, Min, Piecewise,
                              polylog, re, RisingFactorial, sign, sinc, sqrt,
-                             zeta, binomial, legendre)
+                             zeta, binomial, legendre, dirichlet_eta,
+                             riemann_xi)
 from sympy.functions import (sin, cos, tan, cot, sec, csc, asin, acos, acot,
                              atan, asec, acsc, sinh, cosh, tanh, coth, csch,
                              sech, asinh, acosh, atanh, acoth, asech, acsch)
-from sympy.utilities.pytest import raises, XFAIL
+from sympy.testing.pytest import raises, XFAIL
 from sympy.utilities.lambdify import implemented_function
 from sympy.matrices import (eye, Matrix, MatrixSymbol, Identity,
                             HadamardProduct, SparseMatrix, HadamardPower)
@@ -24,9 +25,8 @@ from sympy.functions.special.gamma_functions import (gamma, lowergamma,
 from sympy.functions.special.error_functions import (Chi, Ci, erf, erfc, erfi,
                                                      erfcinv, erfinv, fresnelc,
                                                      fresnels, li, Shi, Si, Li,
-                                                     erf2)
-from sympy import octave_code
-from sympy import octave_code as mcode
+                                                     erf2, Ei)
+from sympy.printing.octave import octave_code, octave_code as mcode
 
 x, y, z = symbols('x,y,z')
 
@@ -90,7 +90,7 @@ def test_Function_change_name():
     assert mcode(RisingFactorial(x, y)) == "pochhammer(x, y)"
     assert mcode(DiracDelta(x)) == "dirac(x)"
     assert mcode(DiracDelta(x, 3)) == "dirac(3, x)"
-    assert mcode(Heaviside(x)) == "heaviside(x)"
+    assert mcode(Heaviside(x)) == "heaviside(x, 1/2)"
     assert mcode(Heaviside(x, y)) == "heaviside(x, y)"
     assert mcode(binomial(x, y)) == "bincoeff(x, y)"
     assert mcode(Mod(x, y)) == "mod(x, y)"
@@ -392,7 +392,7 @@ def test_octave_not_supported():
 
 
 def test_octave_not_supported_not_on_whitelist():
-    from sympy import assoc_laguerre
+    from sympy.functions.special.polynomials import assoc_laguerre
     assert mcode(assoc_laguerre(x, y, z)) == (
         "% Not supported in Octave:\n"
         "% assoc_laguerre\n"
@@ -463,7 +463,7 @@ def test_sparse():
 
 def test_sinc():
     assert mcode(sinc(x)) == 'sinc(x/pi)'
-    assert mcode(sinc((x + 3))) == 'sinc((x + 3)/pi)'
+    assert mcode(sinc(x + 3)) == 'sinc((x + 3)/pi)'
     assert mcode(sinc(pi*(x + 3))) == 'sinc(x + 3)'
 
 
@@ -494,6 +494,11 @@ def test_specfun():
     assert octave_code(LambertW(x)) == 'lambertw(x)'
     assert octave_code(LambertW(x, n)) == 'lambertw(n, x)'
 
+    # Automatic rewrite
+    assert octave_code(Ei(x)) == 'logint(exp(x))'
+    assert octave_code(dirichlet_eta(x)) == '((x == 1).*(log(2)) + (~(x == 1)).*((1 - 2.^(1 - x)).*zeta(x)))'
+    assert octave_code(riemann_xi(x)) == 'pi.^(-x/2).*x.*(x - 1).*gamma(x/2).*zeta(x)/2'
+
 
 def test_MatrixElement_printing():
     # test cases for issue #11821
