@@ -1,20 +1,26 @@
 """ Tools for doing common subexpression elimination.
 """
-from __future__ import print_function, division
+from collections import defaultdict
 
-from sympy.core import Basic, Mul, Add, Pow, sympify, Symbol
-from sympy.core.compatibility import iterable
+from sympy.core import Basic, Mul, Add, Pow, sympify
 from sympy.core.containers import Tuple, OrderedSet
 from sympy.core.exprtools import factor_terms
-from sympy.core.function import _coeff_isneg
 from sympy.core.singleton import S
+from sympy.core.sorting import ordered
+from sympy.core.symbol import symbols, Symbol
+from sympy.matrices import (MatrixBase, Matrix, ImmutableMatrix,
+                            SparseMatrix, ImmutableSparseMatrix)
+from sympy.matrices.expressions import (MatrixExpr, MatrixSymbol, MatMul,
+                                        MatAdd, MatPow, Inverse)
+from sympy.matrices.expressions.matexpr import MatrixElement
+from sympy.polys.rootoftools import RootOf
 from sympy.utilities.iterables import numbered_symbols, sift, \
-        topological_sort, ordered
+        topological_sort, iterable
 
 from . import cse_opts
 
 # (preprocessor, postprocessor) pairs which are commonly useful. They should
-# each take a sympy expression and return a possibly transformed expression.
+# each take a SymPy expression and return a possibly transformed expression.
 # When used in the function ``cse()``, the target expressions will be transformed
 # by each of the preprocessor functions in order. After the common
 # subexpressions are eliminated, each resulting expression will have the
@@ -35,7 +41,7 @@ basic_optimizations = [(cse_opts.sub_pre, cse_opts.sub_post),
 
 
 def reps_toposort(r):
-    """Sort replacements `r` so (k1, v1) appears before (k2, v2)
+    """Sort replacements ``r`` so (k1, v1) appears before (k2, v2)
     if k2 is in v1's free symbols. This orders items in the
     way that cse returns its results (hence, in order to use the
     replacements in a substitution option it would make sense
@@ -88,6 +94,74 @@ def cse_separate(r, e):
     e = d[False]
     return [reps_toposort(r), e]
 
+
+def cse_release_variables(r, e):
+    """
+    Return tuples giving ``(a, b)`` where ``a`` is a symbol and ``b`` is
+    either an expression or None. The value of None is used when a
+    symbol is no longer needed for subsequent expressions.
+
+    Use of such output can reduce the memory footprint of lambdified
+    expressions that contain large, repeated subexpressions.
+
+    Examples
+    ========
+
+    >>> from sympy import cse
+    >>> from sympy.simplify.cse_main import cse_release_variables
+    >>> from sympy.abc import x, y
+    >>> eqs = [(x + y - 1)**2, x, x + y, (x + y)/(2*x + 1) + (x + y - 1)**2, (2*x + 1)**(x + y)]
+    >>> defs, rvs = cse_release_variables(*cse(eqs))
+    >>> for i in defs:
+    ...   print(i)
+    ...
+    (x0, x + y)
+    (x1, (x0 - 1)**2)
+    (x2, 2*x + 1)
+    (_3, x0/x2 + x1)
+    (_4, x2**x0)
+    (x2, None)
+    (_0, x1)
+    (x1, None)
+    (_2, x0)
+    (x0, None)
+    (_1, x)
+    >>> print(rvs)
+    (_0, _1, _2, _3, _4)
+    """
+    if not r:
+        return r, e
+
+    s, p = zip(*r)
+    esyms = symbols('_:%d' % len(e))
+    syms = list(esyms)
+    s = list(s)
+    in_use = set(s)
+    p = list(p)
+    # sort e so those with most sub-expressions appear first
+    e = [(e[i], syms[i]) for i in range(len(e))]
+    e, syms = zip(*sorted(e,
+        key=lambda x: -sum([p[s.index(i)].count_ops()
+        for i in x[0].free_symbols & in_use])))
+    syms = list(syms)
+    p += e
+    rv = []
+    i = len(p) - 1
+    while i >= 0:
+        _p = p.pop()
+        c = in_use & _p.free_symbols
+        if c: # sorting for canonical results
+            rv.extend([(s, None) for s in sorted(c, key=str)])
+        if i >= len(r):
+            rv.append((syms.pop(), _p))
+        else:
+            rv.append((s[i], _p))
+        in_use -= c
+        i -= 1
+    rv.reverse()
+    return rv, esyms
+
+
 # ====end of cse postprocess idioms===========================
 
 
@@ -98,7 +172,7 @@ def preprocess_for_cse(expr, optimizations):
     Parameters
     ==========
 
-    expr : sympy expression
+    expr : SymPy expression
         The target expression to optimize.
     optimizations : list of (callable, callable) pairs
         The (preprocessor, postprocessor) pairs.
@@ -106,7 +180,7 @@ def preprocess_for_cse(expr, optimizations):
     Returns
     =======
 
-    expr : sympy expression
+    expr : SymPy expression
         The transformed expression.
     """
     for pre, post in optimizations:
@@ -116,13 +190,13 @@ def preprocess_for_cse(expr, optimizations):
 
 
 def postprocess_for_cse(expr, optimizations):
-    """ Postprocess an expression after common subexpression elimination to
-    return the expression to canonical sympy form.
+    """Postprocess an expression after common subexpression elimination to
+    return the expression to canonical SymPy form.
 
     Parameters
     ==========
 
-    expr : sympy expression
+    expr : SymPy expression
         The target expression to transform.
     optimizations : list of (callable, callable) pairs, optional
         The (preprocessor, postprocessor) pairs.  The postprocessors will be
@@ -132,7 +206,7 @@ def postprocess_for_cse(expr, optimizations):
     Returns
     =======
 
-    expr : sympy expression
+    expr : SymPy expression
         The transformed expression.
     """
     for pre, post in reversed(optimizations):
@@ -141,7 +215,7 @@ def postprocess_for_cse(expr, optimizations):
     return expr
 
 
-class FuncArgTracker(object):
+class FuncArgTracker:
     """
     A class which manages a mapping from functions to arguments and an inverse
     mapping from arguments to functions.
@@ -196,11 +270,12 @@ class FuncArgTracker(object):
     def get_common_arg_candidates(self, argset, min_func_i=0):
         """Return a dict whose keys are function numbers. The entries of the dict are
         the number of arguments said function has in common with
-        `argset`. Entries have at least 2 items in common.  All keys have
-        value at least `min_func_i`.
+        ``argset``. Entries have at least 2 items in common.  All keys have
+        value at least ``min_func_i``.
         """
-        from collections import defaultdict
         count_map = defaultdict(lambda: 0)
+        if not argset:
+            return count_map
 
         funcsets = [self.arg_to_funcset[arg] for arg in argset]
         # As an optimization below, we handle the largest funcset separately from
@@ -230,7 +305,7 @@ class FuncArgTracker(object):
             if func_i in larger_funcs_container:
                 count_map[func_i] += 1
 
-        return dict((k, v) for k, v in count_map.items() if v >= 2)
+        return {k: v for k, v in count_map.items() if v >= 2}
 
     def get_subset_candidates(self, argset, restrict_to_funcset=None):
         """
@@ -267,7 +342,7 @@ class FuncArgTracker(object):
         self.func_to_argset[func_i].update(new_args)
 
 
-class Unevaluated(object):
+class Unevaluated:
 
     def __init__(self, func, args):
         self.func = func
@@ -309,9 +384,9 @@ def match_common_args(func_class, funcs, opt_subs):
     func_class: class
         The function class (e.g. Add, Mul)
     funcs: list of functions
-        A list of function calls
+        A list of function calls.
     opt_subs: dict
-        A dictionary of substitutions which this function may update
+        A dictionary of substitutions which this function may update.
     """
 
     # Sort to ensure that whole-function subexpressions come before the items
@@ -381,15 +456,14 @@ def match_common_args(func_class, funcs, opt_subs):
         arg_tracker.stop_arg_tracking(i)
 
 
-
 def opt_cse(exprs, order='canonical'):
     """Find optimization opportunities in Adds, Muls, Pows and negative
-    coefficient Muls
+    coefficient Muls.
 
     Parameters
     ==========
 
-    exprs : list of sympy expressions
+    exprs : list of SymPy expressions
         The expressions to optimize.
     order : string, 'none' or 'canonical'
         The order by which Mul and Add arguments are processed. For large
@@ -411,13 +485,13 @@ def opt_cse(exprs, order='canonical'):
     >>> print((k, v.as_unevaluated_basic()))
     (x**(-2), 1/(x**2))
     """
-    from sympy.matrices.expressions import MatAdd, MatMul, MatPow
-    opt_subs = dict()
+    opt_subs = {}
 
     adds = OrderedSet()
     muls = OrderedSet()
 
     seen_subexp = set()
+    collapsible_subexp = set()
 
     def _find_opts(expr):
 
@@ -437,28 +511,51 @@ def opt_cse(exprs, order='canonical'):
 
         list(map(_find_opts, expr.args))
 
-        if _coeff_isneg(expr):
-            neg_expr = -expr
+        if not isinstance(expr, MatrixExpr) and expr.could_extract_minus_sign():
+            # XXX -expr does not always work rigorously for some expressions
+            # containing UnevaluatedExpr.
+            # https://github.com/sympy/sympy/issues/24818
+            if isinstance(expr, Add):
+                neg_expr = Add(*(-i for i in expr.args))
+            else:
+                neg_expr = -expr
+
             if not neg_expr.is_Atom:
                 opt_subs[expr] = Unevaluated(Mul, (S.NegativeOne, neg_expr))
                 seen_subexp.add(neg_expr)
                 expr = neg_expr
 
         if isinstance(expr, (Mul, MatMul)):
-            muls.add(expr)
+            if len(expr.args) == 1:
+                collapsible_subexp.add(expr)
+            else:
+                muls.add(expr)
 
         elif isinstance(expr, (Add, MatAdd)):
-            adds.add(expr)
+            if len(expr.args) == 1:
+                collapsible_subexp.add(expr)
+            else:
+                adds.add(expr)
+
+        elif isinstance(expr, Inverse):
+            # Do not want to treat `Inverse` as a `MatPow`
+            pass
 
         elif isinstance(expr, (Pow, MatPow)):
             base, exp = expr.base, expr.exp
-            if _coeff_isneg(exp):
+            if exp.could_extract_minus_sign():
                 opt_subs[expr] = Unevaluated(Pow, (Pow(base, -exp), -1))
 
     for e in exprs:
         if isinstance(e, (Basic, Unevaluated)):
             _find_opts(e)
 
+    # Handle collapsing of multinary operations with single arguments
+    edges = [(s, s.args[0]) for s in collapsible_subexp
+             if s.args[0] in collapsible_subexp]
+    for e in reversed(topological_sort((collapsible_subexp, edges))):
+        opt_subs[e] = opt_subs.get(e.args[0], e.args[0])
+
     # split muls into commutative
     commutative_muls = OrderedSet()
     for m in muls:
@@ -469,7 +566,10 @@ def opt_cse(exprs, order='canonical'):
                 if c_mul == 1:
                     new_obj = m.func(*nc)
                 else:
-                    new_obj = m.func(c_mul, m.func(*nc), evaluate=False)
+                    if isinstance(m, MatMul):
+                        new_obj = m.func(c_mul, *nc, evaluate=False)
+                    else:
+                        new_obj = m.func(c_mul, m.func(*nc), evaluate=False)
                 opt_subs[m] = new_obj
             if len(c) > 1:
                 commutative_muls.add(c_mul)
@@ -486,7 +586,7 @@ def tree_cse(exprs, symbols, opt_subs=None, order='canonical', ignore=()):
     Parameters
     ==========
 
-    exprs : list of sympy expressions
+    exprs : list of SymPy expressions
         The expressions to reduce.
     symbols : infinite iterator yielding unique Symbols
         The symbols used to label the common subexpressions which are pulled
@@ -499,11 +599,8 @@ def tree_cse(exprs, symbols, opt_subs=None, order='canonical', ignore=()):
     ignore : iterable of Symbols
         Substitutions containing any Symbol from ``ignore`` will be ignored.
     """
-    from sympy.matrices.expressions import MatrixExpr, MatrixSymbol, MatMul, MatAdd
-    from sympy.polys.rootoftools import RootOf
-
     if opt_subs is None:
-        opt_subs = dict()
+        opt_subs = {}
 
     ## Find repeated sub-expressions
 
@@ -519,7 +616,10 @@ def tree_cse(exprs, symbols, opt_subs=None, order='canonical', ignore=()):
         if isinstance(expr, RootOf):
             return
 
-        if isinstance(expr, Basic) and (expr.is_Atom or expr.is_Order):
+        if isinstance(expr, Basic) and (
+                expr.is_Atom or
+                expr.is_Order or
+                isinstance(expr, (MatrixSymbol, MatrixElement))):
             if expr.is_Symbol:
                 excluded_symbols.add(expr)
             return
@@ -556,7 +656,7 @@ def tree_cse(exprs, symbols, opt_subs=None, order='canonical', ignore=()):
 
     replacements = []
 
-    subs = dict()
+    subs = {}
 
     def _rebuild(expr):
         if not isinstance(expr, (Basic, Unevaluated)):
@@ -566,7 +666,7 @@ def tree_cse(exprs, symbols, opt_subs=None, order='canonical', ignore=()):
             return expr
 
         if iterable(expr):
-            new_args = [_rebuild(arg) for arg in expr]
+            new_args = [_rebuild(arg) for arg in expr.args]
             return expr.func(*new_args)
 
         if expr in subs:
@@ -626,13 +726,13 @@ def tree_cse(exprs, symbols, opt_subs=None, order='canonical', ignore=()):
 
 
 def cse(exprs, symbols=None, optimizations=None, postprocess=None,
-        order='canonical', ignore=()):
+        order='canonical', ignore=(), list=True):
     """ Perform common subexpression elimination on an expression.
 
     Parameters
     ==========
 
-    exprs : list of sympy expressions, or a single sympy expression
+    exprs : list of SymPy expressions, or a single SymPy expression
         The expressions to reduce.
     symbols : infinite iterator yielding unique Symbols
         The symbols used to label the common subexpressions which are pulled
@@ -657,6 +757,8 @@ def cse(exprs, symbols=None, optimizations=None, postprocess=None,
         concern, use the setting order='none'.
     ignore : iterable of Symbols
         Substitutions containing any Symbol from ``ignore`` will be ignored.
+    list : bool, (default True)
+        Returns expression in list or else with same type as input (when False).
 
     Returns
     =======
@@ -665,7 +767,7 @@ def cse(exprs, symbols=None, optimizations=None, postprocess=None,
         All of the common subexpressions that were replaced. Subexpressions
         earlier in this list might show up in subexpressions later in this
         list.
-    reduced_exprs : list of sympy expressions
+    reduced_exprs : list of SymPy expressions
         The reduced expressions with all of the replacements above.
 
     Examples
@@ -676,10 +778,6 @@ def cse(exprs, symbols=None, optimizations=None, postprocess=None,
     >>> cse(((w + x + y + z)*(w + y + z))/(w + x)**3)
     ([(x0, y + z), (x1, w + x)], [(w + x0)*(x0 + x1)/x1**3])
 
-    Note that currently, y + z will not get substituted if -y - z is used.
-
-     >>> cse(((w + x + y + z)*(w - y - z))/(w + x)**3)
-     ([(x0, w + x)], [(w - y - z)*(x0 + y + z)/x0**3])
 
     List of expressions with recursive substitutions:
 
@@ -699,9 +797,18 @@ def cse(exprs, symbols=None, optimizations=None, postprocess=None,
     >>> cse([y**2*(x + 1), 3*y**2*(x + 1)], ignore=(y,))
     ([(x0, x + 1)], [x0*y**2, 3*x0*y**2])
 
+    The default return value for the reduced expression(s) is a list, even if there is only
+    one expression. The `list` flag preserves the type of the input in the output:
+
+    >>> cse(x)
+    ([], [x])
+    >>> cse(x, list=False)
+    ([], x)
     """
-    from sympy.matrices import (MatrixBase, Matrix, ImmutableMatrix,
-                                SparseMatrix, ImmutableSparseMatrix)
+    if not list:
+        return _cse_homogeneous(exprs,
+            symbols=symbols, optimizations=optimizations,
+            postprocess=postprocess, order=order, ignore=ignore)
 
     if isinstance(exprs, (int, float)):
         exprs = sympify(exprs)
@@ -714,16 +821,16 @@ def cse(exprs, symbols=None, optimizations=None, postprocess=None,
     temp = []
     for e in exprs:
         if isinstance(e, (Matrix, ImmutableMatrix)):
-            temp.append(Tuple(*e._mat))
+            temp.append(Tuple(*e.flat()))
         elif isinstance(e, (SparseMatrix, ImmutableSparseMatrix)):
-            temp.append(Tuple(*e._smat.items()))
+            temp.append(Tuple(*e.todok().items()))
         else:
             temp.append(e)
     exprs = temp
     del temp
 
     if optimizations is None:
-        optimizations = list()
+        optimizations = []
     elif optimizations == 'basic':
         optimizations = basic_optimizations
 
@@ -770,3 +877,70 @@ def cse(exprs, symbols=None, optimizations=None, postprocess=None,
         return replacements, reduced_exprs
 
     return postprocess(replacements, reduced_exprs)
+
+
+def _cse_homogeneous(exprs, **kwargs):
+    """
+    Same as ``cse`` but the ``reduced_exprs`` are returned
+    with the same type as ``exprs`` or a sympified version of the same.
+
+    Parameters
+    ==========
+
+    exprs : an Expr, iterable of Expr or dictionary with Expr values
+        the expressions in which repeated subexpressions will be identified
+    kwargs : additional arguments for the ``cse`` function
+
+    Returns
+    =======
+
+    replacements : list of (Symbol, expression) pairs
+        All of the common subexpressions that were replaced. Subexpressions
+        earlier in this list might show up in subexpressions later in this
+        list.
+    reduced_exprs : list of SymPy expressions
+        The reduced expressions with all of the replacements above.
+
+    Examples
+    ========
+
+    >>> from sympy.simplify.cse_main import cse
+    >>> from sympy import cos, Tuple, Matrix
+    >>> from sympy.abc import x
+    >>> output = lambda x: type(cse(x, list=False)[1])
+    >>> output(1)
+    <class 'sympy.core.numbers.One'>
+    >>> output('cos(x)')
+    <class 'str'>
+    >>> output(cos(x))
+    cos
+    >>> output(Tuple(1, x))
+    <class 'sympy.core.containers.Tuple'>
+    >>> output(Matrix([[1,0], [0,1]]))
+    <class 'sympy.matrices.dense.MutableDenseMatrix'>
+    >>> output([1, x])
+    <class 'list'>
+    >>> output((1, x))
+    <class 'tuple'>
+    >>> output({1, x})
+    <class 'set'>
+    """
+    if isinstance(exprs, str):
+        replacements, reduced_exprs = _cse_homogeneous(
+            sympify(exprs), **kwargs)
+        return replacements, repr(reduced_exprs)
+    if isinstance(exprs, (list, tuple, set)):
+        replacements, reduced_exprs = cse(exprs, **kwargs)
+        return replacements, type(exprs)(reduced_exprs)
+    if isinstance(exprs, dict):
+        keys = list(exprs.keys()) # In order to guarantee the order of the elements.
+        replacements, values = cse([exprs[k] for k in keys], **kwargs)
+        reduced_exprs = dict(zip(keys, values))
+        return replacements, reduced_exprs
+
+    try:
+        replacements, (reduced_exprs,) = cse(exprs, **kwargs)
+    except TypeError: # For example 'mpf' objects
+        return [], exprs
+    else:
+        return replacements, reduced_exprs
