@@ -12,16 +12,31 @@ Binomial
 BetaBinomial
 Hypergeometric
 Rademacher
+IdealSoliton
+RobustSoliton
 """
 
 
-from sympy import (S, sympify, Rational, binomial, cacheit, Integer,
-                   Dummy, Eq, Intersection, Interval,
-                   Symbol, Lambda, Piecewise, Or, Gt, Lt, Ge, Le, Contains)
-from sympy import beta as beta_fn
+from sympy.core.cache import cacheit
+from sympy.core.function import Lambda
+from sympy.core.numbers import (Integer, Rational)
+from sympy.core.relational import (Eq, Ge, Gt, Le, Lt)
+from sympy.core.singleton import S
+from sympy.core.symbol import (Dummy, Symbol)
+from sympy.core.sympify import sympify
+from sympy.functions.combinatorial.factorials import binomial
+from sympy.functions.elementary.exponential import log
+from sympy.functions.elementary.piecewise import Piecewise
+from sympy.logic.boolalg import Or
+from sympy.sets.contains import Contains
+from sympy.sets.fancysets import Range
+from sympy.sets.sets import (Intersection, Interval)
+from sympy.functions.special.beta_functions import beta as beta_fn
 from sympy.stats.frv import (SingleFiniteDistribution,
                              SingleFinitePSpace)
 from sympy.stats.rv import _value_check, Density, is_random
+from sympy.utilities.iterables import multiset
+from sympy.utilities.misc import filldedent
 
 
 __all__ = ['FiniteRV',
@@ -32,7 +47,9 @@ __all__ = ['FiniteRV',
 'Binomial',
 'BetaBinomial',
 'Hypergeometric',
-'Rademacher'
+'Rademacher',
+'IdealSoliton',
+'RobustSoliton',
 ]
 
 def rv(name, cls, *args, **kwargs):
@@ -78,8 +95,8 @@ def FiniteRV(name, density, **kwargs):
 
     name : Symbol
         Represents name of the random variable.
-    density: A dict
-        Dictionary conatining the pdf of finite distribution
+    density : dict
+        Dictionary containing the pdf of finite distribution
     check : bool
         If True, it will check whether the given density
         integrates to 1 over the given set. If False, it
@@ -115,8 +132,6 @@ class DiscreteUniformDistribution(SingleFiniteDistribution):
         # not using _value_check since there is a
         # suggestion for the user
         if len(set(args)) != len(args):
-            from sympy.utilities.iterables import multiset
-            from sympy.utilities.misc import filldedent
             weights = multiset(args)
             n = Integer(len(args))
             for k in weights:
@@ -155,7 +170,7 @@ def DiscreteUniform(name, items):
     Parameters
     ==========
 
-    items: list/tuple
+    items : list/tuple
         Items over which Uniform distribution is to be made
 
     Examples
@@ -181,7 +196,7 @@ def DiscreteUniform(name, items):
     ==========
 
     .. [1] https://en.wikipedia.org/wiki/Discrete_uniform_distribution
-    .. [2] http://mathworld.wolfram.com/DiscreteUniformDistribution.html
+    .. [2] https://mathworld.wolfram.com/DiscreteUniformDistribution.html
 
     """
     return rv(name, DiscreteUniformDistribution, *items)
@@ -211,12 +226,12 @@ class DieDistribution(SingleFiniteDistribution):
     def set(self):
         if self.is_symbolic:
             return Intersection(S.Naturals0, Interval(0, self.sides))
-        return set(map(Integer, list(range(1, self.sides + 1))))
+        return set(map(Integer, range(1, self.sides + 1)))
 
     def pmf(self, x):
         x = sympify(x)
         if not (x.is_number or x.is_Symbol or is_random(x)):
-            raise ValueError("'x' expected as an argument of type 'number' or 'Symbol' or , "
+            raise ValueError("'x' expected as an argument of type 'number', 'Symbol', or "
                         "'RandomSymbol' not %s" % (type(x)))
         cond = Ge(x, 1) & Le(x, self.sides) & Contains(x, S.Integers)
         return Piecewise((S.One/self.sides, cond), (S.Zero, True))
@@ -228,7 +243,7 @@ def Die(name, sides=6):
     Parameters
     ==========
 
-    sides: Integer
+    sides : Integer
         Represents the number of sides of the Die, by default is 6
 
     Examples
@@ -320,7 +335,7 @@ def Bernoulli(name, p, succ=1, fail=0):
     ==========
 
     .. [1] https://en.wikipedia.org/wiki/Bernoulli_distribution
-    .. [2] http://mathworld.wolfram.com/BernoulliDistribution.html
+    .. [2] https://mathworld.wolfram.com/BernoulliDistribution.html
 
     """
 
@@ -334,7 +349,7 @@ def Coin(name, p=S.Half):
     Parameters
     ==========
 
-    p : Rational Numeber between 0 and 1
+    p : Rational Number between 0 and 1
       Represents probability of getting "Heads", by default is Half
 
     Examples
@@ -402,7 +417,7 @@ class BinomialDistribution(SingleFiniteDistribution):
         n, p = self.n, self.p
         x = sympify(x)
         if not (x.is_number or x.is_Symbol or is_random(x)):
-            raise ValueError("'x' expected as an argument of type 'number' or 'Symbol' or , "
+            raise ValueError("'x' expected as an argument of type 'number', 'Symbol', or "
                         "'RandomSymbol' not %s" % (type(x)))
         cond = Ge(x, 0) & Le(x, n) & Contains(x, S.Integers)
         return Piecewise((binomial(n, x) * p**x * (1 - p)**(n - x), cond), (S.Zero, True))
@@ -415,6 +430,7 @@ class BinomialDistribution(SingleFiniteDistribution):
         return {k*self.succ + (self.n-k)*self.fail: self.pmf(k)
                     for k in range(0, self.n + 1)}
 
+
 def Binomial(name, n, p, succ=1, fail=0):
     r"""
     Create a Finite Random Variable representing a binomial distribution.
@@ -458,7 +474,7 @@ def Binomial(name, n, p, succ=1, fail=0):
     ==========
 
     .. [1] https://en.wikipedia.org/wiki/Binomial_distribution
-    .. [2] http://mathworld.wolfram.com/BinomialDistribution.html
+    .. [2] https://mathworld.wolfram.com/BinomialDistribution.html
 
     """
 
@@ -495,7 +511,7 @@ class BetaBinomialDistribution(SingleFiniteDistribution):
     def set(self):
         if self.is_symbolic:
             return Intersection(S.Naturals0, Interval(0, self.n))
-        return set(map(Integer, list(range(0, self.n + 1))))
+        return set(map(Integer, range(self.n + 1)))
 
     def pmf(self, k):
         n, a, b = self.n, self.alpha, self.beta
@@ -532,7 +548,7 @@ def BetaBinomial(name, n, alpha, beta):
     ==========
 
     .. [1] https://en.wikipedia.org/wiki/Beta-binomial_distribution
-    .. [2] http://mathworld.wolfram.com/BetaBinomialDistribution.html
+    .. [2] https://mathworld.wolfram.com/BetaBinomialDistribution.html
 
     """
 
@@ -553,7 +569,7 @@ class HypergeometricDistribution(SingleFiniteDistribution):
 
     @property
     def is_symbolic(self):
-        return any(not x.is_number for x in (self.N, self.m, self.n))
+        return not all(x.is_number for x in (self.N, self.m, self.n))
 
     @property
     def high(self):
@@ -568,7 +584,7 @@ class HypergeometricDistribution(SingleFiniteDistribution):
         N, m, n = self.N, self.m, self.n
         if self.is_symbolic:
             return Intersection(S.Naturals0, Interval(self.low, self.high))
-        return {i for i in range(max(0, n + m - N), min(n, m) + 1)}
+        return set(range(max(0, n + m - N), min(n, m) + 1))
 
     def pmf(self, k):
         N, m, n = self.N, self.m, self.n
@@ -608,7 +624,7 @@ def Hypergeometric(name, N, m, n):
     ==========
 
     .. [1] https://en.wikipedia.org/wiki/Hypergeometric_distribution
-    .. [2] http://mathworld.wolfram.com/HypergeometricDistribution.html
+    .. [2] https://mathworld.wolfram.com/HypergeometricDistribution.html
 
     """
     return rv(name, HypergeometricDistribution, N, m, n)
@@ -655,3 +671,200 @@ def Rademacher(name):
 
     """
     return rv(name, RademacherDistribution)
+
+class IdealSolitonDistribution(SingleFiniteDistribution):
+    _argnames = ('k',)
+
+    @staticmethod
+    def check(k):
+         _value_check(k.is_integer and k.is_positive,
+                    "'k' must be a positive integer.")
+
+    @property
+    def low(self):
+        return S.One
+
+    @property
+    def high(self):
+        return self.k
+
+    @property
+    def set(self):
+        return set(map(Integer, range(1, self.k + 1)))
+
+    @property # type: ignore
+    @cacheit
+    def dict(self):
+        if self.k.is_Symbol:
+            return Density(self)
+        d = {1: Rational(1, self.k)}
+        d.update({i: Rational(1, i*(i - 1)) for i in range(2, self.k + 1)})
+        return d
+
+    def pmf(self, x):
+        x = sympify(x)
+        if not (x.is_number or x.is_Symbol or is_random(x)):
+            raise ValueError("'x' expected as an argument of type 'number', 'Symbol', or "
+                        "'RandomSymbol' not %s" % (type(x)))
+        cond1 = Eq(x, 1) & x.is_integer
+        cond2 = Ge(x, 1) & Le(x, self.k) & x.is_integer
+        return Piecewise((1/self.k, cond1), (1/(x*(x - 1)), cond2), (S.Zero, True))
+
+def IdealSoliton(name, k):
+    r"""
+    Create a Finite Random Variable of Ideal Soliton Distribution
+
+    Parameters
+    ==========
+
+    k : Positive Integer
+        Represents the number of input symbols in an LT (Luby Transform) code.
+
+    Examples
+    ========
+
+    >>> from sympy.stats import IdealSoliton, density, P, E
+    >>> sol = IdealSoliton('sol', 5)
+    >>> density(sol).dict
+    {1: 1/5, 2: 1/2, 3: 1/6, 4: 1/12, 5: 1/20}
+    >>> density(sol).set
+    {1, 2, 3, 4, 5}
+
+    >>> from sympy import Symbol
+    >>> k = Symbol('k', positive=True, integer=True)
+    >>> sol = IdealSoliton('sol', k)
+    >>> density(sol).dict
+    Density(IdealSolitonDistribution(k))
+    >>> density(sol).dict.subs(k, 10).doit()
+    {1: 1/10, 2: 1/2, 3: 1/6, 4: 1/12, 5: 1/20, 6: 1/30, 7: 1/42, 8: 1/56, 9: 1/72, 10: 1/90}
+
+    >>> E(sol.subs(k, 10))
+    7381/2520
+
+    >>> P(sol.subs(k, 4) > 2)
+    1/4
+
+    Returns
+    =======
+
+    RandomSymbol
+
+    References
+    ==========
+
+    .. [1] https://en.wikipedia.org/wiki/Soliton_distribution#Ideal_distribution
+    .. [2] https://pages.cs.wisc.edu/~suman/courses/740/papers/luby02lt.pdf
+
+    """
+    return rv(name, IdealSolitonDistribution, k)
+
+class RobustSolitonDistribution(SingleFiniteDistribution):
+    _argnames= ('k', 'delta', 'c')
+
+    @staticmethod
+    def check(k, delta, c):
+        _value_check(k.is_integer and k.is_positive,
+                    "'k' must be a positive integer")
+        _value_check(Gt(delta, 0) and Le(delta, 1),
+                    "'delta' must be a real number in the interval (0,1)")
+        _value_check(c.is_positive,
+                    "'c' must be a positive real number.")
+
+    @property
+    def R(self):
+        return self.c * log(self.k/self.delta) * self.k**0.5
+
+    @property
+    def Z(self):
+        z = 0
+        for i in Range(1, round(self.k/self.R)):
+            z += (1/i)
+        z += log(self.R/self.delta)
+        return 1 + z * self.R/self.k
+
+    @property
+    def low(self):
+        return S.One
+
+    @property
+    def high(self):
+        return self.k
+
+    @property
+    def set(self):
+        return set(map(Integer, range(1, self.k + 1)))
+
+    @property
+    def is_symbolic(self):
+        return not (self.k.is_number and self.c.is_number and self.delta.is_number)
+
+    def pmf(self, x):
+        x = sympify(x)
+        if not (x.is_number or x.is_Symbol or is_random(x)):
+            raise ValueError("'x' expected as an argument of type 'number', 'Symbol', or "
+                        "'RandomSymbol' not %s" % (type(x)))
+
+        cond1 = Eq(x, 1) & x.is_integer
+        cond2 = Ge(x, 1) & Le(x, self.k) & x.is_integer
+        rho = Piecewise((Rational(1, self.k), cond1), (Rational(1, x*(x-1)), cond2), (S.Zero, True))
+
+        cond1 = Ge(x, 1) & Le(x, round(self.k/self.R)-1)
+        cond2 = Eq(x, round(self.k/self.R))
+        tau = Piecewise((self.R/(self.k * x), cond1), (self.R * log(self.R/self.delta)/self.k, cond2), (S.Zero, True))
+
+        return (rho + tau)/self.Z
+
+def RobustSoliton(name, k, delta, c):
+    r'''
+    Create a Finite Random Variable of Robust Soliton Distribution
+
+    Parameters
+    ==========
+
+    k : Positive Integer
+        Represents the number of input symbols in an LT (Luby Transform) code.
+    delta : Positive Rational Number
+            Represents the failure probability. Must be in the interval (0,1).
+    c : Positive Rational Number
+        Constant of proportionality. Values close to 1 are recommended
+
+    Examples
+    ========
+
+    >>> from sympy.stats import RobustSoliton, density, P, E
+    >>> robSol = RobustSoliton('robSol', 5, 0.5, 0.01)
+    >>> density(robSol).dict
+    {1: 0.204253668152708, 2: 0.490631107897393, 3: 0.165210624506162, 4: 0.0834387731899302, 5: 0.0505633404760675}
+    >>> density(robSol).set
+    {1, 2, 3, 4, 5}
+
+    >>> from sympy import Symbol
+    >>> k = Symbol('k', positive=True, integer=True)
+    >>> c = Symbol('c', positive=True)
+    >>> robSol = RobustSoliton('robSol', k, 0.5, c)
+    >>> density(robSol).dict
+    Density(RobustSolitonDistribution(k, 0.5, c))
+    >>> density(robSol).dict.subs(k, 10).subs(c, 0.03).doit()
+    {1: 0.116641095387194, 2: 0.467045731687165, 3: 0.159984123349381, 4: 0.0821431680681869, 5: 0.0505765646770100,
+    6: 0.0345781523420719, 7: 0.0253132820710503, 8: 0.0194459129233227, 9: 0.0154831166726115, 10: 0.0126733075238887}
+
+    >>> E(robSol.subs(k, 10).subs(c, 0.05))
+    2.91358846104106
+
+    >>> P(robSol.subs(k, 4).subs(c, 0.1) > 2)
+    0.243650614389834
+
+    Returns
+    =======
+
+    RandomSymbol
+
+    References
+    ==========
+
+    .. [1] https://en.wikipedia.org/wiki/Soliton_distribution#Robust_distribution
+    .. [2] https://www.inference.org.uk/mackay/itprnn/ps/588.596.pdf
+    .. [3] https://pages.cs.wisc.edu/~suman/courses/740/papers/luby02lt.pdf
+
+    '''
+    return rv(name, RobustSolitonDistribution, k, delta, c)
