@@ -1,16 +1,19 @@
-from __future__ import print_function, division
 import functools, itertools
-from sympy.core.sympify import sympify
+from sympy.core.sympify import _sympify, sympify
 from sympy.core.expr import Expr
-from sympy.core import Basic
+from sympy.core import Basic, Tuple
 from sympy.tensor.array import ImmutableDenseNDimArray
-from sympy import Symbol
+from sympy.core.symbol import Symbol
 from sympy.core.numbers import Integer
 
 
 class ArrayComprehension(Basic):
     """
-    Generate a list comprehension
+    Generate a list comprehension.
+
+    Explanation
+    ===========
+
     If there is a symbolic dimension, for example, say [i for i in range(1, N)] where
     N is a Symbol, then the expression will not be expanded to an array. Otherwise,
     calling the doit() function will launch the expansion.
@@ -45,7 +48,7 @@ class ArrayComprehension(Basic):
 
     @property
     def function(self):
-        """The function applied across limits
+        """The function applied across limits.
 
         Examples
         ========
@@ -62,7 +65,7 @@ class ArrayComprehension(Basic):
     @property
     def limits(self):
         """
-        The list of limits that will be applied while expanding the array
+        The list of limits that will be applied while expanding the array.
 
         Examples
         ========
@@ -79,7 +82,7 @@ class ArrayComprehension(Basic):
     @property
     def free_symbols(self):
         """
-        The set of the free_symbols in the array
+        The set of the free_symbols in the array.
         Variables appeared in the bounds are supposed to be excluded
         from the free symbol set.
 
@@ -105,7 +108,7 @@ class ArrayComprehension(Basic):
 
     @property
     def variables(self):
-        """The tuples of the variables in the limits
+        """The tuples of the variables in the limits.
 
         Examples
         ========
@@ -121,7 +124,7 @@ class ArrayComprehension(Basic):
 
     @property
     def bound_symbols(self):
-        """The list of dummy variables
+        """The list of dummy variables.
 
         Note
         ====
@@ -134,7 +137,7 @@ class ArrayComprehension(Basic):
     @property
     def shape(self):
         """
-        The shape of the expanded array, which may have symbols
+        The shape of the expanded array, which may have symbols.
 
         Note
         ====
@@ -161,7 +164,7 @@ class ArrayComprehension(Basic):
     def is_shape_numeric(self):
         """
         Test if the array is shape-numeric which means there is no symbolic
-        dimension
+        dimension.
 
         Examples
         ========
@@ -182,7 +185,7 @@ class ArrayComprehension(Basic):
         return True
 
     def rank(self):
-        """The rank of the expanded array
+        """The rank of the expanded array.
 
         Examples
         ========
@@ -222,8 +225,18 @@ class ArrayComprehension(Basic):
 
     @classmethod
     def _check_limits_validity(cls, function, limits):
-        limits = sympify(limits)
+        #limits = sympify(limits)
+        new_limits = []
         for var, inf, sup in limits:
+            var = _sympify(var)
+            inf = _sympify(inf)
+            #since this is stored as an argument, it should be
+            #a Tuple
+            if isinstance(sup, list):
+                sup = Tuple(*sup)
+            else:
+                sup = _sympify(sup)
+            new_limits.append(Tuple(var, inf, sup))
             if any((not isinstance(i, Expr)) or i.atoms(Symbol, Integer) != i.atoms()
                                                                 for i in [inf, sup]):
                 raise TypeError('Bounds should be an Expression(combination of Integer and Symbol)')
@@ -231,7 +244,7 @@ class ArrayComprehension(Basic):
                 raise ValueError('Lower bound should be inferior to upper bound')
             if var in inf.free_symbols or var in sup.free_symbols:
                 raise ValueError('Variable should not be part of its bounds')
-        return limits
+        return new_limits
 
     @classmethod
     def _calculate_shape_from_limits(cls, limits):
@@ -247,7 +260,7 @@ class ArrayComprehension(Basic):
 
         return loop_size
 
-    def doit(self):
+    def doit(self, **hints):
         if not self.is_shape_numeric:
             return self
 
@@ -269,7 +282,7 @@ class ArrayComprehension(Basic):
         return temp
 
     def tolist(self):
-        """Transform the expanded array to a list
+        """Transform the expanded array to a list.
 
         Raises
         ======
@@ -292,7 +305,7 @@ class ArrayComprehension(Basic):
         raise ValueError("A symbolic array cannot be expanded to a list")
 
     def tomatrix(self):
-        """Transform the expanded array to a matrix
+        """Transform the expanded array to a matrix.
 
         Raises
         ======
