@@ -1,7 +1,7 @@
 from sympy.combinatorics.free_groups import free_group, FreeGroup
 from sympy.core import Symbol
 from sympy.testing.pytest import raises
-from sympy import oo
+from sympy.core.numbers import oo
 
 F, x, y, z = free_group("x, y, z")
 
