@@ -1,17 +1,17 @@
-from typing import List
+from __future__ import annotations
+from functools import reduce
 
 from sympy.core import S, sympify, Dummy, Mod
 from sympy.core.cache import cacheit
-from sympy.core.compatibility import reduce, HAS_GMPY
-from sympy.core.function import Function, ArgumentIndexError
+from sympy.core.function import Function, ArgumentIndexError, PoleError
 from sympy.core.logic import fuzzy_and
-from sympy.core.numbers import Integer, pi
+from sympy.core.numbers import Integer, pi, I
 from sympy.core.relational import Eq
+from sympy.external.gmpy import HAS_GMPY, gmpy
 from sympy.ntheory import sieve
 from sympy.polys.polytools import Poly
 
-from math import sqrt as _sqrt
-
+from math import factorial as _factorial, prod, sqrt as _sqrt
 
 class CombinatorialFunction(Function):
     """Base class for combinatorial functions. """
@@ -82,7 +82,7 @@ class factorial(CombinatorialFunction):
     """
 
     def fdiff(self, argindex=1):
-        from sympy import gamma, polygamma
+        from sympy.functions.special.gamma_functions import (gamma, polygamma)
         if argindex == 1:
             return gamma(self.args[0] + 1)*polygamma(0, self.args[0] + 1)
         else:
@@ -94,7 +94,7 @@ class factorial(CombinatorialFunction):
         35102025, 5014575, 145422675, 9694845, 300540195, 300540195
     ]
 
-    _small_factorials = []  # type: List[int]
+    _small_factorials: list[int] = []
 
     @classmethod
     def _swing(cls, n):
@@ -122,13 +122,8 @@ class factorial(CombinatorialFunction):
                 if (n // prime) & 1 == 1:
                     primes.append(prime)
 
-            L_product = R_product = 1
-
-            for prime in sieve.primerange(n//2 + 1, n + 1):
-                L_product *= prime
-
-            for prime in primes:
-                R_product *= prime
+            L_product = prod(sieve.primerange(n//2 + 1, n + 1))
+            R_product = prod(primes)
 
             return L_product*R_product
 
@@ -164,7 +159,6 @@ class factorial(CombinatorialFunction):
 
                     # GMPY factorial is faster, use it when available
                     elif HAS_GMPY:
-                        from sympy.core.compatibility import gmpy
                         result = gmpy.fac(n)
 
                     else:
@@ -218,7 +212,7 @@ class factorial(CombinatorialFunction):
                     # its inverse (if n > 4 is a composite number, then
                     # (n-1)! = 0 mod n)
                     if isprime:
-                        return S(-1 % q)
+                        return -1 % q
                     elif isprime is False and (aq - 6).is_nonnegative:
                         return S.Zero
                 elif n.is_Integer and q.is_Integer:
@@ -231,14 +225,14 @@ class factorial(CombinatorialFunction):
                     else:
                         fc = self._facmod(n, aq)
 
-                    return S(fc % q)
+                    return fc % q
 
     def _eval_rewrite_as_gamma(self, n, piecewise=True, **kwargs):
-        from sympy import gamma
+        from sympy.functions.special.gamma_functions import gamma
         return gamma(n + 1)
 
     def _eval_rewrite_as_Product(self, n, **kwargs):
-        from sympy import Product
+        from sympy.concrete.products import Product
         if n.is_nonnegative and n.is_integer:
             i = Dummy('i', integer=True)
             return Product(i, (i, 1, n))
@@ -266,31 +260,21 @@ class factorial(CombinatorialFunction):
         if x.is_nonnegative or x.is_noninteger:
             return True
 
-    def _eval_as_leading_term(self, x, cdir=0):
-        from sympy import Order
-        arg = self.args[0]
-        arg_1 = arg.as_leading_term(x)
-        if Order(x, x).contains(arg_1):
+    def _eval_as_leading_term(self, x, logx=None, cdir=0):
+        arg = self.args[0].as_leading_term(x)
+        arg0 = arg.subs(x, 0)
+        if arg0.is_zero:
             return S.One
-        if Order(1, x).contains(arg_1):
-            return self.func(arg_1)
-        ####################################################
-        # The correct result here should be 'None'.        #
-        # Indeed arg in not bounded as x tends to 0.       #
-        # Consequently the series expansion does not admit #
-        # the leading term.                                #
-        # For compatibility reasons, the return value here #
-        # is the original function, i.e. factorial(arg),   #
-        # instead of None.                                 #
-        ####################################################
-        return self.func(arg)
+        elif not arg0.is_infinite:
+            return self.func(arg)
+        raise PoleError("Cannot expand %s around 0" % (self))
 
 class MultiFactorial(CombinatorialFunction):
     pass
 
 
 class subfactorial(CombinatorialFunction):
-    r"""The subfactorial counts the derangements of n items and is
+    r"""The subfactorial counts the derangements of $n$ items and is
     defined for non-negative integers as:
 
     .. math:: !n = \begin{cases} 1 & n = 0 \\ 0 & n = 1 \\
@@ -304,7 +288,7 @@ class subfactorial(CombinatorialFunction):
     .. math:: !x = \Gamma(x + 1, -1)/e
 
     which is valid for non-negative integers `x`. The above formula
-    is not very useful incase of non-integers. :math:`\Gamma(x + 1, -1)` is
+    is not very useful in case of non-integers. `\Gamma(x + 1, -1)` is
     single-valued only for integral arguments `x`, elsewhere on the positive
     real axis it has an infinite number of branches none of which are real.
 
@@ -312,7 +296,7 @@ class subfactorial(CombinatorialFunction):
     ==========
 
     .. [1] https://en.wikipedia.org/wiki/Subfactorial
-    .. [2] http://mathworld.wolfram.com/Subfactorial.html
+    .. [2] https://mathworld.wolfram.com/Subfactorial.html
 
     Examples
     ========
@@ -327,9 +311,8 @@ class subfactorial(CombinatorialFunction):
     See Also
     ========
 
-    sympy.functions.combinatorial.factorials.factorial,
-    sympy.utilities.iterables.generate_derangements,
-    sympy.functions.special.gamma_functions.uppergamma
+    factorial, uppergamma,
+    sympy.utilities.iterables.generate_derangements
     """
 
     @classmethod
@@ -364,17 +347,19 @@ class subfactorial(CombinatorialFunction):
             return True
 
     def _eval_rewrite_as_factorial(self, arg, **kwargs):
-        from sympy import summation
+        from sympy.concrete.summations import summation
         i = Dummy('i')
         f = S.NegativeOne**i / factorial(i)
         return factorial(arg) * summation(f, (i, 0, arg))
 
     def _eval_rewrite_as_gamma(self, arg, piecewise=True, **kwargs):
-        from sympy import exp, gamma, I, lowergamma
-        return ((-1)**(arg + 1)*exp(-I*pi*arg)*lowergamma(arg + 1, -1) + gamma(arg + 1))*exp(-1)
+        from sympy.functions.elementary.exponential import exp
+        from sympy.functions.special.gamma_functions import (gamma, lowergamma)
+        return (S.NegativeOne**(arg + 1)*exp(-I*pi*arg)*lowergamma(arg + 1, -1)
+                + gamma(arg + 1))*exp(-1)
 
     def _eval_rewrite_as_uppergamma(self, arg, **kwargs):
-        from sympy import uppergamma
+        from sympy.functions.special.gamma_functions import uppergamma
         return uppergamma(arg + 1, -1)/S.Exp1
 
     def _eval_is_nonnegative(self):
@@ -494,7 +479,9 @@ class factorial2(CombinatorialFunction):
                 return ((n + 1) / 2).is_even
 
     def _eval_rewrite_as_gamma(self, n, piecewise=True, **kwargs):
-        from sympy import gamma, Piecewise, sqrt
+        from sympy.functions.elementary.miscellaneous import sqrt
+        from sympy.functions.elementary.piecewise import Piecewise
+        from sympy.functions.special.gamma_functions import gamma
         return 2**(n/2)*gamma(n/2 + 1) * Piecewise((1, Eq(Mod(n, 2), 0)),
                 (sqrt(2/pi), Eq(Mod(n, 2), 1)))
 
@@ -506,21 +493,19 @@ class factorial2(CombinatorialFunction):
 
 class RisingFactorial(CombinatorialFunction):
     r"""
-    Rising factorial (also called Pochhammer symbol) is a double valued
+    Rising factorial (also called Pochhammer symbol [1]_) is a double valued
     function arising in concrete mathematics, hypergeometric functions
     and series expansions. It is defined by:
 
-    .. math:: rf(x,k) = x \cdot (x+1) \cdots (x+k-1)
+    .. math:: \texttt{rf(y, k)} = (x)^k = x \cdot (x+1) \cdots (x+k-1)
 
     where `x` can be arbitrary expression and `k` is an integer. For
     more information check "Concrete mathematics" by Graham, pp. 66
-    or visit http://mathworld.wolfram.com/RisingFactorial.html page.
+    or visit https://mathworld.wolfram.com/RisingFactorial.html page.
 
-    When `x` is a Poly instance of degree >= 1 with a single variable,
-    `rf(x,k) = x(y) \cdot x(y+1) \cdots x(y+k-1)`, where `y` is the
-    variable of `x`. This is as described in Peter Paule, "Greatest
-    Factorial Factorization and Symbolic Summation", Journal of
-    Symbolic Computation, vol. 20, pp. 235-268, 1995.
+    When `x` is a `~.Poly` instance of degree $\ge 1$ with a single variable,
+    `(x)^k = x(y) \cdot x(y+1) \cdots x(y+k-1)`, where `y` is the
+    variable of `x`. This is as described in [2]_.
 
     Examples
     ========
@@ -538,7 +523,7 @@ class RisingFactorial(CombinatorialFunction):
 
     Rewriting is complicated unless the relationship between
     the arguments is known, but rising factorial can
-    be rewritten in terms of gamma, factorial and binomial
+    be rewritten in terms of gamma, factorial, binomial,
     and falling factorial.
 
     >>> from sympy import Symbol, factorial, ff, binomial, gamma
@@ -562,6 +547,9 @@ class RisingFactorial(CombinatorialFunction):
     ==========
 
     .. [1] https://en.wikipedia.org/wiki/Pochhammer_symbol
+    .. [2] Peter Paule, "Greatest Factorial Factorization and Symbolic
+           Summation", Journal of Symbolic Computation, vol. 20, pp. 235-268,
+           1995.
 
     """
 
@@ -595,10 +583,10 @@ class RisingFactorial(CombinatorialFunction):
                             else:
                                 return reduce(lambda r, i:
                                               r*(x.shift(i)),
-                                              range(0, int(k)), 1)
+                                              range(int(k)), 1)
                         else:
                             return reduce(lambda r, i: r*(x + i),
-                                        range(0, int(k)), 1)
+                                          range(int(k)), 1)
 
                 else:
                     if x is S.Infinity:
@@ -625,48 +613,44 @@ class RisingFactorial(CombinatorialFunction):
                 return S.Zero
 
     def _eval_rewrite_as_gamma(self, x, k, piecewise=True, **kwargs):
-        from sympy import gamma, Piecewise
+        from sympy.functions.elementary.piecewise import Piecewise
+        from sympy.functions.special.gamma_functions import gamma
         if not piecewise:
             if (x <= 0) == True:
-                return (-1)**k*gamma(1 - x) / gamma(-k - x + 1)
+                return S.NegativeOne**k*gamma(1 - x) / gamma(-k - x + 1)
             return gamma(x + k) / gamma(x)
         return Piecewise(
             (gamma(x + k) / gamma(x), x > 0),
-            ((-1)**k*gamma(1 - x) / gamma(-k - x + 1), True))
+            (S.NegativeOne**k*gamma(1 - x) / gamma(-k - x + 1), True))
 
     def _eval_rewrite_as_FallingFactorial(self, x, k, **kwargs):
         return FallingFactorial(x + k - 1, k)
 
     def _eval_rewrite_as_factorial(self, x, k, **kwargs):
-        from sympy import Piecewise
+        from sympy.functions.elementary.piecewise import Piecewise
         if x.is_integer and k.is_integer:
             return Piecewise(
                 (factorial(k + x - 1)/factorial(x - 1), x > 0),
-                ((-1)**k*factorial(-x)/factorial(-k - x), True))
+                (S.NegativeOne**k*factorial(-x)/factorial(-k - x), True))
 
     def _eval_rewrite_as_binomial(self, x, k, **kwargs):
         if k.is_integer:
             return factorial(k) * binomial(x + k - 1, k)
 
     def _eval_rewrite_as_tractable(self, x, k, limitvar=None, **kwargs):
-        from sympy import gamma
+        from sympy.functions.special.gamma_functions import gamma
         if limitvar:
             k_lim = k.subs(limitvar, S.Infinity)
             if k_lim is S.Infinity:
                 return (gamma(x + k).rewrite('tractable', deep=True) / gamma(x))
             elif k_lim is S.NegativeInfinity:
-                return ((-1)**k*gamma(1 - x) / gamma(-k - x + 1).rewrite('tractable', deep=True))
+                return (S.NegativeOne**k*gamma(1 - x) / gamma(-k - x + 1).rewrite('tractable', deep=True))
         return self.rewrite(gamma).rewrite('tractable', deep=True)
 
     def _eval_is_integer(self):
         return fuzzy_and((self.args[0].is_integer, self.args[1].is_integer,
                           self.args[1].is_nonnegative))
 
-    def _sage_(self):
-        import sage.all as sage
-        return sage.rising_factorial(self.args[0]._sage_(),
-                                     self.args[1]._sage_())
-
 
 class FallingFactorial(CombinatorialFunction):
     r"""
@@ -674,17 +658,15 @@ class FallingFactorial(CombinatorialFunction):
     function arising in concrete mathematics, hypergeometric functions
     and series expansions. It is defined by
 
-    .. math:: ff(x,k) = x \cdot (x-1) \cdots (x-k+1)
+    .. math:: \texttt{ff(x, k)} = (x)_k = x \cdot (x-1) \cdots (x-k+1)
 
     where `x` can be arbitrary expression and `k` is an integer. For
     more information check "Concrete mathematics" by Graham, pp. 66
-    or visit http://mathworld.wolfram.com/FallingFactorial.html page.
+    or [1]_.
 
-    When `x` is a Poly instance of degree >= 1 with single variable,
-    `ff(x,k) = x(y) \cdot x(y-1) \cdots x(y-k+1)`, where `y` is the
-    variable of `x`. This is as described in Peter Paule, "Greatest
-    Factorial Factorization and Symbolic Summation", Journal of
-    Symbolic Computation, vol. 20, pp. 235-268, 1995.
+    When `x` is a `~.Poly` instance of degree $\ge 1$ with single variable,
+    `(x)_k = x(y) \cdot x(y-1) \cdots x(y-k+1)`, where `y` is the
+    variable of `x`. This is as described in
 
     >>> from sympy import ff, Poly, Symbol
     >>> from sympy.abc import x
@@ -726,7 +708,10 @@ class FallingFactorial(CombinatorialFunction):
     References
     ==========
 
-    .. [1] http://mathworld.wolfram.com/FallingFactorial.html
+    .. [1] https://mathworld.wolfram.com/FallingFactorial.html
+    .. [2] Peter Paule, "Greatest Factorial Factorization and Symbolic
+           Summation", Journal of Symbolic Computation, vol. 20, pp. 235-268,
+           1995.
 
     """
 
@@ -760,10 +745,10 @@ class FallingFactorial(CombinatorialFunction):
                             else:
                                 return reduce(lambda r, i:
                                               r*(x.shift(-i)),
-                                              range(0, int(k)), 1)
+                                              range(int(k)), 1)
                         else:
                             return reduce(lambda r, i: r*(x - i),
-                                          range(0, int(k)), 1)
+                                          range(int(k)), 1)
                 else:
                     if x is S.Infinity:
                         return S.Infinity
@@ -784,14 +769,15 @@ class FallingFactorial(CombinatorialFunction):
                                             range(1, abs(int(k)) + 1), 1)
 
     def _eval_rewrite_as_gamma(self, x, k, piecewise=True, **kwargs):
-        from sympy import gamma, Piecewise
+        from sympy.functions.elementary.piecewise import Piecewise
+        from sympy.functions.special.gamma_functions import gamma
         if not piecewise:
             if (x < 0) == True:
-                return (-1)**k*gamma(k - x) / gamma(-x)
+                return S.NegativeOne**k*gamma(k - x) / gamma(-x)
             return gamma(x + 1) / gamma(x - k + 1)
         return Piecewise(
             (gamma(x + 1) / gamma(x - k + 1), x >= 0),
-            ((-1)**k*gamma(k - x) / gamma(-x), True))
+            (S.NegativeOne**k*gamma(k - x) / gamma(-x), True))
 
     def _eval_rewrite_as_RisingFactorial(self, x, k, **kwargs):
         return rf(x - k + 1, k)
@@ -801,18 +787,18 @@ class FallingFactorial(CombinatorialFunction):
             return factorial(k) * binomial(x, k)
 
     def _eval_rewrite_as_factorial(self, x, k, **kwargs):
-        from sympy import Piecewise
+        from sympy.functions.elementary.piecewise import Piecewise
         if x.is_integer and k.is_integer:
             return Piecewise(
                 (factorial(x)/factorial(-k + x), x >= 0),
-                ((-1)**k*factorial(k - x - 1)/factorial(-x - 1), True))
+                (S.NegativeOne**k*factorial(k - x - 1)/factorial(-x - 1), True))
 
     def _eval_rewrite_as_tractable(self, x, k, limitvar=None, **kwargs):
-        from sympy import gamma
+        from sympy.functions.special.gamma_functions import gamma
         if limitvar:
             k_lim = k.subs(limitvar, S.Infinity)
             if k_lim is S.Infinity:
-                return ((-1)**k*gamma(k - x).rewrite('tractable', deep=True) / gamma(-x))
+                return (S.NegativeOne**k*gamma(k - x).rewrite('tractable', deep=True) / gamma(-x))
             elif k_lim is S.NegativeInfinity:
                 return (gamma(x + 1) / gamma(x - k + 1).rewrite('tractable', deep=True))
         return self.rewrite(gamma).rewrite('tractable', deep=True)
@@ -821,11 +807,6 @@ class FallingFactorial(CombinatorialFunction):
         return fuzzy_and((self.args[0].is_integer, self.args[1].is_integer,
                           self.args[1].is_nonnegative))
 
-    def _sage_(self):
-        import sage.all as sage
-        return sage.falling_factorial(self.args[0]._sage_(),
-                                      self.args[1]._sage_())
-
 
 rf = RisingFactorial
 ff = FallingFactorial
@@ -840,7 +821,7 @@ class binomial(CombinatorialFunction):
     in two ways depending on its desired interpretation:
 
     .. math:: \binom{n}{k} = \frac{n!}{k!(n-k)!}\ \text{or}\
-                \binom{n}{k} = \frac{ff(n, k)}{k!}
+                \binom{n}{k} = \frac{(n)_k}{k!}
 
     First, in a strict combinatorial sense it defines the
     number of ways we can choose `k` elements from a set of
@@ -852,8 +833,8 @@ class binomial(CombinatorialFunction):
     however `k` must also be nonnegative. This case is very
     useful when evaluating summations.
 
-    For the sake of convenience for negative integer `k` this function
-    will return zero no matter what valued is the other argument.
+    For the sake of convenience, for negative integer `k` this function
+    will return zero no matter the other argument.
 
     To expand the binomial when `n` is a symbol, use either
     ``expand_func()`` or ``expand(func=True)``. The former will keep
@@ -914,14 +895,14 @@ class binomial(CombinatorialFunction):
     """
 
     def fdiff(self, argindex=1):
-        from sympy import polygamma
+        from sympy.functions.special.gamma_functions import polygamma
         if argindex == 1:
-            # http://functions.wolfram.com/GammaBetaErf/Binomial/20/01/01/
+            # https://functions.wolfram.com/GammaBetaErf/Binomial/20/01/01/
             n, k = self.args
             return binomial(n, k)*(polygamma(0, n + 1) - \
                 polygamma(0, n - k + 1))
         elif argindex == 2:
-            # http://functions.wolfram.com/GammaBetaErf/Binomial/20/01/02/
+            # https://functions.wolfram.com/GammaBetaErf/Binomial/20/01/02/
             n, k = self.args
             return binomial(n, k)*(polygamma(0, n - k + 1) - \
                 polygamma(0, k + 1))
@@ -942,7 +923,6 @@ class binomial(CombinatorialFunction):
                     k = n - k
 
                 if HAS_GMPY:
-                    from sympy.core.compatibility import gmpy
                     return Integer(gmpy.bincoef(n, k))
 
                 d, result = n - k, 1
@@ -955,8 +935,7 @@ class binomial(CombinatorialFunction):
                 for i in range(1, k + 1):
                     d += 1
                     result *= d
-                    result /= i
-                return result
+                return result / _factorial(k)
 
     @classmethod
     def eval(cls, n, k):
@@ -979,7 +958,7 @@ class binomial(CombinatorialFunction):
             # a special case when binomial evaluates to complex infinity
             return S.ComplexInfinity
         elif k.is_number:
-            from sympy import gamma
+            from sympy.functions.special.gamma_functions import gamma
             return gamma(n + 1)/(gamma(k + 1)*gamma(n - k + 1))
 
     def _eval_Mod(self, q):
@@ -1085,8 +1064,7 @@ class binomial(CombinatorialFunction):
                 n, result = self.args[0], 1
                 for i in range(1, k + 1):
                     result *= n - k + i
-                    result /= i
-                return result
+                return result / _factorial(k)
         else:
             return binomial(*self.args)
 
@@ -1094,7 +1072,7 @@ class binomial(CombinatorialFunction):
         return factorial(n)/(factorial(k)*factorial(n - k))
 
     def _eval_rewrite_as_gamma(self, n, k, piecewise=True, **kwargs):
-        from sympy import gamma
+        from sympy.functions.special.gamma_functions import gamma
         return gamma(n + 1)/(gamma(k + 1)*gamma(n - k + 1))
 
     def _eval_rewrite_as_tractable(self, n, k, limitvar=None, **kwargs):
@@ -1118,3 +1096,7 @@ class binomial(CombinatorialFunction):
                 return True
             elif k.is_even is False:
                 return  False
+
+    def _eval_as_leading_term(self, x, logx=None, cdir=0):
+        from sympy.functions.special.gamma_functions import gamma
+        return self.rewrite(gamma)._eval_as_leading_term(x, logx=logx, cdir=cdir)
