@@ -2,10 +2,25 @@ from sympy.holonomic import (DifferentialOperator, HolonomicFunction,
                              DifferentialOperators, from_hyper,
                              from_meijerg, expr_to_holonomic)
 from sympy.holonomic.recurrence import RecurrenceOperators, HolonomicSequence
-from sympy import (symbols, hyper, S, sqrt, pi, exp, erf, erfc, sstr, Symbol,
-                   O, I, meijerg, sin, cos, log, cosh, besselj, hyperexpand,
-                   Ci, EulerGamma, Si, asinh, gamma, beta, Rational)
-from sympy import ZZ, QQ, RR
+from sympy.core import EulerGamma
+from sympy.core.numbers import (I, Rational, pi)
+from sympy.core.singleton import S
+from sympy.core.symbol import (Symbol, symbols)
+from sympy.functions.elementary.exponential import (exp, log)
+from sympy.functions.elementary.hyperbolic import (asinh, cosh)
+from sympy.functions.elementary.miscellaneous import sqrt
+from sympy.functions.elementary.trigonometric import (cos, sin)
+from sympy.functions.special.bessel import besselj
+from sympy.functions.special.beta_functions import beta
+from sympy.functions.special.error_functions import (Ci, Si, erf, erfc)
+from sympy.functions.special.gamma_functions import gamma
+from sympy.functions.special.hyper import (hyper, meijerg)
+from sympy.printing.str import sstr
+from sympy.series.order import O
+from sympy.simplify.hyperexpand import hyperexpand
+from sympy.polys.domains.integerring import ZZ
+from sympy.polys.domains.rationalfield import QQ
+from sympy.polys.domains.realfield import RR
 
 
 def test_DifferentialOperator():
@@ -312,7 +327,7 @@ def test_evalf_euler():
     s = '0.699525841805253'  # approx. equal to log(2) i.e. 0.693147180559945
     assert sstr(p.evalf(r, method='Euler')[-1]) == s
 
-    # path taken is a traingle 0-->1+i-->2
+    # path taken is a triangle 0-->1+i-->2
     r = [0.1 + 0.1*I]
     for i in range(9):
         r.append(r[-1]+0.1+0.1*I)
@@ -389,7 +404,7 @@ def test_evalf_rk4():
     s = '0.693146363174626'  # approx. equal to log(2) i.e. 0.693147180559945
     assert sstr(p.evalf(r)[-1]) == s
 
-    # path taken is a traingle 0-->1+i-->2
+    # path taken is a triangle 0-->1+i-->2
     r = [0.1 + 0.1*I]
     for i in range(9):
         r.append(r[-1]+0.1+0.1*I)
