@@ -1,4 +1,5 @@
-from sympy import Symbol, sin
+from sympy.core.symbol import Symbol
+from sympy.functions.elementary.trigonometric import sin
 from sympy.integrals.trigonometry import trigintegrate
 
 x = Symbol('x')
