@@ -1,15 +1,13 @@
 """Options manager for :class:`~.Poly` and public API functions. """
 
+from __future__ import annotations
 
 __all__ = ["Options"]
 
-from typing import Dict, Type
-from typing import List, Optional
-
 from sympy.core import Basic, sympify
 from sympy.polys.polyerrors import GeneratorsError, OptionError, FlagError
 from sympy.utilities import numbered_symbols, topological_sort, public
-from sympy.utilities.iterables import has_dups
+from sympy.utilities.iterables import has_dups, is_sequence
 
 import sympy.polys
 
@@ -18,15 +16,15 @@ import re
 class Option:
     """Base class for all kinds of options. """
 
-    option = None  # type: Optional[str]
+    option: str | None = None
 
     is_Flag = False
 
-    requires = []  # type: List[str]
-    excludes = []  # type: List[str]
+    requires: list[str] = []
+    excludes: list[str] = []
 
-    after = []  # type: List[str]
-    before = []  # type: List[str]
+    after: list[str] = []
+    before: list[str] = []
 
     @classmethod
     def default(cls):
@@ -123,7 +121,7 @@ class Options(dict):
     """
 
     __order__ = None
-    __options__ = {}  # type: Dict[str, Type[Option]]
+    __options__: dict[str, type[Option]] = {}
 
     def __init__(self, gens, args, flags=None, strict=False):
         dict.__init__(self)
@@ -259,8 +257,8 @@ class Expand(BooleanOption, metaclass=OptionType):
 
     option = 'expand'
 
-    requires = []  # type: List[str]
-    excludes = []  # type: List[str]
+    requires: list[str] = []
+    excludes: list[str] = []
 
     @classmethod
     def default(cls):
@@ -272,8 +270,8 @@ class Gens(Option, metaclass=OptionType):
 
     option = 'gens'
 
-    requires = []  # type: List[str]
-    excludes = []  # type: List[str]
+    requires: list[str] = []
+    excludes: list[str] = []
 
     @classmethod
     def default(cls):
@@ -283,7 +281,7 @@ class Gens(Option, metaclass=OptionType):
     def preprocess(cls, gens):
         if isinstance(gens, Basic):
             gens = (gens,)
-        elif len(gens) == 1 and hasattr(gens[0], '__iter__'):
+        elif len(gens) == 1 and is_sequence(gens[0]):
             gens = gens[0]
 
         if gens == (None,):
@@ -301,8 +299,8 @@ class Wrt(Option, metaclass=OptionType):
 
     option = 'wrt'
 
-    requires = []  # type: List[str]
-    excludes = []  # type: List[str]
+    requires: list[str] = []
+    excludes: list[str] = []
 
     _re_split = re.compile(r"\s*,\s*|\s+")
 
@@ -316,7 +314,7 @@ class Wrt(Option, metaclass=OptionType):
                 raise OptionError('Bad input: missing parameter.')
             if not wrt:
                 return []
-            return [ gen for gen in cls._re_split.split(wrt) ]
+            return list(cls._re_split.split(wrt))
         elif hasattr(wrt, '__getitem__'):
             return list(map(str, wrt))
         else:
@@ -328,8 +326,8 @@ class Sort(Option, metaclass=OptionType):
 
     option = 'sort'
 
-    requires = []  # type: List[str]
-    excludes = []  # type: List[str]
+    requires: list[str] = []
+    excludes: list[str] = []
 
     @classmethod
     def default(cls):
@@ -350,8 +348,8 @@ class Order(Option, metaclass=OptionType):
 
     option = 'order'
 
-    requires = []  # type: List[str]
-    excludes = []  # type: List[str]
+    requires: list[str] = []
+    excludes: list[str] = []
 
     @classmethod
     def default(cls):
@@ -367,7 +365,7 @@ class Field(BooleanOption, metaclass=OptionType):
 
     option = 'field'
 
-    requires = []  # type: List[str]
+    requires: list[str] = []
     excludes = ['domain', 'split', 'gaussian']
 
 
@@ -376,7 +374,7 @@ class Greedy(BooleanOption, metaclass=OptionType):
 
     option = 'greedy'
 
-    requires = []  # type: List[str]
+    requires: list[str] = []
     excludes = ['domain', 'split', 'gaussian', 'extension', 'modulus', 'symmetric']
 
 
@@ -389,7 +387,7 @@ class Composite(BooleanOption, metaclass=OptionType):
     def default(cls):
         return None
 
-    requires = []  # type: List[str]
+    requires: list[str] = []
     excludes = ['domain', 'split', 'gaussian', 'extension', 'modulus', 'symmetric']
 
 
@@ -398,7 +396,7 @@ class Domain(Option, metaclass=OptionType):
 
     option = 'domain'
 
-    requires = []  # type: List[str]
+    requires: list[str] = []
     excludes = ['field', 'greedy', 'split', 'gaussian', 'extension']
 
     after = ['gens']
@@ -513,7 +511,7 @@ class Split(BooleanOption, metaclass=OptionType):
 
     option = 'split'
 
-    requires = []  # type: List[str]
+    requires: list[str] = []
     excludes = ['field', 'greedy', 'domain', 'gaussian', 'extension',
         'modulus', 'symmetric']
 
@@ -528,7 +526,7 @@ class Gaussian(BooleanOption, metaclass=OptionType):
 
     option = 'gaussian'
 
-    requires = []  # type: List[str]
+    requires: list[str] = []
     excludes = ['field', 'greedy', 'domain', 'split', 'extension',
         'modulus', 'symmetric']
 
@@ -544,7 +542,7 @@ class Extension(Option, metaclass=OptionType):
 
     option = 'extension'
 
-    requires = []  # type: List[str]
+    requires: list[str] = []
     excludes = ['greedy', 'domain', 'split', 'gaussian', 'modulus',
         'symmetric']
 
@@ -577,7 +575,7 @@ class Modulus(Option, metaclass=OptionType):
 
     option = 'modulus'
 
-    requires = []  # type: List[str]
+    requires: list[str] = []
     excludes = ['greedy', 'split', 'domain', 'gaussian', 'extension']
 
     @classmethod
@@ -772,7 +770,7 @@ def allowed_flags(args, flags):
 
     for arg in args.keys():
         try:
-            if Options.__options__[arg].is_Flag and not arg in flags:
+            if Options.__options__[arg].is_Flag and arg not in flags:
                 raise FlagError(
                     "'%s' flag is not allowed in this context" % arg)
         except KeyError:
