@@ -8,7 +8,10 @@ from sympy.polys.polyerrors import DomainError
 from sympy.polys.polyclasses import ANP
 from sympy.polys.specialpolys import f_polys, w_polys
 
-from sympy import nextprime, sin, sqrt, I
+from sympy.core.numbers import I
+from sympy.functions.elementary.miscellaneous import sqrt
+from sympy.functions.elementary.trigonometric import sin
+from sympy.ntheory.generate import nextprime
 from sympy.testing.pytest import raises, XFAIL
 
 
