@@ -1,6 +1,7 @@
 """Tests for the implementation of RootOf class and related tools. """
 
 from sympy.polys.polytools import Poly
+import sympy.polys.rootoftools as rootoftools
 from sympy.polys.rootoftools import (rootof, RootOf, CRootOf, RootSum,
     _pure_key_dict as D)
 
@@ -10,10 +11,17 @@ from sympy.polys.polyerrors import (
     PolynomialError,
 )
 
-from sympy import (
-    S, sqrt, I, Rational, Float, Lambda, log, exp, tan, Function, Eq,
-    solve, legendre_poly, Integral
-)
+from sympy.core.function import (Function, Lambda)
+from sympy.core.numbers import (Float, I, Rational)
+from sympy.core.relational import Eq
+from sympy.core.singleton import S
+from sympy.functions.elementary.exponential import (exp, log)
+from sympy.functions.elementary.miscellaneous import sqrt
+from sympy.functions.elementary.trigonometric import tan
+from sympy.integrals.integrals import Integral
+from sympy.polys.orthopolys import legendre_poly
+from sympy.solvers.solvers import solve
+
 
 from sympy.testing.pytest import raises, slow
 from sympy.core.expr import unchanged
@@ -116,7 +124,6 @@ def test_CRootOf_attributes():
     raises(NotImplementedError, lambda: rootof(Poly(x**3 + y*x + 1, x), 0))
 
 
-
 def test_CRootOf___eq__():
     assert (rootof(x**3 + x + 3, 0) == rootof(x**3 + x + 3, 0)) is True
     assert (rootof(x**3 + x + 3, 0) == rootof(x**3 + x + 3, 1)) is False
@@ -153,8 +160,8 @@ def test_CRootOf___eval_Eq__():
             assert Eq(r, s) is S.true
     eq = x**3 + x + 1
     sol = solve(eq)
-    assert [Eq(rootof(eq, i), j) for i in range(3) for j in sol] == [
-        False, False, True, False, True, False, True, False, False]
+    assert [Eq(rootof(eq, i), j) for i in range(3) for j in sol
+        ].count(True) == 3
     assert Eq(rootof(eq, 0), 1 + S.ImaginaryUnit) == False
 
 
@@ -241,8 +248,8 @@ def test_CRootOf_evalf():
     # issue 9019
     r0 = rootof(x**2 + 1, 0, radicals=False)
     r1 = rootof(x**2 + 1, 1, radicals=False)
-    assert r0.n(4) == -1.0*I
-    assert r1.n(4) == 1.0*I
+    assert r0.n(4) == Float(-1.0, 4) * I
+    assert r1.n(4) == Float(1.0, 4) * I
 
     # make sure verification is used in case a max/min traps the "root"
     assert str(rootof(4*x**5 + 16*x**3 + 12*x**2 + 7, 0).n(3)) == '-0.976'
@@ -284,6 +291,10 @@ def test_CRootOf_real_roots():
     assert Poly(x**5 + x + 1).real_roots(radicals=False) == [rootof(
         x**3 - x**2 + 1, 0)]
 
+    # https://github.com/sympy/sympy/issues/20902
+    p = Poly(-3*x**4 - 10*x**3 - 12*x**2 - 6*x - 1, x, domain='ZZ')
+    assert CRootOf.real_roots(p) == [S(-1), S(-1), S(-1), S(-1)/3]
+
 
 def test_CRootOf_all_roots():
     assert Poly(x**5 + x + 1).all_roots() == [
@@ -317,6 +328,50 @@ def test_CRootOf_eval_rational():
              ]
 
 
+def test_CRootOf_lazy():
+    # irreducible poly with both real and complex roots:
+    f = Poly(x**3 + 2*x + 2)
+
+    # real root:
+    CRootOf.clear_cache()
+    r = CRootOf(f, 0)
+    # Not yet in cache, after construction:
+    assert r.poly not in rootoftools._reals_cache
+    assert r.poly not in rootoftools._complexes_cache
+    r.evalf()
+    # In cache after evaluation:
+    assert r.poly in rootoftools._reals_cache
+    assert r.poly not in rootoftools._complexes_cache
+
+    # complex root:
+    CRootOf.clear_cache()
+    r = CRootOf(f, 1)
+    # Not yet in cache, after construction:
+    assert r.poly not in rootoftools._reals_cache
+    assert r.poly not in rootoftools._complexes_cache
+    r.evalf()
+    # In cache after evaluation:
+    assert r.poly in rootoftools._reals_cache
+    assert r.poly in rootoftools._complexes_cache
+
+    # composite poly with both real and complex roots:
+    f = Poly((x**2 - 2)*(x**2 + 1))
+
+    # real root:
+    CRootOf.clear_cache()
+    r = CRootOf(f, 0)
+    # In cache immediately after construction:
+    assert r.poly in rootoftools._reals_cache
+    assert r.poly not in rootoftools._complexes_cache
+
+    # complex root:
+    CRootOf.clear_cache()
+    r = CRootOf(f, 2)
+    # In cache immediately after construction:
+    assert r.poly in rootoftools._reals_cache
+    assert r.poly in rootoftools._complexes_cache
+
+
 def test_RootSum___new__():
     f = x**3 + x + 3
 
@@ -522,12 +577,12 @@ def test_eval_approx_relative():
     CRootOf.clear_cache()
     t = [CRootOf(x**3 + 10*x + 1, i) for i in range(3)]
     assert [i.eval_rational(1e-1) for i in t] == [
-        Rational(-21, 220), Rational(15, 256) - I*Rational(805, 256),
-        Rational(15, 256) + I*Rational(805, 256)]
+        Rational(-21, 220), Rational(15, 256) - I*805/256,
+        Rational(15, 256) + I*805/256]
     t[0]._reset()
     assert [i.eval_rational(1e-1, 1e-4) for i in t] == [
-        Rational(-21, 220), Rational(3275, 65536) - I*Rational(414645, 131072),
-        Rational(3275, 65536) + I*Rational(414645, 131072)]
+        Rational(-21, 220), Rational(3275, 65536) - I*414645/131072,
+        Rational(3275, 65536) + I*414645/131072]
     assert S(t[0]._get_interval().dx) < 1e-1
     assert S(t[1]._get_interval().dx) < 1e-1
     assert S(t[1]._get_interval().dy) < 1e-4
@@ -535,8 +590,8 @@ def test_eval_approx_relative():
     assert S(t[2]._get_interval().dy) < 1e-4
     t[0]._reset()
     assert [i.eval_rational(1e-4, 1e-4) for i in t] == [
-        Rational(-2001, 20020), Rational(6545, 131072) - I*Rational(414645, 131072),
-        Rational(6545, 131072) + I*Rational(414645, 131072)]
+        Rational(-2001, 20020), Rational(6545, 131072) - I*414645/131072,
+        Rational(6545, 131072) + I*414645/131072]
     assert S(t[0]._get_interval().dx) < 1e-4
     assert S(t[1]._get_interval().dx) < 1e-4
     assert S(t[1]._get_interval().dy) < 1e-4
@@ -546,8 +601,8 @@ def test_eval_approx_relative():
     # less than tested, but it should never be greater
     t[0]._reset()
     assert [i.eval_rational(n=2) for i in t] == [
-        Rational(-202201, 2024022), Rational(104755, 2097152) - I*Rational(6634255, 2097152),
-        Rational(104755, 2097152) + I*Rational(6634255, 2097152)]
+        Rational(-202201, 2024022), Rational(104755, 2097152) - I*6634255/2097152,
+        Rational(104755, 2097152) + I*6634255/2097152]
     assert abs(S(t[0]._get_interval().dx)/t[0]) < 1e-2
     assert abs(S(t[1]._get_interval().dx)/t[1]).n() < 1e-2
     assert abs(S(t[1]._get_interval().dy)/t[1]).n() < 1e-2
@@ -555,8 +610,8 @@ def test_eval_approx_relative():
     assert abs(S(t[2]._get_interval().dy)/t[2]).n() < 1e-2
     t[0]._reset()
     assert [i.eval_rational(n=3) for i in t] == [
-        Rational(-202201, 2024022), Rational(1676045, 33554432) - I*Rational(106148135, 33554432),
-        Rational(1676045, 33554432) + I*Rational(106148135, 33554432)]
+        Rational(-202201, 2024022), Rational(1676045, 33554432) - I*106148135/33554432,
+        Rational(1676045, 33554432) + I*106148135/33554432]
     assert abs(S(t[0]._get_interval().dx)/t[0]) < 1e-3
     assert abs(S(t[1]._get_interval().dx)/t[1]).n() < 1e-3
     assert abs(S(t[1]._get_interval().dy)/t[1]).n() < 1e-3
