@@ -2,18 +2,19 @@
 A MathML printer.
 """
 
-from typing import Any, Dict
+from __future__ import annotations
+from typing import Any
 
-from sympy import sympify, S, Mul
-from sympy.core.compatibility import default_sort_key
-from sympy.core.function import _coeff_isneg
+from sympy.core.mul import Mul
+from sympy.core.singleton import S
+from sympy.core.sorting import default_sort_key
+from sympy.core.sympify import sympify
 from sympy.printing.conventions import split_super_sub, requires_partial
 from sympy.printing.precedence import \
     precedence_traditional, PRECEDENCE, PRECEDENCE_TRADITIONAL
 from sympy.printing.pretty.pretty_symbology import greek_unicode
 from sympy.printing.printer import Printer, print_function
 
-import mpmath.libmp as mlib
 from mpmath.libmp import prec_to_dps, repr_dps, to_str as mlib_to_str
 
 
@@ -22,7 +23,7 @@ class MathMLPrinterBase(Printer):
     MathMLPresentationPrinter.
     """
 
-    _default_settings = {
+    _default_settings: dict[str, Any] = {
         "order": None,
         "encoding": "utf-8",
         "fold_frac_powers": False,
@@ -37,7 +38,7 @@ class MathMLPrinterBase(Printer):
         "root_notation": True,
         "symbol_names": {},
         "mul_symbol_mathml_numbers": '&#xB7;',
-    }  # type: Dict[str, Any]
+    }
 
     def __init__(self, settings=None):
         Printer.__init__(self, settings)
@@ -77,8 +78,8 @@ class MathMLPrinterBase(Printer):
         # Date: 2011-11-18
         # Description: http://ronrothman.com/public/leftbraned/xml-dom-minidom\
         #                   -toprettyxml-and-silly-whitespace/#best-solution
-        # Issue: http://bugs.python.org/issue4147
-        # Patch: http://hg.python.org/cpython/rev/7262f8f276ff/
+        # Issue: https://bugs.python.org/issue4147
+        # Patch: https://hg.python.org/cpython/rev/7262f8f276ff/
 
         from xml.dom.minidom import Element, Text, Node, _write_data
 
@@ -199,7 +200,7 @@ class MathMLContentPrinter(MathMLPrinterBase):
 
     def _print_Mul(self, expr):
 
-        if _coeff_isneg(expr):
+        if expr.could_extract_minus_sign():
             x = self.dom.createElement('apply')
             x.appendChild(self.dom.createElement('minus'))
             x.appendChild(self._print_Mul(-expr))
@@ -237,7 +238,7 @@ class MathMLContentPrinter(MathMLPrinterBase):
         lastProcessed = self._print(args[0])
         plusNodes = []
         for arg in args[1:]:
-            if _coeff_isneg(arg):
+            if arg.could_extract_minus_sign():
                 # use minus
                 x = self.dom.createElement('apply')
                 x.appendChild(self.dom.createElement('minus'))
@@ -330,7 +331,7 @@ class MathMLContentPrinter(MathMLPrinterBase):
 
     def _print_GoldenRatio(self, e):
         """We use unicode #x3c6 for Greek letter phi as defined here
-        http://www.w3.org/2003/entities/2007doc/isogrk1.html"""
+        https://www.w3.org/2003/entities/2007doc/isogrk1.html"""
         x = self.dom.createElement('cn')
         x.appendChild(self.dom.createTextNode("\N{GREEK SMALL LETTER PHI}"))
         return x
@@ -464,9 +465,9 @@ class MathMLContentPrinter(MathMLPrinterBase):
             x.appendChild(self.dom.createElement('root'))
             if e.exp.q != 2:
                 xmldeg = self.dom.createElement('degree')
-                xmlci = self.dom.createElement('ci')
-                xmlci.appendChild(self.dom.createTextNode(str(e.exp.q)))
-                xmldeg.appendChild(xmlci)
+                xmlcn = self.dom.createElement('cn')
+                xmlcn.appendChild(self.dom.createTextNode(str(e.exp.q)))
+                xmldeg.appendChild(xmlcn)
                 x.appendChild(xmldeg)
             x.appendChild(self._print(e.base))
             return x
@@ -538,7 +539,7 @@ class MathMLContentPrinter(MathMLPrinterBase):
 
     def _print_list(self, seq):
         """MathML reference for the <list> element:
-        http://www.w3.org/TR/MathML2/chapter4.html#contm.list"""
+        https://www.w3.org/TR/MathML2/chapter4.html#contm.list"""
         dom_element = self.dom.createElement('list')
         for item in seq:
             dom_element.appendChild(self._print(item))
@@ -705,7 +706,7 @@ class MathMLPresentationPrinter(MathMLPrinterBase):
                     mrow.appendChild(y)
             return mrow
         mrow = self.dom.createElement('mrow')
-        if _coeff_isneg(expr):
+        if expr.could_extract_minus_sign():
             x = self.dom.createElement('mo')
             x.appendChild(self.dom.createTextNode('-'))
             mrow.appendChild(x)
@@ -720,7 +721,7 @@ class MathMLPresentationPrinter(MathMLPrinterBase):
         args = self._as_ordered_terms(expr, order=order)
         mrow.appendChild(self._print(args[0]))
         for arg in args[1:]:
-            if _coeff_isneg(arg):
+            if arg.could_extract_minus_sign():
                 # use minus
                 x = self.dom.createElement('mo')
                 x.appendChild(self.dom.createTextNode('-'))
@@ -1173,7 +1174,7 @@ class MathMLPresentationPrinter(MathMLPrinterBase):
     def _print_Float(self, expr):
         # Based off of that in StrPrinter
         dps = prec_to_dps(expr._prec)
-        str_real = mlib.to_str(expr._mpf_, dps, strip_zeros=True)
+        str_real = mlib_to_str(expr._mpf_, dps, strip_zeros=True)
 
         # Must always have a mul symbol (as 2.5 10^{20} just looks odd)
         # thus we use the number separator
@@ -1779,7 +1780,7 @@ class MathMLPresentationPrinter(MathMLPrinterBase):
         return sup
 
     def _print_MatMul(self, expr):
-        from sympy import MatMul
+        from sympy.matrices.expressions.matmul import MatMul
 
         x = self.dom.createElement('mrow')
         args = expr.args
@@ -1788,7 +1789,7 @@ class MathMLPresentationPrinter(MathMLPrinterBase):
         else:
             args = list(args)
 
-        if isinstance(expr, MatMul) and _coeff_isneg(expr):
+        if isinstance(expr, MatMul) and expr.could_extract_minus_sign():
             if args[0] == -1:
                 args = args[1:]
             else:
@@ -2093,7 +2094,7 @@ def print_mathml(expr, printer='content', **settings):
     ========
 
     >>> ##
-    >>> from sympy.printing.mathml import print_mathml
+    >>> from sympy import print_mathml
     >>> from sympy.abc import x
     >>> print_mathml(x+1) #doctest: +NORMALIZE_WHITESPACE
     <apply>
