@@ -1,12 +1,9 @@
 """Formal Power Series"""
 
-from __future__ import print_function, division
-
 from collections import defaultdict
 
-from sympy import oo, zoo, nan
+from sympy.core.numbers import (nan, oo, zoo)
 from sympy.core.add import Add
-from sympy.core.compatibility import iterable
 from sympy.core.expr import Expr
 from sympy.core.function import Derivative, Function, expand
 from sympy.core.mul import Mul
@@ -24,9 +21,10 @@ from sympy.functions.elementary.miscellaneous import Min, Max
 from sympy.functions.elementary.piecewise import Piecewise
 from sympy.series.limits import Limit
 from sympy.series.order import Order
-from sympy.simplify.powsimp import powsimp
 from sympy.series.sequences import sequence
 from sympy.series.series_class import SeriesBase
+from sympy.utilities.iterables import iterable
+
 
 
 def rational_algorithm(f, x, k, order=4, full=False):
@@ -35,6 +33,9 @@ def rational_algorithm(f, x, k, order=4, full=False):
     formula of coefficients of Formal Power Series
     of a function.
 
+    Explanation
+    ===========
+
     Applicable when f(x) or some derivative of f(x)
     is a rational function in x.
 
@@ -46,6 +47,14 @@ def rational_algorithm(f, x, k, order=4, full=False):
     Looks for derivative of a function up to 4'th order (by default).
     This can be overridden using order option.
 
+    Parameters
+    ==========
+
+    x : Symbol
+    order : int, optional
+        Order of the derivative of ``f``, Default is 4.
+    full : bool
+
     Returns
     =======
 
@@ -53,6 +62,7 @@ def rational_algorithm(f, x, k, order=4, full=False):
     ind : Expr
         Independent terms.
     order : int
+    full : bool
 
     Examples
     ========
@@ -64,10 +74,10 @@ def rational_algorithm(f, x, k, order=4, full=False):
     >>> ra(1 / (1 - x), x, k)
     (1, 0, 0)
     >>> ra(log(1 + x), x, k)
-    (-(-1)**(-k)/k, 0, 1)
+    (-1/((-1)**k*k), 0, 1)
 
     >>> ra(atan(x), x, k, full=True)
-    ((-I*(-I)**(-k)/2 + I*I**(-k)/2)/k, 0, 1)
+    ((-I/(2*(-I)**k) + I/(2*I**k))/k, 0, 1)
 
     Notes
     =====
@@ -156,7 +166,8 @@ def rational_algorithm(f, x, k, order=4, full=False):
 
 
 def rational_independent(terms, x):
-    """Returns a list of all the rationally independent terms.
+    """
+    Returns a list of all the rationally independent terms.
 
     Examples
     ========
@@ -189,7 +200,11 @@ def rational_independent(terms, x):
 
 
 def simpleDE(f, x, g, order=4):
-    r"""Generates simple DE.
+    r"""
+    Generates simple DE.
+
+    Explanation
+    ===========
 
     DE is of the form
 
@@ -232,6 +247,9 @@ def simpleDE(f, x, g, order=4):
 def exp_re(DE, r, k):
     """Converts a DE with constant coefficients (explike) into a RE.
 
+    Explanation
+    ===========
+
     Performs the substitution:
 
     .. math::
@@ -277,7 +295,11 @@ def exp_re(DE, r, k):
 
 
 def hyper_re(DE, r, k):
-    """Converts a DE into a RE.
+    """
+    Converts a DE into a RE.
+
+    Explanation
+    ===========
 
     Performs the substitution:
 
@@ -384,10 +406,8 @@ def _compute_formula(f, x, P, Q, k, m, k_max):
         c2 = q.subs(k, 1/k).leadterm(k)[0]
         res *= (-c1 / c2)**k
 
-        for r, mul in roots(p, k).items():
-            res *= rf(-r, k)**mul
-        for r, mul in roots(q, k).items():
-            res /= rf(-r, k)**mul
+        res *= Mul(*[rf(-r, k)**mul for r, mul in roots(p, k).items()])
+        res /= Mul(*[rf(-r, k)**mul for r, mul in roots(q, k).items()])
 
         sol.append((res, kterm))
 
@@ -395,7 +415,11 @@ def _compute_formula(f, x, P, Q, k, m, k_max):
 
 
 def _rsolve_hypergeometric(f, x, P, Q, k, m):
-    """Recursive wrapper to rsolve_hypergeometric.
+    """
+    Recursive wrapper to rsolve_hypergeometric.
+
+    Explanation
+    ===========
 
     Returns a Tuple of (formula, series independent terms,
     maximum power of x in independent terms) if successful
@@ -462,7 +486,11 @@ def _rsolve_hypergeometric(f, x, P, Q, k, m):
 
 
 def rsolve_hypergeometric(f, x, P, Q, k, m):
-    """Solves RE of hypergeometric type.
+    """
+    Solves RE of hypergeometric type.
+
+    Explanation
+    ===========
 
     Attempts to solve RE of the form
 
@@ -655,7 +683,11 @@ def _transform_DE_RE(DE, g, k, order, syms):
 
 
 def solve_de(f, x, DE, order, g, k):
-    """Solves the DE.
+    """
+    Solves the DE.
+
+    Explanation
+    ===========
 
     Tries to solve DE by either converting into a RE containing two terms or
     converting into a DE having constant coefficients.
@@ -707,7 +739,11 @@ def solve_de(f, x, DE, order, g, k):
 
 
 def hyper_algorithm(f, x, k, order=4):
-    """Hypergeometric algorithm for computing Formal Power Series.
+    """
+    Hypergeometric algorithm for computing Formal Power Series.
+
+    Explanation
+    ===========
 
     Steps:
         * Generates DE
@@ -823,9 +859,6 @@ def _compute_fps(f, x, x0, dir, hyper, order, rational, full):
     # Otherwise symb is being set to S.One
     syms = f.free_symbols.difference({x})
     (f, symb) = expand(f).as_independent(*syms)
-    if symb.is_zero:
-        symb = S.One
-    symb = powsimp(symb)
 
     result = None
 
@@ -840,6 +873,11 @@ def _compute_fps(f, x, x0, dir, hyper, order, rational, full):
     if result is None:
         return None
 
+    from sympy.simplify.powsimp import powsimp
+    if symb.is_zero:
+        symb = S.One
+    else:
+        symb = powsimp(symb)
     ak = sequence(result[0], (k, result[2], oo))
     xk_formula = powsimp(x**k * symb)
     xk = sequence(xk_formula, (k, 0, oo))
@@ -850,7 +888,11 @@ def _compute_fps(f, x, x0, dir, hyper, order, rational, full):
 
 def compute_fps(f, x, x0=0, dir=1, hyper=True, order=4, rational=True,
                 full=False):
-    """Computes the formula for Formal Power Series of a function.
+    """
+    Computes the formula for Formal Power Series of a function.
+
+    Explanation
+    ===========
 
     Tries to compute the formula by applying the following techniques
     (in order):
@@ -930,7 +972,11 @@ class Coeff(Function):
 
 
 class FormalPowerSeries(SeriesBase):
-    """Represents Formal Power Series of a function.
+    """
+    Represents Formal Power Series of a function.
+
+    Explanation
+    ===========
 
     No computation is performed. This class should only to be used to represent
     a series. No checks are performed.
@@ -1016,7 +1062,11 @@ class FormalPowerSeries(SeriesBase):
         return pow_x
 
     def polynomial(self, n=6):
-        """Truncated series as polynomial.
+        """
+        Truncated series as polynomial.
+
+        Explanation
+        ===========
 
         Returns series expansion of ``f`` upto order ``O(x**n)``
         as a polynomial(without ``O`` term).
@@ -1037,7 +1087,11 @@ class FormalPowerSeries(SeriesBase):
         return Add(*terms)
 
     def truncate(self, n=6):
-        """Truncated series.
+        """
+        Truncated series.
+
+        Explanation
+        ===========
 
         Returns truncated series expansion of f upto
         order ``O(x**n)``.
@@ -1086,7 +1140,7 @@ class FormalPowerSeries(SeriesBase):
         if old.has(x):
             return self
 
-    def _eval_as_leading_term(self, x, cdir=0):
+    def _eval_as_leading_term(self, x, logx=None, cdir=0):
         for t in self:
             if t is not S.Zero:
                 return t
@@ -1115,7 +1169,8 @@ class FormalPowerSeries(SeriesBase):
         return self.func(f, self.x, self.x0, self.dir, (ak, self.xk, ind))
 
     def integrate(self, x=None, **kwargs):
-        """Integrate Formal Power Series.
+        """
+        Integrate Formal Power Series.
 
         Examples
         ========
@@ -1159,7 +1214,8 @@ class FormalPowerSeries(SeriesBase):
         return self.func(f, self.x, self.x0, self.dir, (ak, self.xk, ind))
 
     def product(self, other, x=None, n=6):
-        """Multiplies two Formal Power Series, using discrete convolution and
+        """
+        Multiplies two Formal Power Series, using discrete convolution and
         return the truncated terms upto specified order.
 
         Parameters
@@ -1188,8 +1244,6 @@ class FormalPowerSeries(SeriesBase):
 
         """
 
-        if x is None:
-            x = self.x
         if n is None:
             return iter(self)
 
@@ -1245,9 +1299,12 @@ class FormalPowerSeries(SeriesBase):
     def compose(self, other, x=None, n=6):
         r"""
         Returns the truncated terms of the formal power series of the composed function,
-        up to specified `n`.
+        up to specified ``n``.
 
-        If `f` and `g` are two formal power series of two different functions,
+        Explanation
+        ===========
+
+        If ``f`` and ``g`` are two formal power series of two different functions,
         then the coefficient sequence ``ak`` of the composed formal power series `fp`
         will be as follows.
 
@@ -1288,8 +1345,6 @@ class FormalPowerSeries(SeriesBase):
 
         """
 
-        if x is None:
-            x = self.x
         if n is None:
             return iter(self)
 
@@ -1318,10 +1373,13 @@ class FormalPowerSeries(SeriesBase):
     def inverse(self, x=None, n=6):
         r"""
         Returns the truncated terms of the inverse of the formal power series,
-        up to specified `n`.
+        up to specified ``n``.
 
-        If `f` and `g` are two formal power series of two different functions,
-        then the coefficient sequence ``ak`` of the composed formal power series `fp`
+        Explanation
+        ===========
+
+        If ``f`` and ``g`` are two formal power series of two different functions,
+        then the coefficient sequence ``ak`` of the composed formal power series ``fp``
         will be as follows.
 
         .. math::
@@ -1361,8 +1419,6 @@ class FormalPowerSeries(SeriesBase):
 
         """
 
-        if x is None:
-            x = self.x
         if n is None:
             return iter(self)
 
@@ -1493,6 +1549,9 @@ class FiniteFormalPowerSeries(FormalPowerSeries):
 class FormalPowerSeriesProduct(FiniteFormalPowerSeries):
     """Represents the product of two formal power series of two functions.
 
+    Explanation
+    ===========
+
     No computation is performed. Terms are calculated using a term by term logic,
     instead of a point by point logic.
 
@@ -1525,7 +1584,7 @@ class FormalPowerSeriesProduct(FiniteFormalPowerSeries):
 
     def _eval_terms(self, n):
         """
-        Returns the first `n` terms of the product formal power series.
+        Returns the first ``n`` terms of the product formal power series.
         Term by term logic is implemented here.
 
         Examples
@@ -1558,7 +1617,11 @@ class FormalPowerSeriesProduct(FiniteFormalPowerSeries):
 
 
 class FormalPowerSeriesCompose(FiniteFormalPowerSeries):
-    """Represents the composed formal power series of two functions.
+    """
+    Represents the composed formal power series of two functions.
+
+    Explanation
+    ===========
 
     No computation is performed. Terms are calculated using a term by term logic,
     instead of a point by point logic.
@@ -1589,6 +1652,9 @@ class FormalPowerSeriesCompose(FiniteFormalPowerSeries):
         Returns the first `n` terms of the composed formal power series.
         Term by term logic is implemented here.
 
+        Explanation
+        ===========
+
         The coefficient sequence of the :obj:`FormalPowerSeriesCompose` object is the generic sequence.
         It is multiplied by ``bell_seq`` to get a sequence, whose terms are added up to get
         the final terms for the polynomial.
@@ -1628,7 +1694,11 @@ class FormalPowerSeriesCompose(FiniteFormalPowerSeries):
 
 
 class FormalPowerSeriesInverse(FiniteFormalPowerSeries):
-    """Represents the Inverse of a formal power series.
+    """
+    Represents the Inverse of a formal power series.
+
+    Explanation
+    ===========
 
     No computation is performed. Terms are calculated using a term by term logic,
     instead of a point by point logic.
@@ -1671,9 +1741,12 @@ class FormalPowerSeriesInverse(FiniteFormalPowerSeries):
 
     def _eval_terms(self, n):
         """
-        Returns the first `n` terms of the composed formal power series.
+        Returns the first ``n`` terms of the composed formal power series.
         Term by term logic is implemented here.
 
+        Explanation
+        ===========
+
         The coefficient sequence of the `FormalPowerSeriesInverse` object is the generic sequence.
         It is multiplied by ``bell_seq`` to get a sequence, whose terms are added up to get
         the final terms for the polynomial.
@@ -1712,7 +1785,11 @@ class FormalPowerSeriesInverse(FiniteFormalPowerSeries):
 
 
 def fps(f, x=None, x0=0, dir=1, hyper=True, order=4, rational=True, full=False):
-    """Generates Formal Power Series of f.
+    """
+    Generates Formal Power Series of ``f``.
+
+    Explanation
+    ===========
 
     Returns the formal series expansion of ``f`` around ``x = x0``
     with respect to ``x`` in the form of a ``FormalPowerSeries`` object.
