@@ -1,14 +1,13 @@
-from __future__ import print_function, division
-
+from sympy.calculus.accumulationbounds import AccumBounds
 from sympy.core import S, Symbol, Add, sympify, Expr, PoleError, Mul
 from sympy.core.exprtools import factor_terms
-from sympy.core.symbol import Dummy
+from sympy.core.numbers import Float, _illegal
 from sympy.functions.combinatorial.factorials import factorial
+from sympy.functions.elementary.complexes import (Abs, sign, arg, re)
+from sympy.functions.elementary.exponential import (exp, log)
 from sympy.functions.special.gamma_functions import gamma
 from sympy.polys import PolynomialError, factor
 from sympy.series.order import Order
-from sympy.simplify.ratsimp import ratsimp
-from sympy.simplify.simplify import together
 from .gruntz import gruntz
 
 def limit(e, z, z0, dir="+"):
@@ -73,14 +72,14 @@ def heuristics(e, z, z0, dir):
     works only for simple limits, but it is fast.
     """
 
-    from sympy.calculus.util import AccumBounds
     rv = None
-    if abs(z0) is S.Infinity:
-        rv = limit(e.subs(z, 1/z), z, S.Zero, "+" if z0 is S.Infinity else "-")
+    if z0 is S.Infinity:
+        rv = limit(e.subs(z, 1/z), z, S.Zero, "+")
         if isinstance(rv, Limit):
             return
     elif e.is_Mul or e.is_Add or e.is_Pow or e.is_Function:
         r = []
+        from sympy.simplify.simplify import together
         for a in e.args:
             l = limit(a, z, z0, dir)
             if l.has(S.Infinity) and l.is_finite is None:
@@ -105,9 +104,9 @@ def heuristics(e, z, z0, dir):
             if rv is S.NaN and e.is_Mul and any(isinstance(rr, AccumBounds) for rr in r):
                 r2 = []
                 e2 = []
-                for ii in range(len(r)):
-                    if isinstance(r[ii], AccumBounds):
-                        r2.append(r[ii])
+                for ii, rval in enumerate(r):
+                    if isinstance(rval, AccumBounds):
+                        r2.append(rval)
                     else:
                         e2.append(e.args[ii])
 
@@ -118,6 +117,7 @@ def heuristics(e, z, z0, dir):
 
             if rv is S.NaN:
                 try:
+                    from sympy.simplify.ratsimp import ratsimp
                     rat_e = ratsimp(e)
                 except PolynomialError:
                     return
@@ -136,7 +136,7 @@ class Limit(Expr):
     >>> from sympy import Limit, sin
     >>> from sympy.abc import x
     >>> Limit(sin(x)/x, x, 0)
-    Limit(sin(x)/x, x, 0)
+    Limit(sin(x)/x, x, 0, dir='+')
     >>> Limit(1/x, x, 0, dir="-")
     Limit(1/x, x, 0, dir='-')
 
@@ -147,11 +147,14 @@ class Limit(Expr):
         z = sympify(z)
         z0 = sympify(z0)
 
-        if z0 is S.Infinity:
+        if z0 in (S.Infinity, S.ImaginaryUnit*S.Infinity):
             dir = "-"
-        elif z0 is S.NegativeInfinity:
+        elif z0 in (S.NegativeInfinity, S.ImaginaryUnit*S.NegativeInfinity):
             dir = "+"
 
+        if(z0.has(z)):
+            raise NotImplementedError("Limits approaching a variable point are"
+                    " not supported (%s -> %s)" % (z, z0))
         if isinstance(dir, str):
             dir = Symbol(dir)
         elif not isinstance(dir, Symbol):
@@ -175,6 +178,24 @@ class Limit(Expr):
         return isyms
 
 
+    def pow_heuristics(self, e):
+        _, z, z0, _ = self.args
+        b1, e1 = e.base, e.exp
+        if not b1.has(z):
+            res = limit(e1*log(b1), z, z0)
+            return exp(res)
+
+        ex_lim = limit(e1, z, z0)
+        base_lim = limit(b1, z, z0)
+
+        if base_lim is S.One:
+            if ex_lim in (S.Infinity, S.NegativeInfinity):
+                res = limit(e1*(b1 - 1), z, z0)
+                return exp(res)
+        if base_lim is S.NegativeInfinity and ex_lim is S.Infinity:
+            return S.ComplexInfinity
+
+
     def doit(self, **hints):
         """Evaluates the limit.
 
@@ -188,15 +209,34 @@ class Limit(Expr):
         hints : optional keyword arguments
             To be passed to ``doit`` methods; only used if deep is True.
         """
-        from sympy import Abs, exp, log, sign
-        from sympy.calculus.util import AccumBounds
 
         e, z, z0, dir = self.args
 
+        if str(dir) == '+-':
+            r = limit(e, z, z0, dir='+')
+            l = limit(e, z, z0, dir='-')
+            if isinstance(r, Limit) and isinstance(l, Limit):
+                if r.args[0] == l.args[0]:
+                    return self
+            if r == l:
+                return l
+            if r.is_infinite and l.is_infinite:
+                return S.ComplexInfinity
+            raise ValueError("The limit does not exist since "
+                             "left hand limit = %s and right hand limit = %s"
+                             % (l, r))
+
         if z0 is S.ComplexInfinity:
             raise NotImplementedError("Limits at complex "
                                     "infinity are not implemented")
 
+        if z0.is_infinite:
+            cdir = sign(z0)
+            cdir = cdir/abs(cdir)
+            e = e.subs(z, cdir*z)
+            dir = "-"
+            z0 = S.Infinity
+
         if hints.get('deep', True):
             e = e.doit(**hints)
             z = z.doit(**hints)
@@ -208,52 +248,120 @@ class Limit(Expr):
         if not e.has(z):
             return e
 
+        if z0 is S.NaN:
+            return S.NaN
+
+        if e.has(*_illegal):
+            return self
+
+        if e.is_Order:
+            return Order(limit(e.expr, z, z0), *e.args[1:])
+
         cdir = 0
         if str(dir) == "+":
             cdir = 1
         elif str(dir) == "-":
             cdir = -1
 
-        def remove_abs(expr):
+        def set_signs(expr):
             if not expr.args:
                 return expr
-            newargs = tuple(remove_abs(arg) for arg in expr.args)
+            newargs = tuple(set_signs(arg) for arg in expr.args)
             if newargs != expr.args:
                 expr = expr.func(*newargs)
-            if isinstance(expr, Abs):
+            abs_flag = isinstance(expr, Abs)
+            arg_flag = isinstance(expr, arg)
+            sign_flag = isinstance(expr, sign)
+            if abs_flag or sign_flag or arg_flag:
                 sig = limit(expr.args[0], z, z0, dir)
                 if sig.is_zero:
                     sig = limit(1/expr.args[0], z, z0, dir)
                 if sig.is_extended_real:
                     if (sig < 0) == True:
-                        return -expr.args[0]
+                        return (-expr.args[0] if abs_flag else
+                                S.NegativeOne if sign_flag else S.Pi)
                     elif (sig > 0) == True:
-                        return expr.args[0]
+                        return (expr.args[0] if abs_flag else
+                                S.One if sign_flag else S.Zero)
             return expr
 
-        e = remove_abs(e)
+        if e.has(Float):
+            # Convert floats like 0.5 to exact SymPy numbers like S.Half, to
+            # prevent rounding errors which can lead to unexpected execution
+            # of conditional blocks that work on comparisons
+            # Also see comments in https://github.com/sympy/sympy/issues/19453
+            from sympy.simplify.simplify import nsimplify
+            e = nsimplify(e)
+        e = set_signs(e)
+
 
         if e.is_meromorphic(z, z0):
-            if abs(z0) is S.Infinity:
-                newe = e.subs(z, -1/z)
+            if z0 is S.Infinity:
+                newe = e.subs(z, 1/z)
+                # cdir changes sign as oo- should become 0+
+                cdir = -cdir
             else:
                 newe = e.subs(z, z + z0)
             try:
-                coeff, ex = newe.leadterm(z, cdir)
-            except (ValueError, NotImplementedError):
+                coeff, ex = newe.leadterm(z, cdir=cdir)
+            except ValueError:
                 pass
             else:
                 if ex > 0:
                     return S.Zero
                 elif ex == 0:
                     return coeff
-                if str(dir) == "+" or not(int(ex) & 1):
+                if cdir == 1 or not(int(ex) & 1):
                     return S.Infinity*sign(coeff)
-                elif str(dir) == "-":
+                elif cdir == -1:
                     return S.NegativeInfinity*sign(coeff)
                 else:
                     return S.ComplexInfinity
 
+        if z0 is S.Infinity:
+            if e.is_Mul:
+                e = factor_terms(e)
+            newe = e.subs(z, 1/z)
+            # cdir changes sign as oo- should become 0+
+            cdir = -cdir
+        else:
+            newe = e.subs(z, z + z0)
+        try:
+            coeff, ex = newe.leadterm(z, cdir=cdir)
+        except (ValueError, NotImplementedError, PoleError):
+            # The NotImplementedError catching is for custom functions
+            from sympy.simplify.powsimp import powsimp
+            e = powsimp(e)
+            if e.is_Pow:
+                r = self.pow_heuristics(e)
+                if r is not None:
+                    return r
+            try:
+                coeff = newe.as_leading_term(z, cdir=cdir)
+                if coeff != newe and coeff.has(exp):
+                    return gruntz(coeff, z, 0, "-" if re(cdir).is_negative else "+")
+            except (ValueError, NotImplementedError, PoleError):
+                pass
+        else:
+            if isinstance(coeff, AccumBounds) and ex == S.Zero:
+                return coeff
+            if coeff.has(S.Infinity, S.NegativeInfinity, S.ComplexInfinity, S.NaN):
+                return self
+            if not coeff.has(z):
+                if ex.is_positive:
+                    return S.Zero
+                elif ex == 0:
+                    return coeff
+                elif ex.is_negative:
+                    if cdir == 1:
+                        return S.Infinity*sign(coeff)
+                    elif cdir == -1:
+                        return S.NegativeInfinity*sign(coeff)*S.NegativeOne**(S.One + ex)
+                    else:
+                        return S.ComplexInfinity
+                else:
+                    raise NotImplementedError("Not sure of sign of %s" % ex)
+
         # gruntz fails on factorials but works with the gamma function
         # If no factorial term is present, e should remain unchanged.
         # factorial is defined to be zero for negative inputs (which
@@ -261,74 +369,10 @@ class Limit(Expr):
         if z0.is_extended_positive:
             e = e.rewrite(factorial, gamma)
 
-        if e.is_Mul and abs(z0) is S.Infinity:
-            e = factor_terms(e)
-            u = Dummy('u', positive=True)
-            if z0 is S.NegativeInfinity:
-                inve = e.subs(z, -1/u)
-            else:
-                inve = e.subs(z, 1/u)
-            try:
-                f = inve.as_leading_term(u).gammasimp()
-                if f.is_meromorphic(u, S.Zero):
-                    r = limit(f, u, S.Zero, "+")
-                    if isinstance(r, Limit):
-                        return self
-                    else:
-                        return r
-            except (ValueError, NotImplementedError, PoleError):
-                pass
-
-        if e.is_Order:
-            return Order(limit(e.expr, z, z0), *e.args[1:])
-
-        if e.is_Pow:
-            if e.has(S.Infinity, S.NegativeInfinity, S.ComplexInfinity, S.NaN):
-                return self
-
-            b1, e1 = e.base, e.exp
-            f1 = e1*log(b1)
-            if f1.is_meromorphic(z, z0):
-                res = limit(f1, z, z0)
-                return exp(res)
-
-            ex_lim = limit(e1, z, z0)
-            base_lim = limit(b1, z, z0)
-
-            if base_lim is S.One:
-                if ex_lim in (S.Infinity, S.NegativeInfinity):
-                    res = limit(e1*(b1 - 1), z, z0)
-                    return exp(res)
-                elif ex_lim.is_real:
-                    return S.One
-
-            if base_lim in (S.Zero, S.Infinity, S.NegativeInfinity) and ex_lim is S.Zero:
-                res = limit(f1, z, z0)
-                return exp(res)
-
-            if base_lim is S.NegativeInfinity:
-                if ex_lim is S.NegativeInfinity:
-                    return S.Zero
-                if ex_lim is S.Infinity:
-                    return S.ComplexInfinity
-
-            if not isinstance(base_lim, AccumBounds) and not isinstance(ex_lim, AccumBounds):
-                res = base_lim**ex_lim
-                if res is not S.ComplexInfinity and not res.is_Pow:
-                    return res
-
         l = None
 
         try:
-            if str(dir) == '+-':
-                r = gruntz(e, z, z0, '+')
-                l = gruntz(e, z, z0, '-')
-                if l != r:
-                    raise ValueError("The limit does not exist since "
-                            "left hand limit = %s and right hand limit = %s"
-                            % (l, r))
-            else:
-                r = gruntz(e, z, z0, dir)
+            r = gruntz(e, z, z0, dir)
             if r is S.NaN or l is S.NaN:
                 raise PoleError()
         except (PoleError, ValueError):
