@@ -1,42 +1,51 @@
-from __future__ import print_function, division
-
 from collections import defaultdict
 
+from sympy.concrete.products import Product
+from sympy.concrete.summations import Sum
 from sympy.core import (Basic, S, Add, Mul, Pow, Symbol, sympify,
                         expand_func, Function, Dummy, Expr, factor_terms,
                         expand_power_exp, Eq)
-from sympy.core.compatibility import iterable, ordered, as_int
+from sympy.core.exprtools import factor_nc
 from sympy.core.parameters import global_parameters
-from sympy.core.function import (expand_log, count_ops, _mexpand, _coeff_isneg,
-    nfloat, expand_mul)
-from sympy.core.numbers import Float, I, pi, Rational, Integer
+from sympy.core.function import (expand_log, count_ops, _mexpand,
+    nfloat, expand_mul, expand)
+from sympy.core.numbers import Float, I, pi, Rational
 from sympy.core.relational import Relational
 from sympy.core.rules import Transform
+from sympy.core.sorting import ordered
 from sympy.core.sympify import _sympify
+from sympy.core.traversal import bottom_up as _bottom_up, walk as _walk
 from sympy.functions import gamma, exp, sqrt, log, exp_polar, re
 from sympy.functions.combinatorial.factorials import CombinatorialFunction
-from sympy.functions.elementary.complexes import unpolarify, Abs
+from sympy.functions.elementary.complexes import unpolarify, Abs, sign
 from sympy.functions.elementary.exponential import ExpBase
 from sympy.functions.elementary.hyperbolic import HyperbolicFunction
 from sympy.functions.elementary.integers import ceiling
-from sympy.functions.elementary.piecewise import Piecewise, piecewise_fold
+from sympy.functions.elementary.piecewise import (Piecewise, piecewise_fold,
+                                                  piecewise_simplify)
 from sympy.functions.elementary.trigonometric import TrigonometricFunction
-from sympy.functions.special.bessel import besselj, besseli, besselk, jn, bessely
+from sympy.functions.special.bessel import (BesselBase, besselj, besseli,
+                                            besselk, bessely, jn)
 from sympy.functions.special.tensor_functions import KroneckerDelta
+from sympy.integrals.integrals import Integral
+from sympy.matrices.expressions import (MatrixExpr, MatAdd, MatMul,
+                                            MatPow, MatrixSymbol)
 from sympy.polys import together, cancel, factor
+from sympy.polys.numberfields.minpoly import _is_sum_surds, _minimal_polynomial_sq
 from sympy.simplify.combsimp import combsimp
 from sympy.simplify.cse_opts import sub_pre, sub_post
+from sympy.simplify.hyperexpand import hyperexpand
 from sympy.simplify.powsimp import powsimp
 from sympy.simplify.radsimp import radsimp, fraction, collect_abs
 from sympy.simplify.sqrtdenest import sqrtdenest
 from sympy.simplify.trigsimp import trigsimp, exptrigsimp
-from sympy.utilities.iterables import has_variety, sift
-
+from sympy.utilities.decorator import deprecated
+from sympy.utilities.iterables import has_variety, sift, subsets, iterable
+from sympy.utilities.misc import as_int
 
 import mpmath
 
 
-
 def separatevars(expr, symbols=[], dict=False, force=False):
     """
     Separates variables in an expression, if possible.  By
@@ -44,7 +53,10 @@ def separatevars(expr, symbols=[], dict=False, force=False):
     expression and collects constant coefficients that are
     independent of symbols.
 
-    If dict=True then the separated terms will be returned
+    Explanation
+    ===========
+
+    If ``dict=True`` then the separated terms will be returned
     in a dictionary keyed to their corresponding symbols.
     By default, all symbols in the expression will appear as
     keys; if symbols are provided, then all those symbols will
@@ -53,7 +65,7 @@ def separatevars(expr, symbols=[], dict=False, force=False):
     string 'coeff'. (Passing None for symbols will return the
     expression in a dictionary keyed to 'coeff'.)
 
-    If force=True, then bases of powers will be separated regardless
+    If ``force=True``, then bases of powers will be separated regardless
     of assumptions on the symbols involved.
 
     Notes
@@ -133,7 +145,7 @@ def _separatevars(expr, force):
         return expr
 
     # get a Pow ready for expansion
-    if expr.is_Pow:
+    if expr.is_Pow and expr.base != S.Exp1:
         expr = Pow(separatevars(expr.base, force=force), expr.exp)
 
     # First try other expansion methods
@@ -174,7 +186,7 @@ def _separatevars(expr, force):
 
 def _separatevars_dict(expr, symbols):
     if symbols:
-        if not all((t.is_Atom for t in symbols)):
+        if not all(t.is_Atom for t in symbols):
             raise ValueError("symbols must be Atoms.")
         symbols = list(symbols)
     elif symbols is None:
@@ -184,7 +196,7 @@ def _separatevars_dict(expr, symbols):
         if not symbols:
             return None
 
-    ret = dict(((i, []) for i in symbols + ['coeff']))
+    ret = {i: [] for i in symbols + ['coeff']}
 
     for i in Mul.make_args(expr):
         expsym = i.free_symbols
@@ -204,25 +216,20 @@ def _separatevars_dict(expr, symbols):
     return ret
 
 
-def _is_sum_surds(p):
-    args = p.args if p.is_Add else [p]
-    for y in args:
-        if not ((y**2).is_Rational and y.is_extended_real):
-            return False
-    return True
-
-
 def posify(eq):
-    """Return eq (with generic symbols made positive) and a
+    """Return ``eq`` (with generic symbols made positive) and a
     dictionary containing the mapping between the old and new
     symbols.
 
+    Explanation
+    ===========
+
     Any symbol that has positive=None will be replaced with a positive dummy
     symbol having the same name. This replacement will allow more symbolic
     processing of expressions, especially those involving powers and
     logarithms.
 
-    A dictionary that can be sent to subs to restore eq to its original
+    A dictionary that can be sent to subs to restore ``eq`` to its original
     symbols is also returned.
 
     >>> from sympy import posify, Symbol, log, solve
@@ -259,7 +266,7 @@ def posify(eq):
             syms = syms.union(e.atoms(Symbol))
         reps = {}
         for s in syms:
-            reps.update(dict((v, k) for k, v in posify(s)[1].items()))
+            reps.update({v: k for k, v in posify(s)[1].items()})
         for i, e in enumerate(eq):
             eq[i] = e.subs(reps)
         return f(eq), {r: s for s, r in reps.items()}
@@ -276,6 +283,9 @@ def hypersimp(f, k):
        integer sequences which have equivalent representation in terms
        of gamma special function.
 
+       Explanation
+       ===========
+
        The algorithm performs three basic steps:
 
        1. Rewrite all functions in terms of gamma, if possible.
@@ -315,13 +325,17 @@ def hypersimp(f, k):
 
 
 def hypersimilar(f, g, k):
-    """Returns True if 'f' and 'g' are hyper-similar.
+    """
+    Returns True if ``f`` and ``g`` are hyper-similar.
 
-       Similarity in hypergeometric sense means that a quotient of
-       f(k) and g(k) is a rational function in k.  This procedure
-       is useful in solving recurrence relations.
+    Explanation
+    ===========
 
-       For more information see hypersimp().
+    Similarity in hypergeometric sense means that a quotient of
+    f(k) and g(k) is a rational function in ``k``. This procedure
+    is useful in solving recurrence relations.
+
+    For more information see hypersimp().
 
     """
     f, g = list(map(sympify, (f, g)))
@@ -335,6 +349,9 @@ def hypersimilar(f, g, k):
 def signsimp(expr, evaluate=None):
     """Make all Add sub-expressions canonical wrt sign.
 
+    Explanation
+    ===========
+
     If an Add subexpression, ``a``, can have a sign extracted,
     as determined by could_extract_minus_sign, it is replaced
     with Mul(-1, a, evaluate=False). This allows signs to be
@@ -367,7 +384,7 @@ def signsimp(expr, evaluate=None):
     >>> n**i
     (-1 + 1/x)**i
 
-    By default, signsimp doesn't leave behind any hollow simplification:
+    By default, signsimp does not leave behind any hollow simplification:
     if making an Add canonical wrt sign didn't change the expression, the
     original Add is restored. If this is not desired then the keyword
     ``evaluate`` can be set to False:
@@ -384,19 +401,28 @@ def signsimp(expr, evaluate=None):
     expr = sympify(expr)
     if not isinstance(expr, (Expr, Relational)) or expr.is_Atom:
         return expr
-    e = sub_post(sub_pre(expr))
+    # get rid of an pre-existing unevaluation regarding sign
+    e = expr.replace(lambda x: x.is_Mul and -(-x) != x, lambda x: -(-x))
+    e = sub_post(sub_pre(e))
     if not isinstance(e, (Expr, Relational)) or e.is_Atom:
         return e
     if e.is_Add:
-        return e.func(*[signsimp(a, evaluate) for a in e.args])
+        rv = e.func(*[signsimp(a) for a in e.args])
+        if not evaluate and isinstance(rv, Add
+                ) and rv.could_extract_minus_sign():
+            return Mul(S.NegativeOne, -rv, evaluate=False)
+        return rv
     if evaluate:
-        e = e.xreplace({m: -(-m) for m in e.atoms(Mul) if -(-m) != m})
+        e = e.replace(lambda x: x.is_Mul and -(-x) != x, lambda x: -(-x))
     return e
 
 
 def simplify(expr, ratio=1.7, measure=count_ops, rational=False, inverse=False, doit=True, **kwargs):
     """Simplifies the given expression.
 
+    Explanation
+    ===========
+
     Simplification is not a well defined term and the exact strategies
     this function tries can change in the future versions of SymPy. If
     your algorithm relies on "simplification" (whatever it is), try to
@@ -405,7 +431,7 @@ def simplify(expr, ratio=1.7, measure=count_ops, rational=False, inverse=False,
     function directly, because those are well defined and thus your algorithm
     will be robust.
 
-    Nonetheless, especially for interactive use, or when you don't know
+    Nonetheless, especially for interactive use, or when you do not know
     anything about the structure of the expression, simplify() tries to apply
     intelligent heuristics to make the input expression "simpler".  For
     example:
@@ -438,7 +464,7 @@ def simplify(expr, ratio=1.7, measure=count_ops, rational=False, inverse=False,
     :func:`~.count_ops`, which returns the total number of operations in the
     expression.
 
-    For example, if ``ratio=1``, ``simplify`` output can't be longer
+    For example, if ``ratio=1``, ``simplify`` output cannot be longer
     than input.
 
     ::
@@ -466,7 +492,7 @@ def simplify(expr, ratio=1.7, measure=count_ops, rational=False, inverse=False,
     should represent the "size" or "complexity" of the input expression.  Note
     that some choices, such as ``lambda expr: len(str(expr))`` may appear to be
     good metrics, but have other problems (in this case, the measure function
-    may slow down simplify too much for very large expressions).  If you don't
+    may slow down simplify too much for very large expressions).  If you do not
     know what a good metric would be, the default, ``count_ops``, is a good
     one.
 
@@ -516,12 +542,12 @@ def simplify(expr, ratio=1.7, measure=count_ops, rational=False, inverse=False,
     function, we get a completely different result that is still different
     from the input expression by doing this.
 
-    If rational=True, Floats will be recast as Rationals before simplification.
-    If rational=None, Floats will be recast as Rationals but the result will
+    If ``rational=True``, Floats will be recast as Rationals before simplification.
+    If ``rational=None``, Floats will be recast as Rationals but the result will
     be recast as Floats. If rational=False(default) then nothing will be done
     to the Floats.
 
-    If inverse=True, it will be assumed that a composition of inverse
+    If ``inverse=True``, it will be assumed that a composition of inverse
     functions, such as sin and asin, can be cancelled in any order.
     For example, ``asin(sin(x))`` will yield ``x`` without checking whether
     x belongs to the set where this relation is true. The default is
@@ -530,6 +556,20 @@ def simplify(expr, ratio=1.7, measure=count_ops, rational=False, inverse=False,
     Note that ``simplify()`` automatically calls ``doit()`` on the final
     expression. You can avoid this behavior by passing ``doit=False`` as
     an argument.
+
+    Also, it should be noted that simplifying a boolean expression is not
+    well defined. If the expression prefers automatic evaluation (such as
+    :obj:`~.Eq()` or :obj:`~.Or()`), simplification will return ``True`` or
+    ``False`` if truth value can be determined. If the expression is not
+    evaluated by default (such as :obj:`~.Predicate()`), simplification will
+    not reduce it and you should use :func:`~.refine()` or :func:`~.ask()`
+    function. This inconsistency will be resolved in future version.
+
+    See Also
+    ========
+
+    sympy.assumptions.refine.refine : Simplification using assumptions.
+    sympy.assumptions.ask.ask : Query for boolean expressions using assumptions.
     """
 
     def shorter(*choices):
@@ -545,16 +585,16 @@ def simplify(expr, ratio=1.7, measure=count_ops, rational=False, inverse=False,
         rv = e.doit() if doit else e
         return shorter(rv, collect_abs(rv))
 
-    expr = sympify(expr)
-    kwargs = dict(
-        ratio=kwargs.get('ratio', ratio),
-        measure=kwargs.get('measure', measure),
-        rational=kwargs.get('rational', rational),
-        inverse=kwargs.get('inverse', inverse),
-        doit=kwargs.get('doit', doit))
+    expr = sympify(expr, rational=rational)
+    kwargs = {
+        "ratio": kwargs.get('ratio', ratio),
+        "measure": kwargs.get('measure', measure),
+        "rational": kwargs.get('rational', rational),
+        "inverse": kwargs.get('inverse', inverse),
+        "doit": kwargs.get('doit', doit)}
     # no routine for Expr needs to check for is_zero
     if isinstance(expr, Expr) and expr.is_zero:
-        return S.Zero
+        return S.Zero if not expr.is_Number else expr
 
     _eval_simplify = getattr(expr, '_eval_simplify', None)
     if _eval_simplify is not None:
@@ -599,7 +639,7 @@ def simplify(expr, ratio=1.7, measure=count_ops, rational=False, inverse=False,
         floats = True
         expr = nsimplify(expr, rational=True)
 
-    expr = bottom_up(expr, lambda w: getattr(w, 'normal', lambda: w)())
+    expr = _bottom_up(expr, lambda w: getattr(w, 'normal', lambda: w)())
     expr = Mul(*powsimp(expr).as_content_primitive())
     _e = cancel(expr)
     expr1 = shorter(_e, _mexpand(_e).cancel())  # issue 6829
@@ -614,11 +654,6 @@ def simplify(expr, ratio=1.7, measure=count_ops, rational=False, inverse=False,
 
     expr = factor_terms(expr, sign=False)
 
-    from sympy.simplify.hyperexpand import hyperexpand
-    from sympy.functions.special.bessel import BesselBase
-    from sympy import Sum, Product, Integral
-    from sympy.functions.elementary.complexes import sign
-
     # must come before `Piecewise` since this introduces more `Piecewise` terms
     if expr.has(sign):
         expr = expr.rewrite(Abs)
@@ -639,7 +674,6 @@ def simplify(expr, ratio=1.7, measure=count_ops, rational=False, inverse=False,
                 expr = kroneckersimp(expr)
             # Still a Piecewise?
             if expr.has(Piecewise):
-                from sympy.functions.elementary.piecewise import piecewise_simplify
                 # Do not apply doit on the segments as it has already
                 # been done above, but simplify
                 expr = piecewise_simplify(expr, deep=True, doit=False)
@@ -676,22 +710,22 @@ def simplify(expr, ratio=1.7, measure=count_ops, rational=False, inverse=False,
         expr = sum_simplify(expr, **kwargs)
 
     if expr.has(Integral):
-        expr = expr.xreplace(dict([
-            (i, factor_terms(i)) for i in expr.atoms(Integral)]))
+        expr = expr.xreplace({
+            i: factor_terms(i) for i in expr.atoms(Integral)})
 
     if expr.has(Product):
-        expr = product_simplify(expr)
+        expr = product_simplify(expr, **kwargs)
 
     from sympy.physics.units import Quantity
-    from sympy.physics.units.util import quantity_simplify
 
     if expr.has(Quantity):
+        from sympy.physics.units.util import quantity_simplify
         expr = quantity_simplify(expr)
 
     short = shorter(powsimp(expr, combine='exp', deep=True), powsimp(expr), expr)
     short = shorter(short, cancel(short))
     short = shorter(short, factor_terms(short), expand_power_exp(expand_mul(short)))
-    if short.has(TrigonometricFunction, HyperbolicFunction, ExpBase):
+    if short.has(TrigonometricFunction, HyperbolicFunction, ExpBase, exp):
         short = exptrigsimp(short)
 
     # get rid of hollow 2-arg Mul factorization
@@ -728,12 +762,9 @@ def simplify(expr, ratio=1.7, measure=count_ops, rational=False, inverse=False,
 
 def sum_simplify(s, **kwargs):
     """Main function for Sum simplification"""
-    from sympy.concrete.summations import Sum
-    from sympy.core.function import expand
-
     if not isinstance(s, Add):
-        s = s.xreplace(dict([(a, sum_simplify(a, **kwargs))
-            for a in s.atoms(Add) if a.has(Sum)]))
+        s = s.xreplace({a: sum_simplify(a, **kwargs)
+            for a in s.atoms(Add) if a.has(Sum)})
     s = expand(s)
     if not isinstance(s, Add):
         return s
@@ -762,8 +793,6 @@ def sum_combine(s_t):
        Attempts to simplify a list of sums, by combining limits / sum function's
        returns the simplified sum
     """
-    from sympy.concrete.summations import Sum
-
     used = [False] * len(s_t)
 
     for method in range(2):
@@ -772,7 +801,7 @@ def sum_combine(s_t):
                 for j, s_term2 in enumerate(s_t):
                     if not used[j] and i != j:
                         temp = sum_add(s_term1, s_term2, method)
-                        if isinstance(temp, Sum) or isinstance(temp, Mul):
+                        if isinstance(temp, (Sum, Mul)):
                             s_t[i] = temp
                             s_term1 = s_t[i]
                             used[j] = True
@@ -804,18 +833,14 @@ def factor_sum(self, limits=None, radical=False, clear=False, fraction=False, si
     y*Sum(x, (x, 1, 3))
     """
     # XXX deprecate in favor of direct call to factor_terms
-    from sympy.concrete.summations import Sum
-    kwargs = dict(radical=radical, clear=clear,
-        fraction=fraction, sign=sign)
+    kwargs = {"radical": radical, "clear": clear,
+        "fraction": fraction, "sign": sign}
     expr = Sum(self, *limits) if limits else self
     return factor_terms(expr, **kwargs)
 
 
 def sum_add(self, other, method=0):
     """Helper function for Sum simplification"""
-    from sympy.concrete.summations import Sum
-    from sympy import Mul
-
     #we know this is something in terms of a constant * a sum
     #so we temporarily put the constants inside for simplification
     #then simplify the result
@@ -835,7 +860,7 @@ def sum_add(self, other, method=0):
     else:
         rother = other
 
-    if type(rself) == type(rother):
+    if type(rself) is type(rother):
         if method == 0:
             if rself.limits == rother.limits:
                 return factor_sum(Sum(rself.function + rother.function, *rself.limits))
@@ -858,17 +883,20 @@ def sum_add(self, other, method=0):
     return Add(self, other)
 
 
-def product_simplify(s):
+def product_simplify(s, **kwargs):
     """Main function for Product simplification"""
-    from sympy.concrete.products import Product
-
     terms = Mul.make_args(s)
     p_t = [] # Product Terms
     o_t = [] # Other Terms
 
+    deep = kwargs.get('deep', True)
     for term in terms:
         if isinstance(term, Product):
-            p_t.append(term)
+            if deep:
+                p_t.append(Product(term.function.simplify(**kwargs),
+                                   *term.limits))
+            else:
+                p_t.append(term)
         else:
             o_t.append(term)
 
@@ -879,8 +907,9 @@ def product_simplify(s):
             if not used[i]:
                 for j, p_term2 in enumerate(p_t):
                     if not used[j] and i != j:
-                        if isinstance(product_mul(p_term1, p_term2, method), Product):
-                            p_t[i] = product_mul(p_term1, p_term2, method)
+                        tmp_prod = product_mul(p_term1, p_term2, method)
+                        if isinstance(tmp_prod, Product):
+                            p_t[i] = tmp_prod
                             used[j] = True
 
     result = Mul(*o_t)
@@ -894,9 +923,7 @@ def product_simplify(s):
 
 def product_mul(self, other, method=0):
     """Helper function for Product simplification"""
-    from sympy.concrete.products import Product
-
-    if type(self) == type(other):
+    if type(self) is type(other):
         if method == 0:
             if self.limits == other.limits:
                 return Product(self.function * other.function, *self.limits)
@@ -924,7 +951,6 @@ def _nthroot_solve(p, n, prec):
      helper function for ``nthroot``
      It denests ``p**Rational(1, n)`` using its minimal polynomial
     """
-    from sympy.polys.numberfields import _minimal_polynomial_sq
     from sympy.solvers import solve
     while n % 2 == 0:
         p = sqrtdenest(sqrt(p))
@@ -951,7 +977,7 @@ def logcombine(expr, force=False):
     - log(x) + log(y) == log(x*y) if both are positive
     - a*log(x) == log(x**a) if x is positive and a is real
 
-    If ``force`` is True then the assumptions above will be assumed to hold if
+    If ``force`` is ``True`` then the assumptions above will be assumed to hold if
     there is no assumption already in place on a quantity. For example, if
     ``a`` is imaginary or the argument negative, force will not perform a
     combination but if ``a`` is a symbol with no assumptions the change will
@@ -1069,7 +1095,7 @@ def logcombine(expr, force=False):
 
         # logs that have oppositely signed coefficients can divide
         for k in ordered(list(log1.keys())):
-            if not k in log1:  # already popped as -k
+            if k not in log1:  # already popped as -k
                 continue
             if -k in log1:
                 # figure out which has the minus sign; the one with
@@ -1085,12 +1111,15 @@ def logcombine(expr, force=False):
 
         return Add(*other)
 
-    return bottom_up(expr, f)
+    return _bottom_up(expr, f)
 
 
 def inversecombine(expr):
     """Simplify the composition of a function and its inverse.
 
+    Explanation
+    ===========
+
     No attention is paid to whether the inverse is a left inverse or a
     right inverse; thus, the result will in general not be equivalent
     to the original expression.
@@ -1108,65 +1137,19 @@ def inversecombine(expr):
     """
 
     def f(rv):
-        if rv.is_Function and hasattr(rv, "inverse"):
+        if isinstance(rv, log):
+            if isinstance(rv.args[0], exp) or (rv.args[0].is_Pow and rv.args[0].base == S.Exp1):
+                rv = rv.args[0].exp
+        elif rv.is_Function and hasattr(rv, "inverse"):
             if (len(rv.args) == 1 and len(rv.args[0].args) == 1 and
-                isinstance(rv.args[0], rv.inverse(argindex=1))):
-                    rv = rv.args[0].args[0]
+               isinstance(rv.args[0], rv.inverse(argindex=1))):
+                rv = rv.args[0].args[0]
+        if rv.is_Pow and rv.base == S.Exp1:
+            if isinstance(rv.exp, log):
+                rv = rv.exp.args[0]
         return rv
 
-    return bottom_up(expr, f)
-
-
-def walk(e, *target):
-    """iterate through the args that are the given types (target) and
-    return a list of the args that were traversed; arguments
-    that are not of the specified types are not traversed.
-
-    Examples
-    ========
-
-    >>> from sympy.simplify.simplify import walk
-    >>> from sympy import Min, Max
-    >>> from sympy.abc import x, y, z
-    >>> list(walk(Min(x, Max(y, Min(1, z))), Min))
-    [Min(x, Max(y, Min(1, z)))]
-    >>> list(walk(Min(x, Max(y, Min(1, z))), Min, Max))
-    [Min(x, Max(y, Min(1, z))), Max(y, Min(1, z)), Min(1, z)]
-
-    See Also
-    ========
-
-    bottom_up
-    """
-    if isinstance(e, target):
-        yield e
-        for i in e.args:
-            for w in walk(i, *target):
-                yield w
-
-
-def bottom_up(rv, F, atoms=False, nonbasic=False):
-    """Apply ``F`` to all expressions in an expression tree from the
-    bottom up. If ``atoms`` is True, apply ``F`` even if there are no args;
-    if ``nonbasic`` is True, try to apply ``F`` to non-Basic objects.
-    """
-    args = getattr(rv, 'args', None)
-    if args is not None:
-        if args:
-            args = tuple([bottom_up(a, F, atoms, nonbasic) for a in args])
-            if args != rv.args:
-                rv = rv.func(*args)
-            rv = F(rv)
-        elif atoms:
-            rv = F(rv)
-    else:
-        if nonbasic:
-            try:
-                rv = F(rv)
-            except TypeError:
-                pass
-
-    return rv
+    return _bottom_up(expr, f)
 
 
 def kroneckersimp(expr):
@@ -1175,6 +1158,9 @@ def kroneckersimp(expr):
 
     The only simplification currently attempted is to identify multiplicative cancellation:
 
+    Examples
+    ========
+
     >>> from sympy import KroneckerDelta, kroneckersimp
     >>> from sympy.abc import i
     >>> kroneckersimp(1 + KroneckerDelta(0, i) * KroneckerDelta(1, i))
@@ -1192,15 +1178,13 @@ def kroneckersimp(expr):
         return False
 
     def cancel_kronecker_mul(m):
-        from sympy.utilities.iterables import subsets
-
         args = m.args
         deltas = [a for a in args if isinstance(a, KroneckerDelta)]
         for delta1, delta2 in subsets(deltas, 2):
             args1 = delta1.args
             args2 = delta2.args
             if args_cancel(args1, args2):
-                return 0*m
+                return S.Zero * m # In case of oo etc
         return m
 
     if not expr.has(KroneckerDelta):
@@ -1223,6 +1207,9 @@ def besselsimp(expr):
     """
     Simplify bessel-type functions.
 
+    Explanation
+    ===========
+
     This routine tries to simplify bessel-type functions. Currently it only
     works on the Bessel J and I functions, however. It works by looking at all
     such functions in turn, and eliminating factors of "I" and "-1" (actually
@@ -1333,7 +1320,7 @@ def besselsimp(expr):
 
 def nthroot(expr, n, max_len=4, prec=15):
     """
-    compute a real nth-root of a sum of surds
+    Compute a real nth-root of a sum of surds.
 
     Parameters
     ==========
@@ -1395,10 +1382,13 @@ def nsimplify(expr, constants=(), tolerance=None, full=False, rational=None,
     rational_conversion='base10'):
     """
     Find a simple representation for a number or, if there are free symbols or
-    if rational=True, then replace Floats with their Rational equivalents. If
+    if ``rational=True``, then replace Floats with their Rational equivalents. If
     no change is made and rational is not False then Floats will at least be
     converted to Rationals.
 
+    Explanation
+    ===========
+
     For numerical expressions, a simple formula that numerically matches the
     given numerical expression is sought (and the input should be possible
     to evalf to a precision of at least 30 digits).
@@ -1410,7 +1400,7 @@ def nsimplify(expr, constants=(), tolerance=None, full=False, rational=None,
     is given then the least precise value will set the tolerance (e.g. Floats
     default to 15 digits of precision, so would be tolerance=10**-15).
 
-    With full=True, a more extensive search is performed
+    With ``full=True``, a more extensive search is performed
     (this is useful to find simpler numbers when the tolerance
     is set low).
 
@@ -1502,7 +1492,7 @@ def nsimplify(expr, constants=(), tolerance=None, full=False, rational=None,
             expr = sympify(newexpr)
             if x and not expr:  # don't let x become 0
                 raise ValueError
-            if expr.is_finite is False and not xv in [mpmath.inf, mpmath.ninf]:
+            if expr.is_finite is False and xv not in [mpmath.inf, mpmath.ninf]:
                 raise ValueError
             return expr
         finally:
@@ -1577,17 +1567,17 @@ def _real_to_rational(expr, tolerance=None, rational_conversion='base10'):
             if fl and not r:
                 r = Rational(fl)
             elif not r.is_Rational:
-                if fl == inf or fl == -inf:
+                if fl in (inf, -inf):
                     r = S.ComplexInfinity
                 elif fl < 0:
                     fl = -fl
-                    d = Pow(10, int((mpmath.log(fl)/mpmath.log(10))))
+                    d = Pow(10, int(mpmath.log(fl)/mpmath.log(10)))
                     r = -Rational(str(fl/d))*d
                 elif fl > 0:
-                    d = Pow(10, int((mpmath.log(fl)/mpmath.log(10))))
+                    d = Pow(10, int(mpmath.log(fl)/mpmath.log(10)))
                     r = Rational(str(fl/d))*d
                 else:
-                    r = Integer(0)
+                    r = S.Zero
         reps[key] = r
     return p.subs(reps, simultaneous=True)
 
@@ -1633,7 +1623,7 @@ def clear_coefficients(expr, rhs=S.Zero):
         c, expr = expr.as_coeff_Add(rational=True)
         rhs -= c
     expr = signsimp(expr, evaluate = False)
-    if _coeff_isneg(expr):
+    if expr.could_extract_minus_sign():
         expr = -expr
         rhs = -rhs
     return expr, rhs
@@ -1645,13 +1635,13 @@ def nc_simplify(expr, deep=True):
     Priority is given to simplifications that give the fewest number
     of arguments in the end (for example, in a*b*a*b*c*a*b*c simplifying
     to (a*b)**2*c*a*b*c gives 5 arguments while a*b*(a*b*c)**2 has 3).
-    If `expr` is a sum of such terms, the sum of the simplified terms
+    If ``expr`` is a sum of such terms, the sum of the simplified terms
     is returned.
 
-    Keyword argument `deep` controls whether or not subexpressions
+    Keyword argument ``deep`` controls whether or not subexpressions
     nested deeper inside the main expression are simplified. See examples
     below. Setting `deep` to `False` can save time on nested expressions
-    that don't need simplifying on all levels.
+    that do not need simplifying on all levels.
 
     Examples
     ========
@@ -1679,10 +1669,6 @@ def nc_simplify(expr, deep=True):
     (a*b*a*b)**2*(a*c)**2
 
     '''
-    from sympy.matrices.expressions import (MatrixExpr, MatAdd, MatMul,
-                                                MatPow, MatrixSymbol)
-    from sympy.core.exprtools import factor_nc
-
     if isinstance(expr, MatrixExpr):
         expr = expr.doit(inv_expand=False)
         _Add, _Mul, _Pow, _Symbol = MatAdd, MatMul, MatPow, MatrixSymbol
@@ -1724,7 +1710,7 @@ def nc_simplify(expr, deep=True):
         inverses = []
         args = []
         for arg in _args:
-            if isinstance(arg, _Pow) and arg.args[1] < 0:
+            if isinstance(arg, _Pow) and arg.args[1].is_extended_negative:
                 inverses = [arg**-1] + inverses
                 inv_tot += 1
             else:
@@ -2017,7 +2003,7 @@ def dotprodsimp(expr, withsimp=False):
                     ops += bool (a.p < 0) + bool (a.q != 1)
 
             elif a.is_Mul:
-                if _coeff_isneg(a):
+                if a.could_extract_minus_sign():
                     ops += 1
                     if a.args[0] is S.NegativeOne:
                         a = a.as_two_terms()[1]
@@ -2047,7 +2033,7 @@ def dotprodsimp(expr, withsimp=False):
                 negs   = 0
 
                 for ai in a.args:
-                    if _coeff_isneg(ai):
+                    if ai.could_extract_minus_sign():
                         negs += 1
                         ai    = -ai
                     args.append(ai)
@@ -2134,3 +2120,31 @@ def dotprodsimp(expr, withsimp=False):
             simplified = True
 
     return (expr, simplified) if withsimp else expr
+
+
+bottom_up = deprecated(
+    """
+    Using bottom_up from the sympy.simplify.simplify submodule is
+    deprecated.
+
+    Instead, use bottom_up from the top-level sympy namespace, like
+
+        sympy.bottom_up
+    """,
+    deprecated_since_version="1.10",
+    active_deprecations_target="deprecated-traversal-functions-moved",
+)(_bottom_up)
+
+
+# XXX: This function really should either be private API or exported in the
+# top-level sympy/__init__.py
+walk = deprecated(
+    """
+    Using walk from the sympy.simplify.simplify submodule is
+    deprecated.
+
+    Instead, use walk from sympy.core.traversal.walk
+    """,
+    deprecated_since_version="1.10",
+    active_deprecations_target="deprecated-traversal-functions-moved",
+)(_walk)
