@@ -9,19 +9,28 @@ sympy.stats.frv
 """
 
 
-from sympy import (Interval, Intersection, symbols, sympify, Dummy, nan,
-        Integral, And, Or, Piecewise, cacheit, integrate, oo, Lambda,
-        Basic, S, exp, I, FiniteSet, Ne, Eq, Union, poly, series, factorial,
-        lambdify)
-from sympy.core.function import PoleError
+from sympy.core.basic import Basic
+from sympy.core.cache import cacheit
+from sympy.core.function import Lambda, PoleError
+from sympy.core.numbers import (I, nan, oo)
+from sympy.core.relational import (Eq, Ne)
+from sympy.core.singleton import S
+from sympy.core.symbol import (Dummy, symbols)
+from sympy.core.sympify import _sympify, sympify
+from sympy.functions.combinatorial.factorials import factorial
+from sympy.functions.elementary.exponential import exp
+from sympy.functions.elementary.piecewise import Piecewise
 from sympy.functions.special.delta_functions import DiracDelta
+from sympy.integrals.integrals import (Integral, integrate)
+from sympy.logic.boolalg import (And, Or)
 from sympy.polys.polyerrors import PolynomialError
+from sympy.polys.polytools import poly
+from sympy.series.series import series
+from sympy.sets.sets import (FiniteSet, Intersection, Interval, Union)
 from sympy.solvers.solveset import solveset
 from sympy.solvers.inequalities import reduce_rational_inequalities
-from sympy.core.sympify import _sympify
-from sympy.external import import_module
 from sympy.stats.rv import (RandomDomain, SingleDomain, ConditionalDomain, is_random,
-        ProductDomain, PSpace, SinglePSpace, random_symbols, NamedArgsMixin)
+        ProductDomain, PSpace, SinglePSpace, random_symbols, NamedArgsMixin, Distribution)
 
 
 class ContinuousDomain(RandomDomain):
@@ -77,7 +86,7 @@ class ProductContinuousDomain(ProductDomain, ContinuousDomain):
 class ConditionalContinuousDomain(ContinuousDomain, ConditionalDomain):
     """
     A domain with continuous support that has been further restricted by a
-    condition such as x > 3
+    condition such as $x > 3$.
     """
 
     def compute_expectation(self, expr, variables=None, **kwargs):
@@ -140,173 +149,23 @@ class ConditionalContinuousDomain(ContinuousDomain, ConditionalDomain):
                 "Set of Conditional Domain not Implemented")
 
 
-class ContinuousDistribution(Basic):
+class ContinuousDistribution(Distribution):
     def __call__(self, *args):
         return self.pdf(*args)
 
 
-class SampleContinuousScipy:
-    """Returns the sample from scipy of the given distribution"""
-    def __new__(cls, dist, size, seed=None):
-        return cls._sample_scipy(dist, size, seed=seed)
-
-    @classmethod
-    def _sample_scipy(cls, dist, size, seed):
-        """Sample from SciPy."""
-        import scipy.stats
-        # scipy does not require map as it can handle using custom distributions.
-        # However, we will still use a map where we can.
-
-        # TODO: do this for drv.py and frv.py if necessary.
-        # TODO: add more distributions here if there are more
-        # See links below referring to sections beginning with "A common parametrization..."
-        # I will remove all these comments if everything is ok.
-        scipy_rv_map = {
-            'BetaDistribution': lambda dist, size: scipy.stats.beta.rvs(
-                a=float(dist.alpha), b=float(dist.beta), size=size, random_state=seed),
-            # same parametrisation
-            'CauchyDistribution': lambda dist, size: scipy.stats.cauchy.rvs(
-                loc=float(dist.x0), scale=float(dist.gamma), size=size, random_state=seed),
-            # same parametrisation
-            'ChiSquaredDistribution': lambda dist, size: scipy.stats.chi2.rvs(
-                df=float(dist.k), size=size, random_state=seed),
-            # same parametrisation
-            'ExponentialDistribution': lambda dist, size: scipy.stats.expon.rvs(
-                scale=1 / float(dist.rate), size=size, random_state=seed),
-            # https://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.expon.html#scipy.stats.expon
-            'GammaDistribution': lambda dist, size: scipy.stats.gamma.rvs(
-                a=float(dist.k), scale=float(dist.theta), size=size, random_state=seed),
-            # https://stackoverflow.com/questions/42150965/how-to-plot-gamma-distribution-with-alpha-and-beta-parameters-in-python
-            'LogNormalDistribution': lambda dist, size: scipy.stats.lognorm.rvs(
-                scale=float(exp(dist.mean)), s=float(dist.std), size=size, random_state=seed),
-            # https://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.lognorm.html
-            'NormalDistribution': lambda dist, size: scipy.stats.norm.rvs(
-                loc=float(dist.mean), scale=float(dist.std), size=size, random_state=seed),
-            # same parametrisation
-            'ParetoDistribution': lambda dist, size: scipy.stats.pareto.rvs(
-                b=float(dist.alpha), scale=float(dist.xm), size=size, random_state=seed),
-            # https://stackoverflow.com/questions/42260519/defining-pareto-distribution-in-python-scipy
-            'StudentTDistribution': lambda dist, size: scipy.stats.t.rvs(
-                df=float(dist.nu), size=size, random_state=seed),
-            # same parametrisation
-            'UniformDistribution': lambda dist, size: scipy.stats.uniform.rvs(
-                loc=float(dist.left), scale=float(dist.right - dist.left), size=size, random_state=seed)
-            # https://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.uniform.html
-        }
-
-        # if we don't need to make a handmade pdf, we won't
-        if dist.__class__.__name__ in scipy_rv_map:
-            return scipy_rv_map[dist.__class__.__name__](dist, size)
-
-        z = Dummy('z')
-        handmade_pdf = lambdify(z, dist.pdf(z), ['numpy', 'scipy'])
-        class scipy_pdf(scipy.stats.rv_continuous):
-            def _pdf(self, x):
-                return handmade_pdf(x)
-        scipy_rv = scipy_pdf(a=float(dist.set._inf),
-                    b=float(dist.set._sup), name='scipy_pdf')
-        return scipy_rv.rvs(size=size, random_state=seed)
-
-class SampleContinuousNumpy:
-    """Returns the sample from numpy of the given distribution"""
-
-    def __new__(cls, dist, size, seed=None):
-        return cls._sample_numpy(dist, size, seed)
-
-    @classmethod
-    def _sample_numpy(cls, dist, size, seed):
-        """Sample from NumPy."""
-
-        import numpy
-        if seed is None or isinstance(seed, int):
-            rand_state = numpy.random.default_rng(seed=seed)
-        else:
-            rand_state = seed
-        numpy_rv_map = {
-            'BetaDistribution': lambda dist, size: rand_state.beta(a=float(dist.alpha),
-                b=float(dist.beta), size=size),
-            'ChiSquaredDistribution': lambda dist, size: rand_state.chisquare(
-                df=float(dist.k), size=size),
-            'ExponentialDistribution': lambda dist, size: rand_state.exponential(
-                1/float(dist.rate), size=size),
-            'GammaDistribution': lambda dist, size: rand_state.gamma(float(dist.k),
-                float(dist.theta), size=size),
-            'LogNormalDistribution': lambda dist, size: rand_state.lognormal(
-                float(dist.mean), float(dist.std), size=size),
-            'NormalDistribution': lambda dist, size: rand_state.normal(
-                float(dist.mean), float(dist.std), size=size),
-            'ParetoDistribution': lambda dist, size: (rand_state.pareto(
-                a=float(dist.alpha), size=size) + 1) * float(dist.xm),
-            'UniformDistribution': lambda dist, size: rand_state.uniform(
-                low=float(dist.left), high=float(dist.right), size=size)
-        }
-
-        dist_list = numpy_rv_map.keys()
-
-        if dist.__class__.__name__ not in dist_list:
-            return None
-
-        return numpy_rv_map[dist.__class__.__name__](dist, size)
-
-class SampleContinuousPymc:
-    """Returns the sample from pymc3 of the given distribution"""
-
-    def __new__(cls, dist, size, seed=None):
-        return cls._sample_pymc3(dist, size, seed)
-
-    @classmethod
-    def _sample_pymc3(cls, dist, size, seed):
-        """Sample from PyMC3."""
-
-        import pymc3
-        pymc3_rv_map = {
-            'BetaDistribution': lambda dist:
-                pymc3.Beta('X', alpha=float(dist.alpha), beta=float(dist.beta)),
-            'CauchyDistribution': lambda dist:
-                pymc3.Cauchy('X', alpha=float(dist.x0), beta=float(dist.gamma)),
-            'ChiSquaredDistribution': lambda dist:
-                pymc3.ChiSquared('X', nu=float(dist.k)),
-            'ExponentialDistribution': lambda dist:
-                pymc3.Exponential('X', lam=float(dist.rate)),
-            'GammaDistribution': lambda dist:
-                pymc3.Gamma('X', alpha=float(dist.k), beta=1/float(dist.theta)),
-            'LogNormalDistribution': lambda dist:
-                pymc3.Lognormal('X', mu=float(dist.mean), sigma=float(dist.std)),
-            'NormalDistribution': lambda dist:
-                pymc3.Normal('X', float(dist.mean), float(dist.std)),
-            'GaussianInverseDistribution': lambda dist:
-                pymc3.Wald('X', mu=float(dist.mean), lam=float(dist.shape)),
-            'ParetoDistribution': lambda dist:
-                pymc3.Pareto('X', alpha=float(dist.alpha), m=float(dist.xm)),
-            'UniformDistribution': lambda dist:
-                pymc3.Uniform('X', lower=float(dist.left), upper=float(dist.right))
-        }
-
-        dist_list = pymc3_rv_map.keys()
-
-        if dist.__class__.__name__ not in dist_list:
-            return None
-
-        with pymc3.Model():
-            pymc3_rv_map[dist.__class__.__name__](dist)
-            return pymc3.sample(size, chains=1, progressbar=False, random_seed=seed)[:]['X']
-
-_get_sample_class_crv = {
-    'scipy': SampleContinuousScipy,
-    'pymc3': SampleContinuousPymc,
-    'numpy': SampleContinuousNumpy
-}
-
-
 class SingleContinuousDistribution(ContinuousDistribution, NamedArgsMixin):
-    """ Continuous distribution of a single variable
+    """ Continuous distribution of a single variable.
+
+    Explanation
+    ===========
 
     Serves as superclass for Normal/Exponential/UniformDistribution etc....
 
     Represented by parameters for each of the specific classes.  E.g
     NormalDistribution is represented by a mean and standard deviation.
 
-    Provides methods for pdf, cdf, and sampling
+    Provides methods for pdf, cdf, and sampling.
 
     See Also
     ========
@@ -324,31 +183,11 @@ class SingleContinuousDistribution(ContinuousDistribution, NamedArgsMixin):
     def check(*args):
         pass
 
-    def sample(self, size=(), library='scipy', seed=None):
-        """ A random realization from the distribution """
-
-        libraries = ['scipy', 'numpy', 'pymc3']
-        if library not in libraries:
-            raise NotImplementedError("Sampling from %s is not supported yet."
-                                        % str(library))
-        if not import_module(library):
-            raise ValueError("Failed to import %s" % library)
-
-        samps = _get_sample_class_crv[library](self, size, seed)
-
-        if samps is not None:
-            return samps
-        raise NotImplementedError(
-                "Sampling for %s is not currently implemented from %s"
-                % (self.__class__.__name__, library)
-                )
-
-
     @cacheit
     def compute_cdf(self, **kwargs):
-        """ Compute the CDF from the PDF
+        """ Compute the CDF from the PDF.
 
-        Returns a Lambda
+        Returns a Lambda.
         """
         x, z = symbols('x, z', real=True, cls=Dummy)
         left_bound = self.set.start
@@ -373,9 +212,9 @@ class SingleContinuousDistribution(ContinuousDistribution, NamedArgsMixin):
 
     @cacheit
     def compute_characteristic_function(self, **kwargs):
-        """ Compute the characteristic function from the PDF
+        """ Compute the characteristic function from the PDF.
 
-        Returns a Lambda
+        Returns a Lambda.
         """
         x, t = symbols('x, t', real=True, cls=Dummy)
         pdf = self.pdf(x)
@@ -395,9 +234,9 @@ class SingleContinuousDistribution(ContinuousDistribution, NamedArgsMixin):
 
     @cacheit
     def compute_moment_generating_function(self, **kwargs):
-        """ Compute the moment generating function from the PDF
+        """ Compute the moment generating function from the PDF.
 
-        Returns a Lambda
+        Returns a Lambda.
         """
         x, t = symbols('x, t', real=True, cls=Dummy)
         pdf = self.pdf(x)
@@ -420,6 +259,8 @@ class SingleContinuousDistribution(ContinuousDistribution, NamedArgsMixin):
         if evaluate:
             try:
                 p = poly(expr, var)
+                if p.is_zero:
+                    return S.Zero
                 t = Dummy('t', real=True)
                 mgf = self._moment_generating_function(t)
                 if mgf is None:
@@ -437,9 +278,9 @@ class SingleContinuousDistribution(ContinuousDistribution, NamedArgsMixin):
 
     @cacheit
     def compute_quantile(self, **kwargs):
-        """ Compute the Quantile from the PDF
+        """ Compute the Quantile from the PDF.
 
-        Returns a Lambda
+        Returns a Lambda.
         """
         x, p = symbols('x, p', real=True, cls=Dummy)
         left_bound = self.set.start
@@ -623,7 +464,7 @@ class ContinuousPSpace(PSpace):
 
 class SingleContinuousPSpace(ContinuousPSpace, SinglePSpace):
     """
-    A continuous probability space over a single univariate variable
+    A continuous probability space over a single univariate variable.
 
     These consist of a Symbol and a SingleContinuousDistribution
 
@@ -641,7 +482,7 @@ class SingleContinuousPSpace(ContinuousPSpace, SinglePSpace):
 
     def sample(self, size=(), library='scipy', seed=None):
         """
-        Internal sample method
+        Internal sample method.
 
         Returns dictionary mapping RandomSymbol to realization value.
         """
