@@ -1,139 +1,40 @@
-from sympy import (Basic, sympify, symbols, Dummy, Lambda, summation,
-                   Piecewise, S, cacheit, Sum, exp, I, Ne, Eq, poly,
-                   series, factorial, And, lambdify)
+from sympy.concrete.summations import (Sum, summation)
+from sympy.core.basic import Basic
+from sympy.core.cache import cacheit
+from sympy.core.function import Lambda
+from sympy.core.numbers import I
+from sympy.core.relational import (Eq, Ne)
+from sympy.core.singleton import S
+from sympy.core.symbol import (Dummy, symbols)
+from sympy.core.sympify import sympify
+from sympy.functions.combinatorial.factorials import factorial
+from sympy.functions.elementary.exponential import exp
+from sympy.functions.elementary.integers import floor
+from sympy.functions.elementary.piecewise import Piecewise
+from sympy.logic.boolalg import And
+from sympy.polys.polytools import poly
+from sympy.series.series import series
 
 from sympy.polys.polyerrors import PolynomialError
 from sympy.stats.crv import reduce_rational_inequalities_wrap
 from sympy.stats.rv import (NamedArgsMixin, SinglePSpace, SingleDomain,
                             random_symbols, PSpace, ConditionalDomain, RandomDomain,
-                            ProductDomain)
+                            ProductDomain, Distribution)
 from sympy.stats.symbolic_probability import Probability
 from sympy.sets.fancysets import Range, FiniteSet
 from sympy.sets.sets import Union
 from sympy.sets.contains import Contains
 from sympy.utilities import filldedent
 from sympy.core.sympify import _sympify
-from sympy.external import import_module
 
 
-class DiscreteDistribution(Basic):
+class DiscreteDistribution(Distribution):
     def __call__(self, *args):
         return self.pdf(*args)
 
 
-class SampleDiscreteScipy:
-    """Returns the sample from scipy of the given distribution"""
-    def __new__(cls, dist, size, seed=None):
-        return cls._sample_scipy(dist, size, seed)
-
-    @classmethod
-    def _sample_scipy(cls, dist, size, seed):
-        """Sample from SciPy."""
-
-        from scipy import stats as scipy_stats
-        scipy_rv_map = {
-            'GeometricDistribution': lambda dist, size: scipy_stats.geom.rvs(p=float(dist.p),
-                size=size, random_state=seed),
-            'LogarithmicDistribution': lambda dist, size: scipy_stats.logser.rvs(p=float(dist.p),
-                size=size, random_state=seed),
-            'NegativeBinomialDistribution': lambda dist, size: scipy_stats.nbinom.rvs(n=float(dist.r),
-                p=float(dist.p), size=size, random_state=seed),
-            'PoissonDistribution': lambda dist, size: scipy_stats.poisson.rvs(mu=float(dist.lamda),
-                size=size, random_state=seed),
-            'SkellamDistribution': lambda dist, size: scipy_stats.skellam.rvs(mu1=float(dist.mu1),
-                mu2=float(dist.mu2), size=size, random_state=seed),
-            'YuleSimonDistribution': lambda dist, size: scipy_stats.yulesimon.rvs(alpha=float(dist.rho),
-                size=size, random_state=seed),
-            'ZetaDistribution': lambda dist, size: scipy_stats.zipf.rvs(a=float(dist.s),
-                size=size, random_state=seed)
-        }
-
-        dist_list = scipy_rv_map.keys()
-
-        if dist.__class__.__name__ == 'DiscreteDistributionHandmade':
-            from scipy.stats import rv_discrete
-            z = Dummy('z')
-            handmade_pmf = lambdify(z, dist.pdf(z), ['numpy', 'scipy'])
-            class scipy_pmf(rv_discrete):
-                def _pmf(self, x):
-                    return handmade_pmf(x)
-            scipy_rv = scipy_pmf(a=float(dist.set._inf), b=float(dist.set._sup),
-                        name='scipy_pmf')
-            return scipy_rv.rvs(size=size, random_state=seed)
-
-        if dist.__class__.__name__ not in dist_list:
-            return None
-
-        return scipy_rv_map[dist.__class__.__name__](dist, size)
-
-class SampleDiscreteNumpy:
-    """Returns the sample from numpy of the given distribution"""
-
-    def __new__(cls, dist, size, seed=None):
-        return cls._sample_numpy(dist, size, seed)
-
-    @classmethod
-    def _sample_numpy(cls, dist, size, seed):
-        """Sample from NumPy."""
-
-        import numpy
-        if seed is None or isinstance(seed, int):
-            rand_state = numpy.random.default_rng(seed=seed)
-        else:
-            rand_state = seed
-        numpy_rv_map = {
-            'GeometricDistribution': lambda dist, size: rand_state.geometric(p=float(dist.p),
-                size=size),
-            'PoissonDistribution': lambda dist, size: rand_state.poisson(lam=float(dist.lamda),
-                size=size),
-            'ZetaDistribution': lambda dist, size: rand_state.zipf(a=float(dist.s),
-                size=size)
-        }
-
-        dist_list = numpy_rv_map.keys()
-
-        if dist.__class__.__name__ not in dist_list:
-            return None
-
-        return numpy_rv_map[dist.__class__.__name__](dist, size)
-
-class SampleDiscretePymc:
-    """Returns the sample from pymc3 of the given distribution"""
-
-    def __new__(cls, dist, size, seed=None):
-        return cls._sample_pymc3(dist, size, seed)
-
-    @classmethod
-    def _sample_pymc3(cls, dist, size, seed):
-        """Sample from PyMC3."""
-
-        import pymc3
-        pymc3_rv_map = {
-            'GeometricDistribution': lambda dist: pymc3.Geometric('X', p=float(dist.p)),
-            'PoissonDistribution': lambda dist: pymc3.Poisson('X', mu=float(dist.lamda)),
-            'NegativeBinomialDistribution': lambda dist: pymc3.NegativeBinomial('X',
-            mu=float((dist.p*dist.r)/(1-dist.p)), alpha=float(dist.r))
-        }
-
-        dist_list = pymc3_rv_map.keys()
-
-        if dist.__class__.__name__ not in dist_list:
-            return None
-
-        with pymc3.Model():
-            pymc3_rv_map[dist.__class__.__name__](dist)
-            return pymc3.sample(size, chains=1, progressbar=False, random_seed=seed)[:]['X']
-
-
-_get_sample_class_drv = {
-    'scipy': SampleDiscreteScipy,
-    'pymc3': SampleDiscretePymc,
-    'numpy': SampleDiscreteNumpy
-}
-
-
 class SingleDiscreteDistribution(DiscreteDistribution, NamedArgsMixin):
-    """ Discrete distribution of a single variable
+    """ Discrete distribution of a single variable.
 
     Serves as superclass for PoissonDistribution etc....
 
@@ -153,38 +54,19 @@ class SingleDiscreteDistribution(DiscreteDistribution, NamedArgsMixin):
     def check(*args):
         pass
 
-    def sample(self, size=(), library='scipy', seed=None):
-        """ A random realization from the distribution"""
-
-        libraries = ['scipy', 'numpy', 'pymc3']
-        if library not in libraries:
-            raise NotImplementedError("Sampling from %s is not supported yet."
-                                        % str(library))
-        if not import_module(library):
-            raise ValueError("Failed to import %s" % library)
-
-        samps = _get_sample_class_drv[library](self, size, seed)
-
-        if samps is not None:
-            return samps
-        raise NotImplementedError(
-                "Sampling for %s is not currently implemented from %s"
-                % (self.__class__.__name__, library)
-                )
-
-
     @cacheit
     def compute_cdf(self, **kwargs):
-        """ Compute the CDF from the PDF
+        """ Compute the CDF from the PDF.
 
-        Returns a Lambda
+        Returns a Lambda.
         """
-        x, z = symbols('x, z', integer=True, cls=Dummy)
+        x = symbols('x', integer=True, cls=Dummy)
+        z = symbols('z', real=True, cls=Dummy)
         left_bound = self.set.inf
 
         # CDF is integral of PDF from left bound to z
         pdf = self.pdf(x)
-        cdf = summation(pdf, (x, left_bound, z), **kwargs)
+        cdf = summation(pdf, (x, left_bound, floor(z)), **kwargs)
         # CDF Ensure that CDF left of left_bound is zero
         cdf = Piecewise((cdf, z >= left_bound), (0, True))
         return Lambda(z, cdf)
@@ -202,9 +84,9 @@ class SingleDiscreteDistribution(DiscreteDistribution, NamedArgsMixin):
 
     @cacheit
     def compute_characteristic_function(self, **kwargs):
-        """ Compute the characteristic function from the PDF
+        """ Compute the characteristic function from the PDF.
 
-        Returns a Lambda
+        Returns a Lambda.
         """
         x, t = symbols('x, t', real=True, cls=Dummy)
         pdf = self.pdf(x)
@@ -242,9 +124,9 @@ class SingleDiscreteDistribution(DiscreteDistribution, NamedArgsMixin):
 
     @cacheit
     def compute_quantile(self, **kwargs):
-        """ Compute the Quantile from the PDF
+        """ Compute the Quantile from the PDF.
 
-        Returns a Lambda
+        Returns a Lambda.
         """
         x = Dummy('x', integer=True)
         p = Dummy('p', real=True)
@@ -412,7 +294,7 @@ class SingleDiscretePSpace(DiscretePSpace, SinglePSpace):
 
     def sample(self, size=(), library='scipy', seed=None):
         """
-        Internal sample method
+        Internal sample method.
 
         Returns dictionary mapping RandomSymbol to realization value.
         """
