@@ -1,5 +1,7 @@
 from sympy.tensor.functions import TensorProduct
-from sympy import MatrixSymbol, Matrix, Array
+from sympy.matrices.dense import Matrix
+from sympy.matrices.expressions.matexpr import MatrixSymbol
+from sympy.tensor.array import Array
 from sympy.abc import x, y, z
 from sympy.abc import i, j, k, l
 
@@ -19,7 +21,7 @@ def test_TensorProduct_construction():
     expr = TensorProduct(TensorProduct(A, B), C)
     assert expr == TensorProduct(A, B, C)
 
-    expr = TensorProduct(Matrix.eye(2), [[0, -1], [1, 0]])
+    expr = TensorProduct(Matrix.eye(2), Array([[0, -1], [1, 0]]))
     assert expr == Array([
         [
             [[0, -1], [1, 0]],
@@ -38,14 +40,14 @@ def test_TensorProduct_shape():
     assert expr.shape == ()
     assert expr.rank() == 0
 
-    expr = TensorProduct([1, 2], [x, y], evaluate=False)
+    expr = TensorProduct(Array([1, 2]), Array([x, y]), evaluate=False)
     assert expr.shape == (2, 2)
     assert expr.rank() == 2
     expr = TensorProduct(expr, expr, evaluate=False)
     assert expr.shape == (2, 2, 2, 2)
     assert expr.rank() == 4
 
-    expr = TensorProduct(Matrix.eye(2), [[0, -1], [1, 0]], evaluate=False)
+    expr = TensorProduct(Matrix.eye(2), Array([[0, -1], [1, 0]]), evaluate=False)
     assert expr.shape == (2, 2, 2, 2)
     assert expr.rank() == 4
 
