@@ -8,15 +8,35 @@ python.
 This module provides a common interface for different external backends, such
 as f2py, fwrap, Cython, SWIG(?) etc. (Currently only f2py and Cython are
 implemented) The goal is to provide access to compiled binaries of acceptable
-performance with a one-button user interface, i.e.
+performance with a one-button user interface, e.g.,
 
     >>> from sympy.abc import x,y
-    >>> expr = ((x - y)**(25)).expand()
-    >>> binary_callable = autowrap(expr)
-    >>> binary_callable(1, 2)
+    >>> expr = (x - y)**25
+    >>> flat = expr.expand()
+    >>> binary_callable = autowrap(flat)
+    >>> binary_callable(2, 3)
     -1.0
 
-The callable returned from autowrap() is a binary python function, not a
+Although a SymPy user might primarily be interested in working with
+mathematical expressions and not in the details of wrapping tools
+needed to evaluate such expressions efficiently in numerical form,
+the user cannot do so without some understanding of the
+limits in the target language. For example, the expanded expression
+contains large coefficients which result in loss of precision when
+computing the expression:
+
+    >>> binary_callable(3, 2)
+    0.0
+    >>> binary_callable(4, 5), binary_callable(5, 4)
+    (-22925376.0, 25165824.0)
+
+Wrapping the unexpanded expression gives the expected behavior:
+
+    >>> e = autowrap(expr)
+    >>> e(4, 5), e(5, 4)
+    (-1.0, 1.0)
+
+The callable returned from autowrap() is a binary Python function, not a
 SymPy object.  If it is desired to use the compiled function in symbolic
 expressions, it is better to use binary_function() which returns a SymPy
 Function object.  The binary callable is attached as the _imp_ attribute and
@@ -30,11 +50,6 @@ lambdify().
     >>> (2*f(x, y) + y).evalf(2, subs={x: 1, y:2})
     0.e-110
 
-The idea is that a SymPy user will primarily be interested in working with
-mathematical expressions, and should not have to learn details about wrapping
-tools in order to evaluate expressions numerically, even if they are
-computationally expensive.
-
 When is this useful?
 
     1) For computations on large arrays, Python iterations may be too slow,
@@ -45,7 +60,7 @@ When is this useful?
        compiled binary should be significantly faster than SymPy's .evalf()
 
     3) If you are generating code with the codegen utility in order to use
-       it in another project, the automatic python wrappers let you test the
+       it in another project, the automatic Python wrappers let you test the
        binaries immediately from within SymPy.
 
     4) To create customized ufuncs for use with numpy arrays.
@@ -61,7 +76,7 @@ When is this module NOT the best approach?
        tempdir="path/to/files/".
 
     2) If the array computation can be handled easily by numpy, and you
-       don't need the binaries for another project.
+       do not need the binaries for another project.
 
 """
 
@@ -74,7 +89,6 @@ from string import Template
 from warnings import warn
 
 from sympy.core.cache import cacheit
-from sympy.core.compatibility import iterable
 from sympy.core.function import Lambda
 from sympy.core.relational import Eq
 from sympy.core.symbol import Dummy, Symbol
@@ -83,6 +97,7 @@ from sympy.utilities.codegen import (make_routine, get_code_generator,
                                      OutputArgument, InOutArgument,
                                      InputArgument, CodeGenArgumentListError,
                                      Result, ResultBase, C99CodeGen)
+from sympy.utilities.iterables import iterable
 from sympy.utilities.lambdify import implemented_function
 from sympy.utilities.decorator import doctest_depends_on
 
@@ -219,12 +234,8 @@ class CythonCodeWrapper(CodeWrapper):
     """Wrapper that uses Cython"""
 
     setup_template = """\
-try:
-    from setuptools import setup
-    from setuptools import Extension
-except ImportError:
-    from distutils.core import setup
-    from distutils.extension import Extension
+from setuptools import setup
+from setuptools import Extension
 from Cython.Build import cythonize
 cy_opts = {cythonize_options}
 {np_import}
@@ -239,6 +250,8 @@ ext_mods = [Extension(
 setup(ext_modules=cythonize(ext_mods, **cy_opts))
 """
 
+    _cythonize_options = {'compiler_directives':{'language_level' : "3"}}
+
     pyx_imports = (
         "import numpy as np\n"
         "cimport numpy as np\n\n")
@@ -258,7 +271,7 @@ setup(ext_modules=cythonize(ext_mods, **cy_opts))
     def __init__(self, *args, **kwargs):
         """Instantiates a Cython code wrapper.
 
-        The following optional parameters get passed to ``distutils.Extension``
+        The following optional parameters get passed to ``setuptools.Extension``
         for building the Python extension module. Read its documentation to
         learn more.
 
@@ -294,7 +307,7 @@ setup(ext_modules=cythonize(ext_mods, **cy_opts))
         self._extra_compile_args = kwargs.pop('extra_compile_args', [])
         self._extra_compile_args.append(self.std_compile_flag)
         self._extra_link_args = kwargs.pop('extra_link_args', [])
-        self._cythonize_options = kwargs.pop('cythonize_options', {})
+        self._cythonize_options = kwargs.pop('cythonize_options', self._cythonize_options)
 
         self._need_numpy = False
 
@@ -341,7 +354,7 @@ setup(ext_modules=cythonize(ext_mods, **cy_opts))
         return getattr(mod, name + '_c')
 
     def dump_pyx(self, routines, f, prefix):
-        """Write a Cython file with python wrappers
+        """Write a Cython file with Python wrappers
 
         This file contains all the definitions of the routines in c code and
         refers to the header file.
@@ -532,7 +545,7 @@ def _validate_backend_language(backend, language):
 @doctest_depends_on(exe=('f2py', 'gfortran'), modules=('numpy',))
 def autowrap(expr, language=None, backend='f2py', tempdir=None, args=None,
              flags=None, verbose=False, helpers=None, code_gen=None, **kwargs):
-    """Generates python callable binaries based on the math expression.
+    """Generates Python callable binaries based on the math expression.
 
     Parameters
     ==========
@@ -648,7 +661,7 @@ def autowrap(expr, language=None, backend='f2py', tempdir=None, args=None,
 
 @doctest_depends_on(exe=('f2py', 'gfortran'), modules=('numpy',))
 def binary_function(symfunc, expr, **kwargs):
-    """Returns a sympy function with expr as binary implementation
+    """Returns a SymPy function with expr as binary implementation
 
     This is a convenience function that automates the steps needed to
     autowrap the SymPy expression and attaching it to a Function object
@@ -657,9 +670,9 @@ def binary_function(symfunc, expr, **kwargs):
     Parameters
     ==========
 
-    symfunc : sympy Function
+    symfunc : SymPy Function
         The function to bind the callable to.
-    expr : sympy Expression
+    expr : SymPy Expression
         The expression used to generate the function.
     kwargs : dict
         Any kwargs accepted by autowrap.
@@ -768,20 +781,17 @@ ufunc${ind} = PyUFunc_FromFuncAndData(${funcname}_funcs, ${funcname}_data, ${fun
     Py_DECREF(ufunc${ind});""")
 
 _ufunc_setup = Template("""\
-def configuration(parent_package='', top_path=None):
-    import numpy
-    from numpy.distutils.misc_util import Configuration
+from setuptools.extension import Extension
+from setuptools import setup
 
-    config = Configuration('',
-                           parent_package,
-                           top_path)
-    config.add_extension('${module}', sources=['${module}.c', '${filename}.c'])
-
-    return config
+from numpy import get_include
 
 if __name__ == "__main__":
-    from numpy.distutils.core import setup
-    setup(configuration=configuration)""")
+    setup(ext_modules=[
+        Extension('${module}',
+                  sources=['${module}.c', '${filename}.c'],
+                  include_dirs=[get_include()])])
+""")
 
 
 class UfuncifyCodeWrapper(CodeWrapper):
@@ -866,7 +876,7 @@ class UfuncifyCodeWrapper(CodeWrapper):
         f.write(setup)
 
     def dump_c(self, routines, f, prefix, funcname=None):
-        """Write a C file with python wrappers
+        """Write a C file with Python wrappers
 
         This file contains all the definitions of the routines in c code.
 
@@ -1033,7 +1043,7 @@ def ufuncify(args, expr, language=None, backend='numpy', tempdir=None,
     References
     ==========
 
-    .. [1] http://docs.scipy.org/doc/numpy/reference/ufuncs.html
+    .. [1] https://numpy.org/doc/stable/reference/ufuncs.html
 
     Examples
     ========
